/*Copyright 2007 Jonathan Irons

This file is part of JUICE.

JUICE is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

JUICE is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with JUICE; if not, write to the Free Software
Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

    package src.backend;

import java.util.ArrayList;
import src.gui.*;
import src.backend.*;

public abstract class Chunk {

    protected Level level;
    protected ChunkHeader header;
    protected boolean hasChanged = false;

    public Chunk(Level level, ChunkHeader header) {
	this.level = level;
	this.header = header;
	this.header.setChunk(this);
    }

    public Chunk() {
    }

    public boolean hasChanged() {
	return this.hasChanged;
    }

    public void change() {
	this.hasChanged = true;
	if (this.level != null)
	    this.level.change();
    }

    public void resetChange() {
	this.hasChanged = false;
    }

    public void setLevel(Level level) {
	change();
	this.level = level;
    }

    public void setHeader(ChunkHeader header) {
	change();
	this.header = header;
    }

    public Level getLevel() {
	return this.level;
    }

    public ChunkHeader getHeader() {
	return this.header;
    }

    public int getTag() {
	return this.header.getTag();
    }

    public int getNextOffset() {
	return header.getNextOffset();
    }

    public int getLength() {
	return header.getLength();
    }

    public int getOffset() {
	return header.getOffset();
    }

    public void write() {
	System.out.print(".");
	this.header.write();
    }

    public void read() {
    }

    public abstract int getEntryLength();

    public abstract int getNumEntries();
}
