/*Copyright 2007 Jonathan Irons

This file is part of JUICE.

JUICE is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

JUICE is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with JUICE; if not, write to the Free Software
Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

    //The new JUCE chunk is a one-line comment limited to 64 characters
    //that is printed by Aleph One at the beginning of a game.

    package src.backend;

import src.gui.*;
import src.backend.*;
import java.util.ArrayList;

public class JuceChunk extends Chunk{

    private ArrayList entries = new ArrayList(0);


    public JuceChunk() {
	super();
    }

    public JuceChunk(Level level, ChunkHeader header) {
	super(level, header);

	MapWriter mw = this.level.getMapData().getMapWriter();
	mw.seek(this.header.getOffset() + 16);
	int numEntries = this.header.getLength()/JuceEntry.ENTRY_LENGTH;
	for (int i=0; i<numEntries; i++) {
	    mw.seek(this.header.getOffset() + 16 + (i * JuceEntry.ENTRY_LENGTH));
	    byte[] data = new byte[JuceEntry.ENTRY_LENGTH];
	    mw.read(data);
	    JuceEntry juceEntry = new JuceEntry(this, data);
	    this.entries.add(juceEntry);
	}
    }

    public JuceChunk(String comment) {
	super();
	JuceEntry entry = new JuceEntry(this);
	this.entries.add(entry);
	setComment(comment);
    }

    public String getComment() {
	return ((JuceEntry)this.entries.get(0)).getComment();
    }

    public void setComment(String comment) {
	if (this.entries.get(0) == null) {
	    JuceEntry entry = new JuceEntry(this);
	    this.entries.add(entry);
	}
	((JuceEntry)this.entries.get(0)).setComment(comment);
    }

    public void write() {
	super.write();
	MapWriter mw = this.level.getMapData().getMapWriter();
	mw.seek(this.header.getOffset() + 16);
	System.out.println("Writing JUCE @ "+(this.header.getOffset() + 16));
	for (int i=0; i<this.entries.size(); i++) {
	    System.out.println("Writing comment:\n" + ((JuceEntry)this.entries.get(i)).getComment());
	    ((JuceEntry)this.entries.get(i)).write();
	}
    }

    public int getEntryLength() {
	return ((JuceEntry)this.entries.get(0)).ENTRY_LENGTH;
    }

    public int getNumEntries() {
	return this.entries.size();
    }

}
