/* TODO:
   Make stripSavedGame work
*/

/*Copyright 2007 Jonathan Irons

This file is part of JUICE.

JUICE is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

JUICE is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with JUICE; if not, write to the Free Software
Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

		      package src.backend;

import src.gui.*;
import src.backend.*;
import java.util.ArrayList;

public class Level {

    private static boolean READ_PATCH = true;
    private static String PATCH_PATH = "Shapes Patch";

    private static short SOLO = 0x1;
    private static short COOP = 0x2;
    private static short EMFH = 0x4;
    private static short BALL = 0x8;
    private static short KOTH = 0x10;
    private static short RUGBY = 0x20;
    private static short DEFENSE = 0x40;

    MapData md;
    ArrayList chunks;
    private boolean hasChanged = false;
    private PlacChunk placChunk;
    private MinfChunk minfChunk;
    private JuceChunk juceChunk;
    private PolyChunk polyChunk;
    private ObjsChunk objsChunk;
    private LinsChunk linsChunk;
    private SidsChunk sidsChunk;
    private ShPaChunk shpaChunk;
    private short flags;
    private int offset;
    private int length;
    private String levelSelectName;
    private boolean isSavedGame = false;
    private int chunksToStrip = 0;

    public Level(MapData md) {
	this.md = md;
    }

    public void processChunks() {
	MapWriter mw = this.md.getMapWriter();
	chunks = new ArrayList(0);

	mw.seek(this.offset);
	int numChunks = 0;
	while (mw.getFilePointer() < this.offset + this.length) {
	    long presentOffset = mw.getFilePointer();
	    //	    System.out.print(presentOffset);
	    String tag = "";
	    for (int z=0; z<4; z++) {
		tag += (char) mw.readByte();
	    }
	    //	    System.out.print(tag);
	    mw.seek(presentOffset);
	    ChunkHeader header = new ChunkHeader();
	    header.setTag(mw.readInt());
	    header.setTagString(tag);
	    header.setNextOffset(mw.readInt());
	    header.setLength(mw.readInt());
	    header.setOffset((int)(presentOffset));
	    mw.readInt();
	    System.out.print("\tChunk found: " + header.getTagString());
	    numChunks++;
	    int cTag = header.getTag();
	    switch (cTag) {
	    case StaticFunctions.plac_CHUNK: //"plac" tag
		this.placChunk = new PlacChunk(this, header);
		chunks.add(placChunk);
		System.out.print("\t[X]\n");
		break;
	    case StaticFunctions.Minf_CHUNK: //"Minf" tag
		this.minfChunk = new MinfChunk(this, header);
		this.chunks.add(this.minfChunk);
		System.out.print("\t[X]\n");
		break;
	    case StaticFunctions.JUCE_CHUNK:
		this.juceChunk = new JuceChunk(this, header);
		this.chunks.add(this.juceChunk);
		System.out.print("\t[X]\n");
		break;
	    case StaticFunctions.POLY_CHUNK:
		this.polyChunk = new PolyChunk(this, header);
		this.polyChunk.read();
		this.chunks.add(this.polyChunk);
		System.out.print("\t[X]\n");
		break;
	    case StaticFunctions.OBJS_CHUNK:
		this.objsChunk = new ObjsChunk(this, header);
		this.objsChunk.read();
		this.chunks.add(this.objsChunk);
		System.out.print("\t[X]\n");
		break;
	    case StaticFunctions.LINS_CHUNK:
		this.linsChunk = new LinsChunk(this, header);
		this.linsChunk.read();
		this.chunks.add(this.linsChunk);
		System.out.print("\t[X]\n");
		break;
	    case StaticFunctions.SIDS_CHUNK:
		this.sidsChunk = new SidsChunk(this, header);
		this.sidsChunk.read();
		this.chunks.add(this.sidsChunk);
		System.out.print("\t[X]\n");
		break;
	    case StaticFunctions.ShPa_CHUNK:
		this.shpaChunk = new ShPaChunk(this, header);
		this.shpaChunk.read();
		this.chunks.add(this.shpaChunk);
		System.out.print("\t[X]\n");
		break;
      	    default:
		System.out.print("\t[ ]\n");
		UnknownChunk chunk = new UnknownChunk(this, header);
		chunks.add(chunk);
		break;
	    }
	    if (header.getNextOffset() == 0) {
		break;
	    }
	    mw.seek(header.getNextOffset() + this.offset);
	}

	//check to see if level (and map) is optimized:
	for (int opt=0; opt<this.chunks.size(); opt++) {
	    Chunk chun = (Chunk) this.chunks.get(opt);
	    if (chun.getTag() == StaticFunctions.iidx_CHUNK)
		this.md.setOptimized(true);
	}

	System.out.println("Number of chunks: "+numChunks);

	/*	for (int z=0; z<this.chunks.size(); z++) {
	  boolean saveChunk = false;
	  Chunk chunk = this.chunks.get(z);
	  int cTag = chunk.getHeader().getTag();
	  for (int y=0; y<StaticFunctions.SAVE_CHUNKS.length; y++) {
	  if (cTag == StaticFunctions.SAVE_CHUNKS[y]) {
	  this.isSavedGame = true;
	  saveChunk = true;
	  System.out.println("Removing chunk "+chunk.getHeader().getTagString());
	  }
	  }
	  if (saveChunk) {
	  Chunk aChunk = this.chunks.get(z);
	  ChunkHeader aHeader = chunk.getHeader();
	  aHeader.setTag(0);
	  this.chunksToStrip++;
	  }
	  Chunk aChunk = this.chunks.get(z);
	  ChunkHeader aHeader = chunk.getHeader();
	  if (aHeader.getTag() == StaticFunctions.iidx_CHUNK) {
	  this.chunksToStrip++;
	  System.out.println(this.chunksToStrip);
	  }
	  }
	*/

	// convertLevelTextures(18, 11, 19, 25); //in loving memory of the most awesome moment in JUICE
    }

    /*
      public void stripSavedGame() {
      if (this.chunksToStrip > 0) {
      System.out.println("Detected chunks to strip from level.");
      for (int i=0; i<this.chunksToStrip; i++) {
      for (int j=0; j<this.chunks.size(); j++) {
      Chunk chunk = this.chunks.get(j);
      ChunkHeader header = chunk.getHeader();
      if (header.getTag() == 0 || header.getTag() == StaticFunctions.iidx_CHUNK) {
      removeChunk(j);
      break;
      }
      }
      }
      }
      }
    */

    public boolean hasChanged() {
	return this.hasChanged;
    }

    public void change() {
	this.hasChanged = true;
	this.md.change();
    }

    public void resetChange() {
	this.hasChanged = false;
	for (int i=0; i < this.chunks.size(); i++) {
	    ((Chunk)this.chunks.get(i)).resetChange();
	}
    }

    public MapData getMapData() {
	return this.md;
    }

    public short getFlags() {
	return this.flags;
    }

    public void setFlags(short flags) {
	this.flags = flags;
	change();
    }

    public int getOffset() {
	return this.offset;
    }

    public void setOffset(int offset) {
	this.offset = offset;
	change();
    }

    public int getLength() {
	return this.length;
    }

    public void updateLength(int length) {
	int diff = length - this.length;
	this.length = length;
	change();
	this.md.setMapLength(md.getMapLength() + diff);
    }

    public void setLength(int length) {
	this.length = length;
	change();
    }

    public String getlevelSelectName() {
	return this.levelSelectName;
    }

    public void setLevelSelectName(String name) {
	this.levelSelectName = name;
	change();
    }

    public ArrayList getChunks() {
	return this.chunks;
    }

    public void addChunk(Chunk chunk) {
	ChunkHeader header = new ChunkHeader();
	int size = chunk.getEntryLength() * chunk.getNumEntries();
	System.out.println(size);
	header.setLength(size);
	header.setNextOffset(0);
	Chunk lastChunk = (Chunk)this.chunks.get(this.chunks.size() - 1);
	ChunkHeader lastHeader = lastChunk.getHeader();
	header.setOffset(lastHeader.getOffset() + lastHeader.getLength() + 16);
	lastHeader.setNextOffset(header.getOffset() - this.offset);
	chunk.setHeader(header);
	header.setChunk(chunk);
	header.setTag(StaticFunctions.JUCE_CHUNK);
	chunk.setLevel(this);
	if (chunk instanceof JuceChunk)
	    this.juceChunk = (JuceChunk) chunk;
	else if (chunk instanceof ShPaChunk) {
	    header.setTag(StaticFunctions.ShPa_CHUNK);
	    this.shpaChunk = (ShPaChunk) chunk;
	}
	this.md.autoAdjust(16 + header.getLength(), header.getOffset()-1);//-1 is to make sure the > in Level.autoAdjust doesn't skip the level right after this chunk.
	this.chunks.add(chunk);
    }

    public Chunk removeChunk(int index) {
	Chunk deadChunk = (Chunk)this.chunks.remove(index);
	int adjustment = deadChunk.getHeader().getLength();
	adjustment += 16;
	this.md.autoAdjust(adjustment * -1, deadChunk.getHeader().getOffset());
	return deadChunk;
    }

    public PlacChunk getPlacChunk() {
	return this.placChunk;
    }

    public MinfChunk getMinfChunk() {
	return this.minfChunk;
    }

    public JuceChunk getJuceChunk() {
	return this.juceChunk;
    }

    public PolyChunk getPolyChunk() {
	return this.polyChunk;
    }

    public ObjsChunk getObjsChunk() {
	return this.objsChunk;
    }

    public LinsChunk getLinsChunk() {
        return this.linsChunk;
    }

    public SidsChunk getSidsChunk() {
	return this.sidsChunk;
    }

    public ShPaChunk getShPaChunk() {
	return this.shpaChunk;
    }

    public void convertLevelTextures() {
	this.polyChunk.convertTextures();
	this.sidsChunk.convertTextures();
    }

    /*
    //Code to make the landscape check hack less tedious...
    public short checkBmp(int bmp) {
    switch (bmp) {
    case -2:
    newcoll = 27;
    break;
    case -3:
    newcoll = 28;
    newbmp = 0;
    break;
    case -4:
    newcoll = 29;
    newbmp = 0;
    break;
    case -5:
    newcoll = 30;
    newbmp = 0;
    break;
    default:
    break;
    }
    }
    */

    public void convertLevelTextures(int coll1, int bmp1,
				     int coll2, int bmp2) {
	short newcoll = (short) coll2;
	short newbmp = (short) bmp2;
	switch (bmp2) {
	case -2:
	    newcoll = 27;
	    newbmp = 0;
	    break;
	case -3:
	    newcoll = 28;
	    newbmp = 0;
	    break;
	case -4:
	    newcoll = 29;
	    newbmp = 0;
	    break;
	case -5:
	    newcoll = 30;
	    newbmp = 0;
	    break;
	default:
	    break;
	}

	this.polyChunk.convertTextures((short) coll1, (short) bmp1,
				       newcoll, newbmp);
	this.sidsChunk.convertTextures((short) coll1, (short) bmp1,
				       newcoll, newbmp);
    }


    public void convertLevelTextures(int c1, int b1, int x1,
				     int c2, int b2, int x2) {
	short newcoll = (short) c2;
	short newbmp = (short) b2;
	switch (b2) {
	case -2:
	    newcoll = 27;
	    newbmp = 0;
	    break;
	case -3:
	    newcoll = 28;
	    newbmp = 0;
	    break;
	case -4:
	    newcoll = 29;
	    newbmp = 0;
	    break;
	case -5:
	    newcoll = 30;
	    newbmp = 0;
	    break;
	default:
	    break;
	}

	this.polyChunk.convertTextures((short) c1, (short) b1, (short) x1,
				       newcoll, newbmp,(short) x2);
	this.sidsChunk.convertTextures((short) c1, (short) b1, (short) x1,
				       newcoll, newbmp, (short) x2);
    }

    public void convertLevelTextures(int c1, int b1, int x1, int cp1,
				     int c2, int b2, int x2, int cp2) {
	short newcoll = (short) c2;
	short newbmp = (short) b2;
	switch (b2) {
	case -2:
	    newcoll = 27;
	    newbmp = 0;
	    break;
	case -3:
	    newcoll = 28;
	    newbmp = 0;
	    break;
	case -4:
	    newcoll = 29;
	    newbmp = 0;
	    break;
	case -5:
	    newcoll = 30;
	    newbmp = 0;
	    break;
	default:
	    break;
	}

	this.polyChunk.convertTextures((short) c1, (short) b1, (short) x1,
				       newcoll, newbmp,(short) x2);
	this.sidsChunk.convertTextures((short) c1, (short) b1,
				       (short) x1, (short) cp1,
				       newcoll, newbmp,
				       (short) x2, (short) cp2);
    }

    public void write() {
	for (int i=0; i < this.chunks.size(); i++) {
	    ((Chunk)this.chunks.get(i)).write();
	}
    }

    public void autoAdjust(int len, int off) {
	for (int i=0; i<this.chunks.size(); i++) {
	    Chunk chunk = (Chunk)this.chunks.get(i);
	    if (chunk.getHeader().getOffset() > off) {
		ChunkHeader header = chunk.getHeader();
		header.setOffset(header.getOffset() + len);
	    }
	}
	if (this.offset >= off) {
	    this.offset += len;
	    System.out.println("\t @ "+this.offset);
	} else if (this.offset + this.length >= off && this.offset < off) {
	    adjust(len, off);
	}
    }

    public void adjust(int len, int off) {
	System.out.println("\tAdjusting self length by "+len);
	setLength(getLength() + len);
	for (int i=0; i<this.chunks.size(); i++) {
	    Chunk chunk = (Chunk)chunks.get(i);
	    if (chunk.getHeader().getOffset() >= off) {
		ChunkHeader header = chunk.getHeader();
		if (header.getNextOffset() != 0)
		    header.setNextOffset(header.getNextOffset() + len);
	    }
	}
    }

    public boolean isSavedGame() {
	return this.isSavedGame;
    }
}
