/*Copyright 2007 Jonathan Irons

This file is part of JUICE.

JUICE is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

JUICE is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with JUICE; if not, write to the Free Software
Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/


    package src.backend;

import src.gui.*;
import src.backend.*;
import java.util.ArrayList;

public class LinsChunk extends Chunk {
    
    private ArrayList entries = new ArrayList(0);
    private int numEntries;
    
    public LinsChunk(Level level, ChunkHeader header) {
	super(level, header);
    }

    public LinsChunk() {
	super();
    }

    public LinsEntry getEntry(int entry) {
	if (entry < this.entries.size()) {
	    return (LinsEntry)this.entries.get(entry);
	} else {
	    System.out.println("ObjsChunk does not contain that many entries. Returning highest entry.");
	    return (LinsEntry)this.entries.get(this.entries.size() - 1);
	}
    }

    public void read() {
	super.read();
	MapWriter mw = this.level.getMapData().getMapWriter();
	this.numEntries = (int) this.header.getLength() / LinsEntry.ENTRY_LENGTH;
       	mw.seek(this.header.getOffset() + 16);
	for (int i=0; i<this.numEntries; i++) {
	    LinsEntry linsEntry = new LinsEntry(this);
	    linsEntry.read();
	    this.entries.add(linsEntry);
	}
    }

    public void write() {
	super.write();
	MapWriter mw = this.level.getMapData().getMapWriter();
	mw.seek(this.header.getOffset() + 16);
	for (int i=0; i<this.entries.size(); i++) {
	    ((LinsEntry)this.entries.get(i)).write();
	}
    }

    public int getNumEntries() {
	return this.entries.size();
    }

    public int getEntryLength() {
	return LinsEntry.ENTRY_LENGTH;
    }

    public LinsEntry addEntry() {
	LinsEntry o = new LinsEntry(this);
	int len = getEntryLength();
	this.header.autoAdjust(len);
	this.numEntries++;
	return o;
    }

    public void addEntry(LinsEntry oe) {
	int len = getEntryLength();
	this.entries.add(oe);
	this.header.autoAdjust(len);
	this.numEntries++;
    }

    public LinsEntry removeEntry(LinsEntry o) {
	for (int i=0; i<this.entries.size(); i++) {
	    if (o.equals(getEntry(i))) {
		this.entries.remove(i);
		int len = -1 * getEntryLength();
		this.header.autoAdjust(len);
		this.numEntries--;
		return o;
	    }
	}
	System.out.println("That LinsEntry does not exist in the LinsChunk and can't be removed.");
	return o;
    }

}
