/*Copyright 2007 Jonathan Irons

This file is part of JUICE.

JUICE is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

JUICE is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with JUICE; if not, write to the Free Software
Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/


    package src.backend;

import src.gui.*;
import src.backend.*;
import java.util.ArrayList;

public class LinsEntry extends Entry {

    public static final int ENTRY_LENGTH = 32;

    private short endpoint1, endpoint2;
    private short flags;
    private short length; //only used in optimized levels
    private short highest_adjacent_floor, lowest_adjacent_ceiling; //see length
    private short clockwise_poly_side_index, counterclockwise_poly_side_index;
    private short clockwise_poly_owner, counterclockwise_poly_owner;
    private short[] unused_bastards = new short[6]; //CHISELCHISELCHISEL

    public LinsEntry (LinsChunk chunk) {
	super(chunk);
    }

    public LinsEntry read() {
	//	System.out.println("\nReading LINS entry.");
	MapWriter mapWriter = this.chunk.getLevel().getMapData().getMapWriter();
	byte[] data = new byte[this.ENTRY_LENGTH];
	//System.out.println("Reading next "+this.ENTRY_LENGTH+" bytes of data.");
	mapWriter.read(data);
	WrkArrayReader wrk = new WrkArrayReader(data);
	this.endpoint1 = wrk.readShort();
	this.endpoint2 = wrk.readShort();
	this.flags = wrk.readShort();
	this.length = wrk.readShort();
	this.highest_adjacent_floor = wrk.readShort();
	this.lowest_adjacent_ceiling = wrk.readShort();
	this.clockwise_poly_side_index = wrk.readShort();
	this.counterclockwise_poly_side_index = wrk.readShort();
	this.clockwise_poly_owner = wrk.readShort();
	this.counterclockwise_poly_owner = wrk.readShort();
	for (int i=0; i<this.unused_bastards.length; i++) {
	    this.unused_bastards[i] = wrk.readShort();
	}
	/*
	System.out.println("LINS entry:"+
			   "\t EPNT 0: "+this.endpoint1+
			   "\t EPNT 1: "+this.endpoint1+
			   "\t Flags : "+this.flags+
			   "\t Length: "+this.length+
			   "\t H.A.F : "+this.highest_adjacent_floor+
			   "\t L.A.C : "+this.lowest_adjacent_ceiling+
			   "\t CWPSI : "+this.clockwise_poly_side_index+
			   "\t CCWPSI: "+this.counterclockwise_poly_side_index+
			   "\t CWPOI : "+this.clockwise_poly_owner+
			   "\t CCWPOI: "+this.counterclockwise_poly_owner
			   );
	*/
	return this;
    }

    public void write() {
	MapWriter mapWriter = this.chunk.getLevel().getMapData().getMapWriter();
	byte[] data = new byte[this.ENTRY_LENGTH];
	WrkArrayReader wrk = new WrkArrayReader(data);
	wrk.writeShort(this.endpoint1);
	wrk.writeShort(this.endpoint2);
	wrk.writeShort(this.flags);
	wrk.writeShort(this.length);
	wrk.writeShort(this.highest_adjacent_floor);
	wrk.writeShort(this.lowest_adjacent_ceiling);
	wrk.writeShort(this.clockwise_poly_side_index);
	wrk.writeShort(this.counterclockwise_poly_side_index);
	wrk.writeShort(this.clockwise_poly_owner);
	wrk.writeShort(this.counterclockwise_poly_owner);
	for (int i=0; i<unused_bastards.length; i++) {
	    wrk.writeShort(this.unused_bastards[i]);
	}
	mapWriter.write(data);
    }

    public short[] getEndpoints() {
	short[] epnt = new short[2];
	epnt[0] = this.endpoint1;
	epnt[1] = this.endpoint2;
	return epnt;
    }

    public short getFlags() {
	return this.flags;
    }

    public short getLength() {
	return this.length;
    }

    public short getHighestAdjacentFloor() {
	return this.highest_adjacent_floor;
    }

    public short getLowestAdjacentCeiling() {
	return this.lowest_adjacent_ceiling;
    }

    public short getClockwisePolySideIndex() {
	return this.clockwise_poly_side_index;
    }

    public short getCounterClockwisePolySideIndex() {
	return this.counterclockwise_poly_side_index;
    }

    public short getClockwisePolyOwner() {
	return this.clockwise_poly_owner;
    }

    public short getCounterClockwisePolyOwner() {
	return this.counterclockwise_poly_owner;
    }

    public void setEndpoints(short e1, short e2) {
	this.endpoint1 = e1;
	this.endpoint2 = e2;
    }

    public void setEndpoints(short[] e) {
	this.endpoint1 = e[0];
	this.endpoint2 = e[1];
    }

    public void setFlags(int flags) {
	System.out.println("Flags orig: "+this.flags);
	this.flags = (short)flags;
	System.out.println("Flags new : "+this.flags);
    }

    public void setFlags(short flags) {
	this.flags = flags;
	System.out.println("Flag set to "+flags);
    }

    public void setLength(short length) {
	this.length = length;
    }

    public void setHighestAdjacentFloor(short h) {
	this.highest_adjacent_floor = h;
    }

    public void setLowestAdjacentCeiling(short l) {
	this.lowest_adjacent_ceiling = l;
    }

    public void setClockwisePolySideIndex(short c) {
	this.clockwise_poly_side_index = c;
    }

    public void setCounterClockwisePolySideIndex(short c) {
	this.counterclockwise_poly_side_index = c;
    }

    public void setClockwisePolyOwner(short c) {
	this.clockwise_poly_owner = c;
    }

    public void setCounterClockwisePolyOwner(short c) {
	this.counterclockwise_poly_owner = c;
    }

}
