/*Copyright 2007 Jonathan Irons

This file is part of JUICE.

JUICE is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

JUICE is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with JUICE; if not, write to the Free Software
Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

    package src.backend;

import src.gui.*;
import src.backend.*;
import java.util.ArrayList;

public class MapData {

    private static boolean READ_PATCH = false;
    private static String PATCH_PATH = "Shapes Patch";

    private ArrayList levels;
    private MapWriter mw;
    private boolean isValidMapFile = true;
    private boolean isMergedMapFile = true;
    private short version;
    private short dataVersion;
    private char[] mapName = new char[64];
    private long checksum;
    private int mapLength;
    private short numLevels;
    private short appSpecificDirSize;
    private short entryHeaderSize;
    private short baseDirectoryEntrySize;
    private int parentChecksum;
    private long fileLength = 0;
    private boolean hasChanged = false;
    private boolean isOptimized = false;

    public MapData(String filePath) {
	System.out.println("Reading map ...");
	long startTime = System.currentTimeMillis();
	this.mw = new MapWriter(filePath);

	// Error checking
	if(!mw.getFileLoaded())
	    return;


	if(this.mw.isValidMapFile())
	    isValidMapFile = true;
	else{
	    isValidMapFile = false;
	    return;
	}
	if (this.mw.isMacBinary()) {
	    mw.seek(83);
	    this.fileLength = mw.readLong();
	} else {
	    this.fileLength = mw.length();
	}

	readHeader();

	this.levels = new ArrayList(0);

	for (int a=0; a < this.numLevels; a++) {
	    System.out.println("Level "+a);
	    Level level = new Level(this);
	    this.mw.seek(this.mapLength + (a * 84));
	    print(this.mapLength);
	    print("Seeking to "+ (this.mapLength + (a * 84)));
	    level.setOffset(this.mw.readInt());
	    level.setLength(this.mw.readInt());

	    long sum = level.getLength() + 128 + 10;
	    if (sum == this.fileLength)
		this.isMergedMapFile = false;

	    if (this.isMergedMapFile) {
		level = processMergedLevel(level);
	    } else {
		level = processUnmergedLevel(level);
	    }
	    print("\tName: " +level.getlevelSelectName() + "\n------------------");
	    level.resetChange();
	    this.levels.add(level);
	    System.out.println(this.levels.size()+" levels in arraylist");
	}

	/* The following is to enable the shapes patch test code*/
	//shapesPatches();

	/*
	if (true)
	    this.levels.get(0).stripSavedGame();
	*/
    	resetChange();
	System.out.println("Reading took "+(System.currentTimeMillis() - startTime)+" milliseconds.");
    }

    /*

    public void shapesPatches() {
	for (int i=0; i<getNumLevels(); i++){
	    Level l = getLevel(i);
	    if (l.getShPaChunk() == null && READ_PATCH) {
		System.out.println("Adding shapes patch: "+PATCH_PATH);
		ShPaChunk c = new ShPaChunk(PATCH_PATH);
		l.addChunk(c);
	    }
	}
    }

    */

    public void change() {
	this.hasChanged = true;
    }

    public void resetChange() {
	this.hasChanged = false;
    }

    public boolean hasChanged() {
	return this.hasChanged;
    }

    private Level processMergedLevel(Level level) {
	this.mw.skipBytes(8);
	level.setFlags(mw.readShort());
	String tmpName = "";
	byte present = this.mw.readByte();
	while (present != 0) {
	    char newChar = (char) present;
	    tmpName += newChar;
	    present = this.mw.readByte();
	}
	level.setLevelSelectName(tmpName);
	level.processChunks();
	return level;
    }

    private Level processUnmergedLevel(Level level) {
	level.processChunks();
	level.setLevelSelectName(level.getMinfChunk().getName());
	level.setFlags((short) (level.getMinfChunk().getEntry()));
	return level;
    }

    public boolean isMergedMapFile() {
	return this.isMergedMapFile;
    }

    public void setMapWriter(MapWriter mw) {
	this.mw = mw;
	change();
    }

    public void setNumLevels(short numLevels) {
	this.numLevels = numLevels;
	change();
    }

    public void setMapLength(int mapLength) {
	int diff = mapLength - this.mapLength;
	this.mapLength = mapLength;
	//	this.mw.adjust(diff);
	print("Map Length set to " + mapLength);
	change();
    }

    public void setLevels(ArrayList levels) {
	this.levels = levels;
	change();
    }

    public void setLevel(int num, Level level) {
	levels.set(num, level);
	change();
    }

    public MapWriter getMapWriter() {
	return this.mw;
    }

    public short getNumLevels() {
	return this.numLevels;
    }

    public int getMapLength() {
	return this.mapLength;
    }

    public ArrayList getLevels() {
	return this.levels;
    }

    public Level getLevel(int num) {
	return (Level)levels.get(num);
    }

    public boolean getIsValidMap(){
	return isValidMapFile;
    }

    public boolean isOptimized() {
	return this.isOptimized;
    }

    public void setOptimized(boolean b) {
	this.isOptimized = b;
    }

    public static void print(String msg) {
	System.out.println(msg);
    }

    public static void print(long msg) {
	System.out.println(msg);
    }

    public void write() {
	System.out.println("Saving map...");
	long startTime = System.currentTimeMillis();
	this.mw.setFileSize(this);
	this.mw.zeroMap();
	if (this.mw.isMacBinary())
	    this.mw.writeMacBinary();
	writeHeader();
	for (int i=0; i < this.numLevels; i++) {
	    System.out.print("\n\tWriting level "+i);
	    getLevel(i).write();
	    getLevel(i).resetChange();
	}
	writeTrailers();
	writeChecksum();
	resetChange();
	System.out.println("Writing took "+(System.currentTimeMillis() - startTime)+" milliseconds.");
    }

    public void writeFlags(int level) {
	short flags = ((Level)levels.get(level)).getFlags();
	mw.seek(this.mapLength + (84 * level) + 16);
	mw.writeShort(flags);
    }

    public void writeTrailers() {
	for (int i=0; i<this.numLevels; i++) {
	    writeOffset(i);
	    writeLength(i);
	    if (this.isMergedMapFile) {
		writeIndex(i);
		writeFlags(i);
		writeLevelSelectName(i);
	    } else {
		this.mw.writeShort((short) 0);
	    }
	}
    }

    public void writeIndex(int level) {
	mw.seek(this.mapLength + (84 * level) + 8);
	mw.writeShort((short) level);
    }

    public void writeLevelSelectName(int level) {
	String levelSelectName = ((Level)levels.get(level)).getlevelSelectName();
	if (levelSelectName.length() > 64) {
	    levelSelectName = levelSelectName.substring(0, 64); //max name length is 64 characters
	}
	mw.seek(this.mapLength + (84 * level) + 18);
	for (int i=0; i < 64; i++) {
	    if (i < levelSelectName.length()) {
		mw.writeByte(levelSelectName.charAt(i));
	    } else {
		mw.writeByte(0);
	    }
	}
    }

    public void writeOffset(int level) {
	int offset = ((Level)this.levels.get(level)).getOffset();
	this.mw.seek(this.mapLength + (84 * level));
	this.mw.writeInt(offset);
    }

    public void writeLength(int level) {
	int length = ((Level)this.levels.get(level)).getLength();
	this.mw.seek(this.mapLength + (84 * level) + 4);
	this.mw.writeInt(length);
    }

    //adjust all map offsets after off to be + len
    public void autoAdjust(int len, int off) {
	System.out.println("Autoadjusting map.");
	for (int i=0; i<this.levels.size(); i++) {
	    System.out.println("Level "+i);
	    Level level = (Level)this.levels.get(i);
	    System.out.println("Level "+i+" autoAdjust:");
	    level.autoAdjust(len, off);
	}
	adjust(len);
    }

    public void adjust(int len) {
	setMapLength(getMapLength() + len);
	//	this.mw.adjust(len);
    }

    public void readHeader() {
	this.mw.seek(0);
	this.version = this.mw.readShort();
	print(this.version);
	this.dataVersion = this.mw.readShort();
	print(this.dataVersion + "\n");
	for (int z=0; z<64; z++) {
	    this.mapName[z] = (char) this.mw.readByte();
	    if (this.mapName[z] != 0)
		System.out.print(this.mapName[z]);
	}
	this.checksum = (long) this.mw.readInt();
	print("\n"+this.checksum);
	this.mapLength = this.mw.readInt();
	print(this.mapLength);
	this.numLevels = this.mw.readShort();
	print(this.numLevels + " level(s) found in map.");
	this.appSpecificDirSize = this.mw.readShort();
	this.entryHeaderSize = this.mw.readShort();
	this.parentChecksum = this.mw.readInt();
    }

    public void writeHeader() {
	this.mw.seek(0);
	this.mw.writeShort(this.version);
	this.mw.writeShort(this.dataVersion);
	for (int z=0; z<64; z++) {
	    this.mw.writeByte((byte) this.mapName[z]);
	}
	System.out.println("Original Checksum: "+((int) this.checksum));
	this.mw.writeInt(0);
	this.mw.writeInt(this.mapLength);
	print(this.mapLength);
	this.mw.writeShort(this.numLevels);
	this.mw.writeShort(this.appSpecificDirSize);
	this.mw.writeShort(this.entryHeaderSize);
	this.mw.writeInt(this.parentChecksum);
	for (int i=0; i<20; i++) {
	    this.mw.writeShort((short) 0);
	}
    }

    public void writeNew(String path) {
	this.mw.newRF(path, this); //makes mapwriter use a temporary file @ path
	write();
	//	this.mw.restoreRF(); //sets mapwriter back to the original file
    }

    public void writeChecksum() {
	int trailerLength = this.numLevels * 84;
	if (!this.isMergedMapFile)
	    trailerLength = 10;
	this.checksum = mw.generateChecksum(this.mapLength, trailerLength);
	System.out.println("\nNew Checksum     : "+((int) this.checksum));
	this.mw.seek(0x44);
	this.mw.writeInt((int) this.checksum);
    }
}
