/*Copyright 2007 Jonathan Irons

This file is part of JUICE.

JUICE is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

JUICE is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with JUICE; if not, write to the Free Software
Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

    package src.backend;

import src.gui.*;
import src.backend.*;

public class MinfChunk extends Chunk{

    //mission flags
    private static short MISSION_NONE = 0x0;
    private static short MISSION_EXTERMINATION = 0x1;
    private static short MISSION_EXPLORATION = 0x2;
    private static short MISSION_RETRIEVAL = 0x4;
    private static short MISSION_REPAIR = 0x8;
    private static short MISSION_RESCUE = 0x10;

    //environment flags
    private static short ENVIRONMENT_NORMAL = 0x0;
    private static short ENVIRONMENT_VACUUM = 0x1;
    private static short ENVIRONMENT_MAGNETIC = 0x2;
    private static short ENVIRONMENT_REBELLION = 0x4;
    private static short ENVIRONMENT_LOW_GRAVITY = 0x8;
    private static short ENVIRONMENT_NETWORK = 0x10;
    private static short ENVIRONMENT_SINGLE_PLAYER = 0x20;

    private short environment_code;
    private short physics;
    private short landscape;
    private short mission;
    private short environment_flags;
    private String name;
    private int entry; //entry point flags, just like in map trailer: redundant except when map is not merged

    public MinfChunk(Level level, ChunkHeader header) {
	super(level, header);
	MapWriter mw = this.level.getMapData().getMapWriter();
	mw.seek(this.header.getOffset() + 16);
	this.environment_code = mw.readShort();
	this.physics = mw.readShort();
	this.landscape = mw.readShort();
	this.mission = mw.readShort();
	this.environment_flags = mw.readShort();
	mw.skipBytes(8);
	String tmpName = "";
	byte present = mw.readByte();
	while (present != 0) {
	    char newChar = (char) present;
	    tmpName += newChar;
	    present = mw.readByte();
	}
	this.name = tmpName;
	mw.seek(this.header.getOffset() + 100);
	this.entry = mw.readInt();
    }

    public short getEnvironmentCode() {
	return this.environment_code;
    }

    public short getPhysics() {
	return this.physics;
    }

    public short getLandscape() {
	return this.landscape;
    }

    public short getMission() {
	return this.mission;
    }

    public short getEnvironmentFlags() {
	return this.environment_flags;
    }

    public String getName() {
	return this.name;
    }

    public int getEntry() {
	return this.entry;
    }

    public void setEnvironmentCode(short code) {
	this.environment_code = code;
	change();
    }

    public void setPhysics(short phys) {
	this.physics = phys;
	change();
    }

    public void setLandscape(short landscape) {
	//	this.landscape = landscape;
	change();
    }

    public void setMission(short mission) {
	this.mission = mission;
	change();
    }

    public void setEnvironmentFlags(short flags) {
	this.environment_flags = flags;
	change();
    }

    public void setName(String name) {
	this.name = name;
	change();
    }

    public void setEntry(int entry) {
	this.entry = entry;
	change();
    }

    public void write() {
	super.write();
	MapWriter mw = this.level.getMapData().getMapWriter();
       	mw.seek(this.header.getOffset() + 16);
	mw.writeShort(this.environment_code);
	mw.writeShort(this.physics);
	mw.writeShort(this.landscape);
	mw.writeShort(this.mission);
	mw.writeShort(this.environment_flags);
	mw.skipBytes(8);
	for (int i=0; i < 66; i++) {
	    if (i < this.name.length()) {
		mw.writeByte(this.name.charAt(i));
	    } else {
		mw.writeByte(0);
	    }
	}
	mw.writeInt(this.entry);
    }

    public int getNumEntries() {
	return 1;
    }

    public int getEntryLength() {
	return 1536;
    }
}
