/*Copyright 2007 Jonathan Irons

This file is part of JUICE.

JUICE is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

JUICE is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with JUICE; if not, write to the Free Software
Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/


    package src.backend;

import src.gui.*;
import src.backend.*;
import java.util.ArrayList;

public class ObjsChunk extends Chunk {

    private ArrayList entries = new ArrayList(0);
    private int numEntries;

    public ObjsChunk(Level level, ChunkHeader header) {
	super(level, header);
    }

    public ObjsChunk() {
	super();
    }

    public ObjsEntry getEntry(int entry) {
	if (entry < this.entries.size()) {
	    return (ObjsEntry)this.entries.get(entry);
	} else {
	    System.out.println("ObjsChunk does not contain that many entries. Returning highest entry.");
	    return (ObjsEntry)this.entries.get(this.entries.size() - 1);
	}
    }

    public void read() {
	super.read();
	MapWriter mw = this.level.getMapData().getMapWriter();
	this.numEntries = (int) this.header.getLength() / ObjsEntry.ENTRY_LENGTH;
       	mw.seek(this.header.getOffset() + 16);
	for (int i=0; i<this.numEntries; i++) {
	    ObjsEntry objsEntry = new ObjsEntry(this);
	    objsEntry.read();
	    this.entries.add(objsEntry);
	}
    }

    public void write() {
	super.write();
	MapWriter mw = this.level.getMapData().getMapWriter();
	mw.seek(this.header.getOffset() + 16);
	for (int i=0; i<this.entries.size(); i++) {
	    ((ObjsEntry)this.entries.get(i)).write();
	}
    }

    public int getNumEntries() {
	return this.entries.size();
    }

    public int getEntryLength() {
	return ObjsEntry.ENTRY_LENGTH;
    }

    public ObjsEntry addEntry() {
	ObjsEntry o = new ObjsEntry(this);
	o.setType((short) 1);
	o.setSubtype((short) 0);
	o.setFacing((short) 0);
	o.setPolygon((short) 0);
	PolyEntry p = this.level.getPolyChunk().getEntry(0);
	double x, y, z;
	x = p.getX();
	y = p.getY();
	z = 0.0;
	o.setX(x);
	o.setY(y);
	o.setZ(z);
	o.setFlags((short) 0);
	this.entries.add(o);
	int len = 16;
	this.header.autoAdjust(len);
	this.numEntries++;
	o.changePlacData(1);
	return o;
    }

    public void addEntry(ObjsEntry oe) {
	int len = 16;
	this.entries.add(oe);
	this.header.autoAdjust(len);
	oe.changePlacData(1);
	this.numEntries++;
    }

    public ObjsEntry removeEntry(ObjsEntry o) {
	for (int i=0; i<this.entries.size(); i++) {
	    if (o.equals(getEntry(i))) {
		this.entries.remove(i);
		int len = -16;
		this.header.autoAdjust(len);
		this.numEntries--;
		o.changePlacData(-1);
		return o;
	    }
	}
	System.out.println("That ObjsEntry does not exist in the ObjsChunk and can't be removed.");
	return o;
    }
}
