/*Copyright 2007 Jonathan Irons

This file is part of JUICE.

JUICE is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

JUICE is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with JUICE; if not, write to the Free Software
Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

    package src.backend;

import src.gui.*;
import src.backend.*;

public class ObjsEntry extends Entry {

    public static final int ENTRY_LENGTH = 16;

    private short type;
    private short subtype;
    private short facing;
    private short polygon;
    private short x, y, z;
    private short flags;

    public ObjsEntry(ObjsChunk chunk) {
	super(chunk);
    }

    public ObjsEntry read() {
	MapWriter mapWriter = this.chunk.getLevel().getMapData().getMapWriter();
	byte[] data = new byte[this.ENTRY_LENGTH];
	mapWriter.read(data);
	WrkArrayReader wrk = new WrkArrayReader(data);
	this.type = wrk.readShort();
	this.subtype = wrk.readShort();
	this.facing = wrk.readShort();
	this.polygon = wrk.readShort();
	this.x = wrk.readShort();
	this.y = wrk.readShort();
	this.z = wrk.readShort();
	this.flags = wrk.readShort();
	/*	System.out.println("\t"+StaticFunctions.OBJS_TYPES[this.type]+"\n"+
			   "\t"+StaticFunctions.OBJS_SUBTYPES[this.type][this.subtype]+"\n"+
			   "\t"+this.facing+"\n"+
			   "\t"+this.polygon+"\n"+
			   "\t"+this.x+"\n"+
			   "\t"+this.y+"\n"+
			   "\t"+this.z+"\n"+
			   "\t"+this.flags+"\n");
	*/
	return this;
    }

    public void write() {
	MapWriter mapWriter = this.chunk.getLevel().getMapData().getMapWriter();
	byte[] data = new byte[this.ENTRY_LENGTH];
	WrkArrayReader wrk = new WrkArrayReader(data);
	wrk.writeShort(this.type);
	wrk.writeShort(this.subtype);
	wrk.writeShort(this.facing);
	wrk.writeShort(this.polygon);
	wrk.writeShort(this.x);
	wrk.writeShort(this.y);
	wrk.writeShort(this.z);
	wrk.writeShort(this.flags);
	mapWriter.write(data);
    }

    public short getType() {
	return this.type;
    }

    public short getSubtype() {
	return this.subtype;
    }

    public short getFacing() {
	return (short) ((this.facing/512.0) * 360);
    }

    public short getPolygon() {
	return this.polygon;
    }

    public double getX() {
	return this.x/1024.0;
    }

    public double getY() {
	return this.y/1024.0;
    }

    public double getZ() {
	return this.z/1024.0;
    }

    public short getFlags() {
	return this.flags;
    }

    public void setType(short t) {
	short old = getType();
	short oldSub = getSubtype();
	this.type = t;
	if (old != this.type) {
	    if (old == 2 || old == 0)
		changePlacData(old, oldSub, -1);
	    if (this.type == 2 || this.type == 0)
		changePlacData(this.type, this.subtype, 1);
	}
    }

    public void setSubtype(short s) {
	short old = getType();
	short oldSub = getSubtype();
	this.subtype = s;
	if (oldSub != this.subtype) {
	    changePlacData(old, oldSub, -1);
	    changePlacData(this.type, this.subtype, 1);
	}
    }

    public void setFacing(short f) {
	short face = (short) ((f/360.0) * 512);
	this.facing = face;
    }

    public void setPolygon(short p) {
	this.polygon = p;
    }

    public void setX(double x) {
	this.x = (short) (x * 1024);
    }

    public void setY(double y) {
	this.y = (short) (y * 1024);
    }

    public void setZ(double z) {
	this.z = (short) (z  * 1024);
    }

    public void setFlags(short f) {
	this.flags = f;
    }

    private void changePlacData(short type, short subtype, int delta) {

	if (type == 0 || type == 2) {
	    int begin = 0;
	    int off = 0;
	    if (type == 0) {
		begin = PlacChunk.ITEM_END;
		off = 1;
	    }
	    PlacEntry plac = this.chunk.getLevel().getPlacChunk().getEntry(begin + subtype + off);
	    if (plac.getInitial() > 0 || delta > 0)
		plac.setInitial((short) (plac.getInitial() + delta));
	}
    }

    public void changePlacData(int delta) {
	if (this.type == 0 || this.type == 2) {
	    int begin = 0;
	    int off = 0;
	    if (this.type == 0) {
		begin = PlacChunk.ITEM_END;
		off = 1;
	    }
	    PlacEntry plac = this.chunk.getLevel().getPlacChunk().getEntry(begin + this.subtype + off);
	    if (plac.getInitial() > 0 || delta > 0)
		plac.setInitial((short) (plac.getInitial() + delta));
	}
    }
}
