/*Copyright 2007 Jonathan Irons

This file is part of JUICE.

JUICE is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

JUICE is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with JUICE; if not, write to the Free Software
Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

    package src.backend;

import java.util.ArrayList;
import src.gui.*;
import src.backend.*;

public class PlacChunk extends Chunk {

    private ArrayList placEntries;
    public final static byte ITEM_END = 36; //existing item types end here
    public final static byte MONSTER_BEGIN = 64; //monsters start here
    public final static byte MONSTER_END = 111; //existing monster types end here
    private static int DIFFERENCE = MONSTER_BEGIN - ITEM_END; //difference between ITEM_END and MONSTER_BEGIN
    private static int PLAC_END = 128;

    public PlacChunk(Level level, ChunkHeader header) {
	super(level, header);
	this.placEntries = new ArrayList(0);
	placEntries = getItemPlacement(placEntries);
	placEntries = getMonsterPlacement(placEntries);
    }

    public void resetChange() {
	super.resetChange();
	for (int i=0; i<this.placEntries.size(); i++) {
	    this.getEntry(i).resetChange();
	}
    }


    public PlacEntry getEntry(int i) {
	return (PlacEntry)this.placEntries.get(i);
    }

    public ArrayList getItemPlacement(ArrayList entries) {
	MapWriter mw = this.getLevel().getMapData().getMapWriter();
	long dataStart = this.header.getOffset() + 16;
	for (int i=0; i<ITEM_END; i++) {
	    mw.seek(dataStart + (i * 12));
	    PlacEntry placEntry = new PlacEntry(this);
	    placEntry.setFlags(mw.readShort());
	    placEntry.setInitial(mw.readShort());
	    placEntry.setMinimum(mw.readShort());
	    placEntry.setMaximum(mw.readShort());
	    placEntry.setRandomCount(mw.readUnsignedShort());
	    placEntry.setRandomChance(mw.readUnsignedShort());
	    placEntries.add(placEntry);

	    /*
	    System.out.print("\n\tPlacement for "+StaticFunctions.getPlacName(i)+
			     "\n\t\tFlags:   "+placEntry.getFlags()+
			     "\n\t\tInitial: "+placEntry.getInitial()+
			     "\n\t\tMinimum: "+placEntry.getMinimum()+
			     "\n\t\tMaximum: "+placEntry.getMaximum()+
			     "\n\t\tRandom:  "+placEntry.getRandomCount()+
			     "\n\t\tChance:  "+placEntry.getRandomChance()+ "("+ placEntry.getRandomChancePercent() + "%)");
	    */
	}
	return entries;
    }

    public ArrayList getMonsterPlacement(ArrayList entries) {
	MapWriter mw = this.getLevel().getMapData().getMapWriter();
	long dataStart = this.header.getOffset() + 16;
	for (int i=MONSTER_BEGIN; i<MONSTER_END; i++) {
	    mw.seek(dataStart + (i * 12));
	    PlacEntry placEntry = new PlacEntry(this);
	    placEntry.setFlags(mw.readShort());
	    placEntry.setInitial(mw.readShort());
	    placEntry.setMinimum(mw.readShort());
	    placEntry.setMaximum(mw.readShort());
	    placEntry.setRandomCount(mw.readUnsignedShort());
	    placEntry.setRandomChance(mw.readUnsignedShort());
	    placEntries.add(placEntry);

	    /* Print info about these things
	    System.out.print("\n\tPlacement for "+i+" "+StaticFunctions.getPlacName(i-DIFFERENCE)+
			     "\n\t\tFlags:   "+placEntry.getFlags()+
			     "\n\t\tInitial: "+placEntry.getInitial()+
			     "\n\t\tMinimum: "+placEntry.getMinimum()+
			     "\n\t\tMaximum: "+placEntry.getMaximum()+
			     "\n\t\tRandom:  "+placEntry.getRandomCount()+
			     "\n\t\tChance:  "+placEntry.getRandomChance()+
			     "\n\t Entry "+(entries.size()-1));
	    */
	}
	return entries;
    }

    public void writeItemPlacement() {
	long dataStart = this.header.getOffset() + 16;
	MapWriter mw = this.getLevel().getMapData().getMapWriter();
	for (int i=0; i<ITEM_END; i++) {
	    mw.seek(dataStart + (i * 12));
	    this.getEntry(i).write();
	}
    }

    public void writePadding() {
	int dataStart = this.header.getOffset() + 16;
	byte[] blank_entry = new byte[12];
	MapWriter mw = this.level.getMapData().getMapWriter();
	for (int i=ITEM_END; i<MONSTER_BEGIN; i++) {
	    mw.seek(dataStart + (i * 12));
	    mw.write(blank_entry);
	}
    }


    public void writeMonsterPlacement() {
	long dataStart = this.header.getOffset() + 16;
	MapWriter mw = this.getLevel().getMapData().getMapWriter();
	for (int i=MONSTER_BEGIN; i<MONSTER_END; i++) {
	    mw.seek(dataStart + (i * 12));
	    this.getEntry(i-DIFFERENCE).write();
	}
    }

    public void writeCrapAtEnd() {
	int dataStart = this.header.getOffset() + 16;
	MapWriter mw = this.getLevel().getMapData().getMapWriter();
	byte[] blank = new byte[12];
	for (int i=MONSTER_END; i<PLAC_END; i++) {
	    mw.seek(dataStart + (i * 12));
	    mw.write(blank);
	}
    }

    public void write() {
	super.write();
       	writeItemPlacement();
	writePadding();
	writeMonsterPlacement();
	writeCrapAtEnd();
    }

    public int getNumEntries() {
	return this.placEntries.size();
    }

    public int getEntryLength() {
	return 7;
    }
}
