/*Copyright 2007 Jonathan Irons

This file is part of JUICE.

JUICE is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

JUICE is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with JUICE; if not, write to the Free Software
Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

    package src.backend;

import src.gui.*;
import src.backend.*;

public class PlacEntry extends Entry {

    private short flags;
    private short initial;
    private short minimum;
    private short maximum;
    private int random_count;
    private int random_chance;
    private final static int MAX_CHANCE = 0xffdc;

    public PlacEntry(PlacChunk chunk) {
	super(chunk);
    }

    public void setFlags(short flags) {
	this.flags = flags;
	change();
    }

    public void setInitial(short initial) {
	this.initial = initial;
	change();
    }

    public void setMinimum(short minimum) {
	this.minimum = minimum;
	change();
    }

    public void setMaximum(short maximum) {
	this.maximum = maximum;
	change();
    }

    public void setRandomCount(int count) {
	this.random_count = count;
	change();
    }

    public void setRandomChance(int chance) {
	this.random_chance = chance;
	change();
    }

    public short getFlags() {
	return this.flags;
    }

    public short getInitial() {
	return this.initial;
    }

    public short getMinimum() {
	return this.minimum;
    }

    public short getMaximum() {
	return this.maximum;
    }

    public int getRandomCount() {
	return this.random_count;
    }

    public int getRandomChance() {
	return this.random_chance;
    }

    public int getRandomChancePercent() {
	int percent = (int)(100 * this.random_chance)/MAX_CHANCE;
	return percent;
    }

    public void setRandomChancePercent(int percent) {
	int chance = (percent * MAX_CHANCE)/100;
	this.random_chance = chance;
	change();
    }

    public void write() {
	super.write();
	MapWriter mw = this.getChunk().getLevel().getMapData().getMapWriter();
	mw.writeShort(this.flags);
	mw.writeShort(this.initial);
	mw.writeShort(this.minimum);
	mw.writeShort(this.maximum);
	mw.writeShort((short)this.random_count);
	mw.writeShort((short)this.random_chance);
    }
}
