/*Copyright 2007 Jonathan Irons

This file is part of JUICE.

JUICE is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

JUICE is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with JUICE; if not, write to the Free Software
Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/


    package src.backend;

import src.gui.*;
import src.backend.*;

public class PolyChunk extends Chunk {

    private PolyEntry[] entries;
    private int numEntries;

    public PolyChunk(Level level, ChunkHeader header) {
	super(level, header);
    }

    public PolyChunk() {
	super();
    }

    public PolyEntry getEntry(int entry) {
	if (entry < this.entries.length) {
	    return this.entries[entry];
	} else {
	    System.out.println("PolyChunk does not contain that many entries. Returning highest entry.");
	    return this.entries[this.entries.length - 1];
	}
    }

    public void read() {
	super.read();
	MapWriter mw = this.level.getMapData().getMapWriter();
	this.numEntries = (int) this.header.getLength() / PolyEntry.ENTRY_LENGTH;
	this.entries = new PolyEntry[this.numEntries];
	mw.seek(this.header.getOffset() + 16);
	for (int i=0; i<this.entries.length; i++) {
	    PolyEntry polyEntry = new PolyEntry(this);
	    this.entries[i] = polyEntry.read();
	}
    }

    public void write() {
	super.write();
	MapWriter mw = this.level.getMapData().getMapWriter();
	mw.seek(this.header.getOffset() + 16);
	for (int i=0; i<this.entries.length; i++) {
	    this.entries[i].write();
	}
    }

    public int getNumEntries() {
	return this.entries.length;
    }

    public int getEntryLength() {
	return PolyEntry.ENTRY_LENGTH;
    }

    public void convertTextures() {
	for (int i=0; i<this.entries.length; i++) {
	    PolyEntry pe = this.entries[i];
	    ShapeDescriptor[] sd = {pe.getFloorTexture(), pe.getCeilingTexture()};
	    for(int j=0; j<sd.length; j++) {
		for (int k=1; k<StaticFunctions.TEXTURE_CUTOFFS.length; k++) {
		    if (sd[j].getColl() == StaticFunctions.WATER_COLLECTION &&
			sd[j].getBmp() >= StaticFunctions.TEXTURE_CUTOFFS[k-1] &&
			sd[j].getBmp() < StaticFunctions.TEXTURE_CUTOFFS[k]) {

			sd[j].setColl(StaticFunctions.WATER_COLLECTION + k);
			short diff = (short) (sd[j].getBmp() - StaticFunctions.TEXTURE_CUTOFFS[k-1]);
			sd[j].setBmp(diff);
			sd[j].setColl(k);
		    }
		}
	    }
	}
    }

    public void convertTextures(short coll1, short bmp1,
				short coll2, short bmp2) {

	for (int i=0; i<this.entries.length; i++) {
	    PolyEntry pe = this.entries[i];
	    ShapeDescriptor[] sd = {pe.getFloorTexture(),
				    pe.getCeilingTexture()};

	    for(int j=0; j<sd.length; j++) {
		if (sd[j].getColl() == coll1 && sd[j].getBmp() == bmp1) {
		    sd[j].setColl(coll2);
		    sd[j].setBmp(bmp2);

		    if (coll2 >= 27 && coll2 <= 30) {
			if (j == 0)
			    pe.setFloorTransferMode((short) 9);
			else
			    pe.setCeilingTransferMode((short) 9);
		    }
		}
	    }
	}
    }

    public void convertTextures(short coll1, short bmp1, short x1,
				short coll2, short bmp2, short x2) {

	for (int i=0; i<this.entries.length; i++) {
	    PolyEntry pe = this.entries[i];
	    ShapeDescriptor[] sd = {pe.getFloorTexture(),
				    pe.getCeilingTexture()};

	    for(int j=0; j<sd.length; j++) {
		if (sd[j].getColl() == coll1 && sd[j].getBmp() == bmp1) {
		    if (j == 0 && pe.getFloorTransferMode() == x1) {
			pe.setFloorTransferMode(x2);
			sd[j].setColl(coll2);
			sd[j].setBmp(bmp2);
		    }
		    if (j == 1 && pe.getCeilingTransferMode() == x1) {
			pe.setCeilingTransferMode(x2);
			sd[j].setColl(coll2);
			sd[j].setBmp(bmp2);
		    }
		}
	    }
	}
    }
}
