/*Copyright 2007 Jonathan Irons

This file is part of JUICE.

JUICE is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

JUICE is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with JUICE; if not, write to the Free Software
Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

    package src.backend;

import src.gui.*;
import src.backend.*;

public class PolyEntry extends Entry {

    public static final int ENTRY_LENGTH = 128;

    short type; //2
    int flags; //4
    short permutation; //6
    int vertex_count; //normally uint16
    short[] endpoint_indexes = new short[8]; //26
    short[] line_indexes = new short[8]; //42
    ShapeDescriptor floor_texture, ceiling_texture; //normally uint16;
    short floor_height, ceiling_height; //44. 46
    short floor_light_index, ceiling_light_index; //48, 50
    int area; //54
    short first_object; //56
    short first_exclusion_zone_index, line_exclusion_count, point_exclusion_count; //58, 60, 62
    short floor_transfer_mode, ceiling_transfer_mode; //64, 66
    short[] adjacent_polygon_indexes = new short[8]; //82
    short first_neighbor_index; //84
    short neighbor_count; //86
    short center_x, center_y; //88, 90
    short[] side_indexes = new short[8]; //106
    short floor_x, floor_y, ceiling_x, ceiling_y; //108, 110, 112, 114
    short media_index; //116
    short media_light_index; //118
    short sound_source_index; //120
    short ambient_sound_index, random_sound_index; //122, 124
    short unused;
    //Poly types
    public static final int NORMAL = 0;
    public static final int ITEM_IMPASSABLE = 1;
    public static final int MONSTER_IMPASSABLE = 2;
    public static final int HILL = 3;
    public static final int BASE = 4;
    public static final int PLATFORM = 5;
    public static final int LIGHT_ON_TRIGGER= 6;
    public static final int PLATFORM_ON_TRIGGER = 7;
    public static final int LIGHT_OFF_TRIGGER = 8;
    public static final int PLATFORM_OFF_TRIGGER = 9;
    public static final int TELEPORTER = 10;
    public static final int ZONE_BORDER = 11;
    public static final int GOAL = 12;
    public static final int VISIBLE_MONSTER_TRIGGER = 13;
    public static final int INVISIBLE_MONSTER_TRIGGER = 14;
    public static final int DUAL_MONSTER_TRIGGER = 15;
    public static final int ITEM_TRIGGER = 16;
    public static final int MUST_BE_EXPLORED = 17;
    public static final int AUTOMATIC_EXIT = 18;
    public static final int MINOR_OUCH = 19;
    public static final int MAJOR_OUCH = 20;
    public static final int GLUE = 21;
    public static final int GLUE_TRIGGER = 22;
    public static final int SUPERGLUE = 23;

    public PolyEntry(PolyChunk chunk) {
	super(chunk);
    }

    public void setPermutation(short permutation) {
	this.permutation = permutation;
    }

    public short getPermutation() {
	return this.permutation;
    }

    public void setType(short type) {
	this.type = type;
    }

    public short getType() {
	return this.type;
    }

    public void setFlags(int flags) {
	this.flags = flags;
    }

    public int getFlags() {
	return this.flags;
    }

    public double getX() {
	return this.center_x/1024.0;
    }

    public double getY() {
	return this.center_y/1024.0;
    }

    public ShapeDescriptor getFloorTexture() {
	return this.floor_texture;
    }

    public ShapeDescriptor getCeilingTexture() {
	return this.ceiling_texture;
    }

    public void setFloorTexture(ShapeDescriptor s) {
	this.floor_texture = s;
    }

    public void setCeilingTexture(ShapeDescriptor s) {
	this.ceiling_texture = s;
    }

    public void setCeilingTransferMode(short t) {
	this.ceiling_transfer_mode = t;
    }
    public void setFloorTransferMode(short t) {this.floor_transfer_mode = t;}

    public short getCeilingTransferMode() {return this.ceiling_transfer_mode;}
    public short getFloorTransferMode() {return this.floor_transfer_mode;}

    public PolyEntry read() {
	MapWriter mapWriter = this.chunk.getLevel().getMapData().getMapWriter();
	byte[] data = new byte[128];
	mapWriter.read(data);
	WrkArrayReader mw = new WrkArrayReader(data);
	this.type = mw.readShort(); //2
	//	System.out.print(this.type+"\n");
	this.flags = mw.readShort(); //4
	this.permutation = mw.readShort(); //6
	this.vertex_count = mw.readShort(); //8
	for (int i=0; i<this.endpoint_indexes.length; i++) {
	    this.endpoint_indexes[i] = mw.readShort(); //24
	}
	for (int i=0; i<this.line_indexes.length; i++) {
	    this.line_indexes[i] = mw.readShort(); //40
	}
	short shr = mw.readShort();
	this.floor_texture = new ShapeDescriptor(shr); //42
	shr = mw.readShort();
	this.ceiling_texture = new ShapeDescriptor(shr); //44
	this.floor_height = mw.readShort(); //46
	this.ceiling_height = mw.readShort(); //48
	this.floor_light_index = mw.readShort(); //50
	this.ceiling_light_index = mw.readShort(); //52
	this.area = mw.readInt(); //56
	this.first_object = mw.readShort(); //58
	this.first_exclusion_zone_index = mw.readShort(); //60
	this.line_exclusion_count = mw.readShort(); //62
	this.point_exclusion_count = mw.readShort(); //64
	this.floor_transfer_mode = mw.readShort(); //66
	this.ceiling_transfer_mode = mw.readShort(); //68
	for (int i=0; i<this.adjacent_polygon_indexes.length; i++) {
	    this.adjacent_polygon_indexes[i] = mw.readShort(); //84
	}
	this.first_neighbor_index = mw.readShort(); //86
	this.neighbor_count = mw.readShort(); //88
	this.center_x = mw.readShort(); //90
	this.center_y = mw.readShort(); //92
	for (int i=0; i<this.side_indexes.length; i++) {
	    this.side_indexes[i] = mw.readShort(); //108
	}
	this.floor_x = mw.readShort(); //110
	this.floor_y = mw.readShort(); //112
	this.ceiling_x = mw.readShort(); //114
	this.ceiling_y = mw.readShort(); //116
	this.media_index = mw.readShort(); //118
	this.media_light_index = mw.readShort(); //120
	this.sound_source_index = mw.readShort(); //122
	this.ambient_sound_index = mw.readShort(); //124
	this.random_sound_index = mw.readShort();
	this.unused = mw.readShort(); //126
	
	return this;
    }

    public void write() {
	MapWriter mapWriter = this.chunk.getLevel().getMapData().getMapWriter();
	byte[] data = new byte[128];
	WrkArrayReader mw = new WrkArrayReader(data);
	mw.writeShort(this.type);
	mw.writeShort((short) this.flags);
	mw.writeShort(this.permutation);
	mw.writeShort((short) this.vertex_count);
	for (int i=0; i<this.endpoint_indexes.length; i++) {
	    mw.writeShort(this.endpoint_indexes[i]);
	}
	for (int i=0; i<this.line_indexes.length; i++) {
	    mw.writeShort(this.line_indexes[i]);
	}
	mw.writeShort((short) this.floor_texture.getDesc());
	mw.writeShort((short) this.ceiling_texture.getDesc());
	mw.writeShort(this.floor_height);
	mw.writeShort(this.ceiling_height);
	mw.writeShort(this.floor_light_index);
	mw.writeShort(this.ceiling_light_index);
	mw.writeInt(this.area);
	mw.writeShort(this.first_object);
	mw.writeShort(this.first_exclusion_zone_index);
	mw.writeShort(this.line_exclusion_count);
	mw.writeShort(this.point_exclusion_count);
	mw.writeShort(this.floor_transfer_mode);
	mw.writeShort(this.ceiling_transfer_mode);
	for (int i=0; i<this.adjacent_polygon_indexes.length; i++) {
	    mw.writeShort(this.adjacent_polygon_indexes[i]);
	}
	mw.writeShort(this.first_neighbor_index);
	mw.writeShort(this.neighbor_count);
	mw.writeShort(this.center_x);
	mw.writeShort(this.center_y);
	for (int i=0; i<this.side_indexes.length; i++) {
	    mw.writeShort(this.side_indexes[i]);
	}
	mw.writeShort(this.floor_x);
	mw.writeShort(this.floor_y);
	mw.writeShort(this.ceiling_x);
	mw.writeShort(this.ceiling_y);
	mw.writeShort(this.media_index);
	mw.writeShort(this.media_light_index);
	mw.writeShort(this.sound_source_index);
	mw.writeShort(this.ambient_sound_index);
	mw.writeShort(this.random_sound_index);
	mw.writeShort((short) this.unused);

	mapWriter.write(data);
    }
}
