/*Copyright 2007 Jonathan Irons

This file is part of JUICE.

JUICE is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

JUICE is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with JUICE; if not, write to the Free Software
Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/


    package src.backend;

import java.io.*;
import src.gui.*;
import src.backend.*;
import java.util.ArrayList;

public class ShPaChunk extends Chunk {
    //this is really just a binary file; hopefully, it contains a
    //shapes patch.

    private static final int MAXIMUM_LENGTH = 262144;
    private static final int SHPA_MAGIC_END = 0x656e6463; //'endc' crappy magic number
    private ShPaEntry patch;

    public ShPaChunk (Level level, ChunkHeader header) {
	super(level, header);
    }

    public ShPaChunk() {
	super();
    }

    public ShPaChunk(byte[] data) {
	super();
	ShPaEntry e = new ShPaEntry(this, data);
	this.patch = e;
    }

    public ShPaChunk(String path) throws NotShPaException, TooLongException {
	super();
	try {
	    RandomAccessFile r = new RandomAccessFile(path, "r");
	    if (r.length() > MAXIMUM_LENGTH) {
		System.out.println("File "+
				   path+" exceeds maximum patch length.");
		throw new TooLongException();
	    }
	    r.seek(r.length()-4);
	    int magic = r.readInt();
	    if (magic != SHPA_MAGIC_END) {
		System.out.println("File "+
				   path+" is not a shapes patch.");
		throw new NotShPaException();
	    }
	    r.seek(0);
	    byte[] b = new byte[(int)r.length()];
	    r.read(b);
	    ShPaEntry e = new ShPaEntry(this, b);
	    this.patch = e;
	} catch(NotShPaException notShape){
            throw new NotShPaException();
	} catch(TooLongException tooLong){
            throw new TooLongException();
	} catch (Exception e) {
	    System.out.println("Exception in ShPa chunk:");
	    e.printStackTrace();
	}
    }

    public void read() {
	super.read();
	MapWriter mw = this.level.getMapData().getMapWriter();
       	mw.seek(this.header.getOffset() + 16);
	byte[] b = new byte[this.header.getLength()];
	mw.read(b);
	ShPaEntry s = new ShPaEntry(this, b);
	this.patch = s;
    }

    public void write() {
	super.write();
	MapWriter mw = this.level.getMapData().getMapWriter();
	mw.seek(this.header.getOffset() + 16);
	mw.write(this.patch.getData());
    }

    public int getEntryLength() {return this.patch.length();}
    public int getNumEntries() {return 1;}

    public ShPaEntry getPatch() {return this.patch;}

    public void setPatch(ShPaEntry p) {
	int old = this.patch.length();
	int neo = p.length();
	int diff = neo - old;
	this.patch = p;
	this.header.autoAdjust(diff);
    }
    public void blankPatch() {
	byte[] data = new byte[0];
	setPatch(new ShPaEntry(this, data));
    }
}

//class NotShPaException extends Exception {}

//class TooLongException extends Exception {}
