/*Copyright 2007 Jonathan Irons

This file is part of JUICE.

JUICE is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

JUICE is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with JUICE; if not, write to the Free Software
Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/


    package src.backend;

import src.gui.*;
import src.backend.*;
import java.util.ArrayList;

public class SidsChunk extends Chunk {
    
    private ArrayList entries = new ArrayList(0);
    private int numEntries;
    
    public SidsChunk(Level level, ChunkHeader header) {
	super(level, header);
    }

    public SidsChunk() {
	super();
    }

    public SidsEntry getEntry(int entry) {
	if (entry < this.entries.size()) {
	    return (SidsEntry)this.entries.get(entry);
	} else {
	    System.out.println("ObjsChunk does not contain that many entries. Returning highest entry.");
	    return (SidsEntry)this.entries.get(this.entries.size() - 1);
	}
    }

    public void read() {
	super.read();
	MapWriter mw = this.level.getMapData().getMapWriter();
	this.numEntries = (int) this.header.getLength() / SidsEntry.ENTRY_LENGTH;
       	mw.seek(this.header.getOffset() + 16);
	for (int i=0; i<this.numEntries; i++) {
	    SidsEntry sidsEntry = new SidsEntry(this);
	    sidsEntry.read();
	    this.entries.add(sidsEntry);
	}
    }

    public void write() {
	super.write();
	MapWriter mw = this.level.getMapData().getMapWriter();
	mw.seek(this.header.getOffset() + 16);
	for (int i=0; i<this.entries.size(); i++) {
	    //    System.out.print("s"+i);
	    ((SidsEntry)this.entries.get(i)).write();
	}
    }

    public int getNumEntries() {
	return this.entries.size();
    }

    public int getEntryLength() {
	return SidsEntry.ENTRY_LENGTH;
    }

    public SidsEntry addEntry() {
	SidsEntry o = new SidsEntry(this);
	int len = getEntryLength();
	this.header.autoAdjust(len);
	this.numEntries++;
	return o;
    }

    public void addEntry(SidsEntry oe) {
	int len = getEntryLength();
	this.entries.add(oe);
	this.header.autoAdjust(len);
	this.numEntries++;
    }

    public SidsEntry removeEntry(SidsEntry o) {
	for (int i=0; i<this.entries.size(); i++) {
	    if (o.equals(getEntry(i))) {
		this.entries.remove(i);
		int len = -1 * getEntryLength();
		this.header.autoAdjust(len);
		this.numEntries--;
		return o;
	    }
	}
	System.out.println("That SidsEntry does not exist in the SidsChunk and can't be removed.");
	return o;
    }

    public void convertTextures() {
	for (int i=0; i<this.entries.size(); i++) {
	    SidsEntry se = (SidsEntry) this.entries.get(i);
	    SideTexture[] st = se.getSideTextures();
	    for (int j=0; j<st.length; j++) {
		ShapeDescriptor sd = st[j].getTexture();
		for (int k=1; k<StaticFunctions.TEXTURE_CUTOFFS.length; k++) {
		    if (sd.getColl() == StaticFunctions.WATER_COLLECTION &&
			sd.getBmp() >= StaticFunctions.TEXTURE_CUTOFFS[k-1] &&
			sd.getBmp() < StaticFunctions.TEXTURE_CUTOFFS[k]) {

			sd.setColl(StaticFunctions.WATER_COLLECTION + k);
			short diff = (short) (sd.getBmp() - StaticFunctions.TEXTURE_CUTOFFS[k-1]);
			sd.setBmp(diff);
			sd.setColl(k);
		    }
		}
	    }
	}	
    }

    public void convertTextures(short coll1, short bmp1,
				short coll2, short bmp2) {
	for (int i=0; i<this.entries.size(); i++) {
	    SidsEntry se = (SidsEntry) this.entries.get(i);
	    SideTexture[] st = se.getSideTextures();
	    for (int j=0; j<st.length; j++) {
		ShapeDescriptor sd = st[j].getTexture();
		if (sd.getColl() == coll1 && sd.getBmp() == bmp1) {
		    sd.setColl(coll2);
		    sd.setBmp(bmp2);
		    if (coll2 >= 27 && coll2 <= 30)
			if (j == 0)
			    se.setPrimaryTransfer((short) 9);
			else if (j==1)
			    se.setSecondaryTransfer((short) 9);
			else
			    se.setTransparentTransfer((short) 9);
		}
	    }
	}
    }
    public void convertTextures(short coll1, short bmp1, short x1,
				short coll2, short bmp2, short x2) {

	for (int i=0; i<this.entries.size(); i++) {
	    SidsEntry se = (SidsEntry) this.entries.get(i);
	    SideTexture[] st = se.getSideTextures();
	    for (int j=0; j<st.length; j++) {
		ShapeDescriptor sd = st[j].getTexture();
		if (sd.getColl() == coll1 && sd.getBmp() == bmp1) {

		    if (j == 0 && se.getPrimaryTransfer() == x1) {
			sd.setColl(coll2);
			sd.setBmp(bmp2);
			se.setPrimaryTransfer(x2);
		    }
		    if (j==1 && se.getSecondaryTransfer() == x1) {
			se.setSecondaryTransfer(x2);
			sd.setColl(coll2);
			sd.setBmp(bmp2);
		    }
		    if (j==2 && se.getTransparentTransfer() == x1) {
			se.setTransparentTransfer(x2);
			sd.setColl(coll2);
			sd.setBmp(bmp2);
		    }
		}
	    }
	}
    }

    //Solving Aleph One's control panel hard-code madness...
    
    /* A1 maps panel textures based on "panel type," so we have to
       use a fourth parameter to change panel textures.
    */

    public void convertTextures(short coll1, short bmp1, short x1, short cp1,
				short coll2, short bmp2, short x2, short cp2) {

	for (int i=0; i<this.entries.size(); i++) {
	    SidsEntry se = (SidsEntry) this.entries.get(i);

	    boolean control = ((se.getFlags() & 0x0002) != 0);
	    boolean match = (se.getPanelType() == cp1);
	    boolean ok = (control && match);

	    if (ok) {
		SideTexture[] st = se.getSideTextures();
		for (int j=0; j<st.length; j++) {
		    ShapeDescriptor sd = st[j].getTexture();
		    if (sd.getColl() == coll1 && sd.getBmp() == bmp1) {
			boolean shouldSet = false;
			if (j == 0 && se.getPrimaryTransfer() == x1) {
			    sd.setColl(coll2);
			    sd.setBmp(bmp2);
			    se.setPrimaryTransfer(x2);
			    shouldSet = true;
			}
			if (j==1 && se.getSecondaryTransfer() == x1) {
			    se.setSecondaryTransfer(x2);
			    sd.setColl(coll2);
			    sd.setBmp(bmp2);
			    shouldSet = true;
			}
			if (j==2 && se.getTransparentTransfer() == x1) {
			    se.setTransparentTransfer(x2);
			    sd.setColl(coll2);
			    sd.setBmp(bmp2);
			    shouldSet = true;
			}

			if (shouldSet)
			    se.setPanelType(cp2);
		    }
		}
	    }
	}
    }
}
