/*Copyright 2007 Jonathan Irons

This file is part of JUICE.

JUICE is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

JUICE is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with JUICE; if not, write to the Free Software
Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

    package src.backend;

import src.gui.*;
import src.backend.*;

public class SidsEntry extends Entry {

    public final static int ENTRY_LENGTH = 64;

    short type; //2
    short flags; //4
    SideTexture primary, secondary, transparent; //10, 16, 22
    int e0, e1, e2, e3; //26, 30, 34, 38
    short panel_type; //40
    short panel_permutation; //42
    short primary_transfer, secondary_transfer, transparent_transfer; //44, 46, 48
    short polygon_index, line_index; //52
    short primary_light, secondary_light, transparent_light; //54, 56,58
    int ambient_delta; //62 wtf is this + unused short = 64
    short unused;

    public SidsEntry (SidsChunk chunk) {
	super(chunk);
    }

    public short getType() {return this.type;}
    public short getFlags() {return this.flags;}
    public SideTexture[] getSideTextures() {
	SideTexture s[] = {this.primary, this.secondary, this.transparent};
	return s;
    }
    public short getPanelType() {return this.panel_type;}
    public short getPanelPermutation() {return this.panel_permutation;}
    public short getPrimaryTransfer() {return this.primary_transfer;}
    public short getSecondaryTransfer() {return this.secondary_transfer;}
    public short getTransparentTransfer() {return this.transparent_transfer;}
    public short getPolygonIndex() {return this.polygon_index;}
    public short getLineIndex() {return this.line_index;}
    public short getPrimaryLight() {return this.primary_light;}
    public short getSecondaryLight() {return this.secondary_light;}
    public short getTransparentLight() {return this.transparent_light;}
    
    public void setType(short t) {this.type = t;}
    public void setFlags(short f) {this.flags = f;}
    public void setSideTextures(SideTexture s1, SideTexture s2, SideTexture s3) {
	this.primary = s1;
	this.secondary = s2;
	this.transparent = s3;
    }
    public void setPanelType (short t) {this.panel_type = t;}
    public void setPanelPermutation(short p) {this.panel_permutation = p;}
    public void setPrimaryTransfer(short t) {this.primary_transfer = t;}
    public void setSecondaryTransfer(short t) {this.secondary_transfer = t;}
    public void setTransparentTransfer(short t) {this.transparent_transfer = t;}
    public void setPolygonIndex(short i) {this.polygon_index = i;}
    public void setLineIndex(short i) {this.line_index = i;}
    public void setPrimaryLight(short l) {this.primary_light = l;}
    public void setSecondaryLight(short l) {this.secondary_light = l;}
    public void setTransparentLight(short l) {this.transparent_light = l;}

    public SidsEntry read() {
	//	System.out.println("\nReading SIDS entry.");
	MapWriter mapWriter = this.chunk.getLevel().getMapData().getMapWriter();
	byte[] data = new byte[this.ENTRY_LENGTH];
	//System.out.println("Reading next "+this.ENTRY_LENGTH+" bytes of data.");
	mapWriter.read(data);
	WrkArrayReader wrk = new WrkArrayReader(data);
	this.type = wrk.readShort();
	this.flags = wrk.readShort();

	//read the sidetexutres
	short x, y, d;
	x = wrk.readShort();
	y = wrk.readShort();
	d = wrk.readShort();
	ShapeDescriptor s1 = new ShapeDescriptor(d);
	this.primary = new SideTexture(x, y, s1);
	x = wrk.readShort();
	y = wrk.readShort();
	d = wrk.readShort();
	ShapeDescriptor s2 = new ShapeDescriptor(d);
	this.secondary = new SideTexture(x, y, s2);
	x = wrk.readShort();
	y = wrk.readShort();
	d = wrk.readShort();
	ShapeDescriptor s3 = new ShapeDescriptor(d);
	this.transparent = new SideTexture(x, y, s3);
	//exclusion zone?!
	this.e0 = wrk.readInt();
	this.e1 = wrk.readInt();
	this.e2 = wrk.readInt();
	this.e3 = wrk.readInt();

	this.panel_type = wrk.readShort();
	this.panel_permutation = wrk.readShort();
	this.primary_transfer = wrk.readShort();
	this.secondary_transfer = wrk.readShort();
	this.transparent_transfer = wrk.readShort();
	this.polygon_index = wrk.readShort();
	this.line_index = wrk.readShort();
	this.primary_light = wrk.readShort();
	this.secondary_light = wrk.readShort();
	this.transparent_light = wrk.readShort();
	this.ambient_delta = wrk.readInt();

	return this;
    }

    public void write() {
	//System.out.println("Writing SIDS ENTRY!!!!!");
	MapWriter mapWriter = this.chunk.getLevel().getMapData().getMapWriter();
	byte[] data = new byte[this.ENTRY_LENGTH];
	WrkArrayReader wrk = new WrkArrayReader(data);
	wrk.writeShort(this.type);
	wrk.writeShort(this.flags);

	short x, y, desc;
	ShapeDescriptor d;
	x = this.primary.getX();
	y = this.primary.getY();
	d = this.primary.getTexture();
	desc = (short) d.getDesc();
	wrk.writeShort(x);
	wrk.writeShort(y);
	wrk.writeShort(desc);

	x = this.secondary.getX();
	y = this.secondary.getY();
	d = this.secondary.getTexture();
	desc = (short) d.getDesc();
	wrk.writeShort(x);
	wrk.writeShort(y);
	wrk.writeShort(desc);

	x = this.transparent.getX();
	y = this.transparent.getY();
	d = this.transparent.getTexture();
	desc = (short) d.getDesc();
	wrk.writeShort(x);
	wrk.writeShort(y);
	wrk.writeShort(desc);
	
	wrk.writeInt(this.e0);
	wrk.writeInt(this.e1);
	wrk.writeInt(this.e2);
	wrk.writeInt(this.e3);

	wrk.writeShort(this.panel_type);
	wrk.writeShort(this.panel_permutation);
	wrk.writeShort(this.primary_transfer);
	wrk.writeShort(this.secondary_transfer);
	wrk.writeShort(this.transparent_transfer);
	wrk.writeShort(this.polygon_index);
	wrk.writeShort(this.line_index);
	wrk.writeShort(this.primary_light);
	wrk.writeShort(this.secondary_light);
	wrk.writeShort(this.transparent_light);
	wrk.writeInt(this.ambient_delta);
	wrk.writeShort(this.unused);
	
	mapWriter.write(data);
    }
}
