/*
  Copyright 2007 Darren Watts

  This file is part of JUICE.

  JUICE is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  JUICE is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with JUICE; if not, write to the Free Software
  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

package src.backend;

import java.util.StringTokenizer;
import src.gui.Model;

public class TextureScriptParser {

    public TextureScriptParser(int levelIndex, Model model, String infile, String outfile) {
        parse(levelIndex, model, infile, outfile);
    }

    private void parse(int levelIndex, Model model, String infile, String outfile) {
        GetFile file = new GetFile(infile);
        GetFile ofile = new GetFile(outfile);
        ofile = new GetFile(outfile); // Need this twice so it can write if file did not exist before.
        StringTokenizer st1, st2;
        String currentToken = "";
        int col1, bit1, trans1, cp1, col2, bit2, trans2, cp2;
        String partLine = "";
        int line = 0;
        String writeLine = "";
        while (!file.havehitEOF()) {
            line++;
            trans1 = -1; trans2 = -1; cp1 = -1; cp2 = -1;
            try{
                st1 = new StringTokenizer(file.getLine(), "-");
                partLine = st1.nextToken();
                if(partLine.substring(0,1).equals("#")) continue;
                st2 = new StringTokenizer(partLine, ",");
            } catch(Exception e){ continue; }
            try{
                col1 = Integer.parseInt(st2.nextToken());
                bit1 = Integer.parseInt(st2.nextToken());
                if (st2.hasMoreTokens()) {
                    trans1 = Integer.parseInt(st2.nextToken());
                    if(st2.hasMoreTokens()){
                        cp1 = Integer.parseInt(st2.nextToken());
                        st2 = new StringTokenizer(st1.nextToken(), ",");
                        col2 = Integer.parseInt(st2.nextToken());
                        bit2 = Integer.parseInt(st2.nextToken());
                        trans2 = Integer.parseInt(st2.nextToken());
                        cp2 = Integer.parseInt(st2.nextToken());
                        writeLine += col2 + "," + bit2 + "," + trans2 + "," + cp2 + "-" +
                                col1 + "," + bit1 + "," + trans1 + cp1 + "\n";
                    }
                    else {
                        st2 = new StringTokenizer(st1.nextToken(), ",");
                        col2 = Integer.parseInt(st2.nextToken());
                        bit2 = Integer.parseInt(st2.nextToken());
                        trans2 = Integer.parseInt(st2.nextToken());
                        writeLine += col2 + "," + bit2 + "," + trans2 + "-" + col1 +
                                "," + bit1 + "," + trans1 + "\n";
                    }
                } else {
                    st2 = new StringTokenizer(st1.nextToken(), ",");
                    col2 = Integer.parseInt(st2.nextToken());
                    bit2 = Integer.parseInt(st2.nextToken());
                    writeLine += col2 + "," + bit2 + "-" + col1 + "," + bit1 + "\n";
                }
                if (trans1 == -1)
                    model.convertTextures(levelIndex, col1, bit1, col2, bit2);
                else if( cp1 == -1)
                    model.convertTextures(levelIndex, col1, bit1, trans1, col2,
                                          bit2, trans2);
                else model.convertTextures(levelIndex, col1, bit1, trans1, cp1, col2,
                                          bit2, trans2, cp2);
            } catch(Exception e) {
                System.out.println("Syntax error in script: Line " + line);
                continue;
            }
        }
        ofile.setLine(writeLine);
        file.close();
        ofile.close();
    }

}
