/*
Copyright 2007 Darren Watts

This file is part of JUICE.

JUICE is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

JUICE is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with JUICE; if not, write to the Free Software
Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

package src.gui;

import java.awt.*;
import javax.swing.*;
import javax.swing.text.PlainDocument;
import javax.swing.text.AttributeSet;
import javax.swing.text.BadLocationException;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import src.backend.*;
import java.io.File;

public class CommentFrame extends JFrame {
    private Model model;
    private int levelIndex;

    public CommentFrame(Model model, int levelIndex) {
        try {
            this.model = model;
            this.levelIndex = levelIndex;
            jbInit();
        } catch (Exception exception) {
            exception.printStackTrace();
        }
    }

    private void jbInit() throws Exception {
        FixedSizePlainDocument doc = new FixedSizePlainDocument(StaticFunctions.MAX_JUCE_COMMENT_SIZE);
        getContentPane().setLayout(null);
        txtComment.setBorder(BorderFactory.createLineBorder(Color.black));
        txtComment.setDocument(doc);
        txtComment.setText("");
        txtComment.setLineWrap(true);
        txtComment.setWrapStyleWord(true);
        txtComment.setBounds(new Rectangle( 0, 0, 400, 255));
        btnOK.setBounds(new Rectangle(202, 261, 80, 31));
        btnOK.setText("OK");
        btnOK.addActionListener(new CommentFrame_btnOK_actionAdapter(this));
        btnCancel.setBounds(new Rectangle(113, 261, 80, 31));
        btnCancel.setText("Cancel");
        btnCancel.addActionListener(new CommentFrame_btnCancel_actionAdapter(this));
        this.setJMenuBar(jMenuBar1);
        jMenu1.setText("Commands");
        jMenuItem1.setText("Read from file");
        jMenuItem1.addActionListener(new CommentFrame_jMenuItem1_actionAdapter(this));
        this.getContentPane().add(txtComment);
        this.getContentPane().add(btnOK);
        this.getContentPane().add(btnCancel);
        jMenuBar1.add(jMenu1);
        jMenu1.add(jMenuItem1);

        txtComment.setText(model.getComment(levelIndex));

        setSize(410, 345);
        setTitle("Level Comment - " +
                   model.getLevelSelectName());
        StaticFunctions.centerFrame(this);
        setResizable(false);
        setVisible(true);

    }

    JTextArea txtComment = new JTextArea();
    JButton btnOK = new JButton();
    JButton btnCancel = new JButton();
    JMenuBar jMenuBar1 = new JMenuBar();
    JMenu jMenu1 = new JMenu();
    JMenuItem jMenuItem1 = new JMenuItem();

    public void btnCancel_actionPerformed(ActionEvent e) {
        model.removeChildFrame(this);
        this.dispose();
    }

    public void btnOK_actionPerformed(ActionEvent e) {
        model.setComment(levelIndex, txtComment.getText());
        model.removeChildFrame(this);
        this.dispose();
    }

    public void jMenuItem1_actionPerformed(ActionEvent e) { // Read from file.
        String openFile = "";
        final JFileChooser fc = new JFileChooser();

        String path = Preferences.MAPS_PATH;
        try {
            if (!path.equals(""))
                fc.setCurrentDirectory(new File(path));
        } catch (Exception ex) {
            fc.setCurrentDirectory(new File("user.dir"));
        }
        int retVal = fc.showOpenDialog(CommentFrame.this);
        if (retVal == JFileChooser.APPROVE_OPTION)
            openFile = fc.getSelectedFile().getPath();
        else if (retVal == JFileChooser.CANCEL_OPTION)return;
        GetFile theFile = new GetFile(openFile);
        String text = theFile.getNumChars(512);
        txtComment.setText(text);
        theFile.close();
    }
}


class CommentFrame_jMenuItem1_actionAdapter implements ActionListener {
    private CommentFrame adaptee;
    CommentFrame_jMenuItem1_actionAdapter(CommentFrame adaptee) {
        this.adaptee = adaptee;
    }

    public void actionPerformed(ActionEvent e) {
        adaptee.jMenuItem1_actionPerformed(e);
    }
}


class CommentFrame_btnOK_actionAdapter implements ActionListener {
    private CommentFrame adaptee;
    CommentFrame_btnOK_actionAdapter(CommentFrame adaptee) {
        this.adaptee = adaptee;
    }

    public void actionPerformed(ActionEvent e) {
        adaptee.btnOK_actionPerformed(e);
    }
}


class CommentFrame_btnCancel_actionAdapter implements ActionListener {
    private CommentFrame adaptee;
    CommentFrame_btnCancel_actionAdapter(CommentFrame adaptee) {
        this.adaptee = adaptee;
    }

    public void actionPerformed(ActionEvent e) {
        adaptee.btnCancel_actionPerformed(e);
    }
}


class FixedSizePlainDocument extends PlainDocument {
    int maxSize;

    public FixedSizePlainDocument(int limit) {
        maxSize = limit;
    }

    public void insertString(int offset, String str, AttributeSet a) throws BadLocationException {
        if ((getLength() + str.length()) <= maxSize) {
            super.insertString(offset, str, a);
        } else {
            throw new BadLocationException("Insertion location is not valid", offset);
        }
    }
}
