/*
  Copyright 2007 Darren Watts

  This file is part of JUICE.

  JUICE is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  JUICE is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with JUICE; if not, write to the Free Software
  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

package src.gui;

import java.awt.*;
import javax.swing.*;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import src.backend.StaticFunctions;
import src.backend.Preferences;
import src.backend.TextureScriptParser;
import java.io.File;

public class ConvertFrame extends JFrame {
    private Model model;
    private int levelIndex;

    public ConvertFrame(Model model, int levelIndex) {
        try {
            this.model = model;
            this.levelIndex = levelIndex;
            jbInit();
        } catch (Exception exception) {
            exception.printStackTrace();
        }
    }

    private void jbInit() throws Exception {
        getContentPane().setLayout(null);
        chkUseScript.setText("Use converter script");
        chkUseScript.setBounds(new Rectangle(7, 11, 149, 21));
        chkUseScript.addActionListener(new
                                       ConvertFrame_chkUseScript_actionAdapter(this));
        btnOK.setBounds(new Rectangle(101, 215, 80, 31));
        btnOK.setText("OK");
        btnOK.addActionListener(new ConvertFrame_btnOK_actionAdapter(this));
        jLabel1.setText("From:");
        jLabel1.setBounds(new Rectangle(7, 64, 103, 19));
        jLabel2.setText("Collection:");
        jLabel2.setBounds(new Rectangle(7, 84, 103, 19));
        cmbFromCollection.setBounds(new Rectangle(7, 109, 97, 18));
        cmbFromCollection.addActionListener(new
                ConvertFrame_cmbFromCollection_actionAdapter(this));
        cmbToCollection.setBounds(new Rectangle(7, 183, 97, 18));
        cmbToCollection.addActionListener(new
                ConvertFrame_cmbToCollection_actionAdapter(this));
        btnCancel.setBounds(new Rectangle(190, 215, 80, 31));
        btnCancel.setText("Done");
        btnCancel.addActionListener(new ConvertFrame_btnCancel_actionAdapter(this));
        jLabel3.setFont(new java.awt.Font("Dialog", Font.PLAIN, 15));
        jLabel3.setText("Swap a transfer mode");
        jLabel3.setBounds(new Rectangle(227, 37, 210, 29));
        jLabel4.setText("Bitmap:");
        jLabel4.setBounds(new Rectangle(117, 159, 103, 19));
        jLabel5.setText("Collection:");
        jLabel5.setBounds(new Rectangle(7, 159, 103, 19));
        jLabel6.setText("To:");
        jLabel6.setBounds(new Rectangle(7, 133, 94, 18));
        jLabel7.setText("Bitmap:");
        jLabel7.setBounds(new Rectangle(117, 84, 103, 19));
        txtFromBitmap.setBounds(new Rectangle(117, 109, 97, 18));
        txtToBitmap.setText("");
        txtToBitmap.setBounds(new Rectangle(117, 183, 97, 18));
        jCheckBox1.setText("Use converter script");
        jCheckBox1.setBounds(new Rectangle(7, 11, 149, 21));
        chkTransferMode.setText("Convert transfer modes");
        chkTransferMode.setBounds(new Rectangle(195, 11, 197, 21));
        chkTransferMode.addActionListener(new
                ConvertFrame_chkTransferMode_actionAdapter(this));
        jLabel8.setFont(new java.awt.Font("Dialog", Font.PLAIN, 15));
        jLabel8.setText("Swap texture");
        jLabel8.setBounds(new Rectangle(62, 37, 135, 29));
        cmbFromTransfer.setBounds(new Rectangle(254, 109, 97, 18));
        cmbToTransfer.setBounds(new Rectangle(254, 183, 97, 18));
        jLabel9.setToolTipText("");
        jLabel9.setText("Transfer mode:");
        jLabel9.setBounds(new Rectangle(254, 84, 103, 19));
        jLabel10.setText("Transfer Mode:");
        jLabel10.setBounds(new Rectangle(254, 159, 103, 19));
        this.getContentPane().add(jLabel1);
        this.getContentPane().add(jLabel6);
        this.getContentPane().add(jLabel5);
        this.getContentPane().add(cmbToCollection);
        this.getContentPane().add(jLabel2);
        this.getContentPane().add(jLabel7);
        this.getContentPane().add(jLabel4);
        this.getContentPane().add(txtFromBitmap);
        this.getContentPane().add(chkUseScript);
        this.getContentPane().add(jLabel8);
        this.getContentPane().add(jLabel3);
        this.getContentPane().add(cmbFromCollection);
        this.getContentPane().add(cmbToTransfer);
        this.getContentPane().add(cmbFromTransfer);
        this.getContentPane().add(txtToBitmap);
        this.getContentPane().add(btnCancel);
        this.getContentPane().add(btnOK);
        this.getContentPane().add(jLabel9);
        this.getContentPane().add(jLabel10);
        this.getContentPane().add(chkTransferMode);

        cmbFromCollection.addItem("Water");
        cmbFromCollection.addItem("Lava");
        cmbFromCollection.addItem("Sewage");
        cmbFromCollection.addItem("Jjaro");
        cmbFromCollection.addItem("Pfhor");

        cmbToCollection.addItem("Water");
        cmbToCollection.addItem("Lava");
        cmbToCollection.addItem("Sewage");
        cmbToCollection.addItem("Jjaro");
        cmbToCollection.addItem("Pfhor");

        for(int i = 0; i < StaticFunctions.TRANSFER_MODES.length; i++){
            cmbFromTransfer.addItem(StaticFunctions.TRANSFER_MODES[i]);
            cmbToTransfer.addItem(StaticFunctions.TRANSFER_MODES[i]);
        }

        cmbFromTransfer.setEnabled(false);
        cmbToTransfer.setEnabled(false);

        setSize(400, 300);
        setTitle("Convert Textures...");
        setResizable(false);
        StaticFunctions.centerFrame(this);
        setVisible(true);
    }

    JCheckBox chkUseScript = new JCheckBox();
    JButton btnOK = new JButton();
    JLabel jLabel1 = new JLabel();
    JLabel jLabel2 = new JLabel();
    JComboBox cmbFromCollection = new JComboBox();
    JComboBox cmbToCollection = new JComboBox();
    JButton btnCancel = new JButton();
    JLabel jLabel3 = new JLabel();
    JLabel jLabel4 = new JLabel();
    JLabel jLabel5 = new JLabel();
    JLabel jLabel6 = new JLabel();
    JLabel jLabel7 = new JLabel();
    JTextField txtFromBitmap = new JTextField();
    JTextField txtToBitmap = new JTextField();
    JCheckBox jCheckBox1 = new JCheckBox();
    JCheckBox chkTransferMode = new JCheckBox();
    JLabel jLabel8 = new JLabel();
    JComboBox cmbFromTransfer = new JComboBox();
    JComboBox cmbToTransfer = new JComboBox();
    JLabel jLabel9 = new JLabel();
    JLabel jLabel10 = new JLabel();

    public void btnOK_actionPerformed(ActionEvent e) { // OK button
        int bmp1 = 0, bmp2 = 0;
        if(!chkUseScript.isSelected()){
            try{
                bmp1 = Integer.parseInt(txtFromBitmap.getText());
                bmp2 = Integer.parseInt(txtToBitmap.getText());
                if (bmp1 < -5) bmp1 = 0;
                if (bmp1 > 65535) bmp1 = 65535;
                if (bmp2 < -5) bmp2 = 0;
                if (bmp2 > 65535) bmp2 = 65535;
            } catch(Exception ex){
                return;
            }
        }
        if(!chkUseScript.isSelected() && chkTransferMode.isSelected()){
            model.convertTextures(levelIndex,
                                  cmbFromCollection.getSelectedIndex() + 17,
                                  bmp1, cmbFromTransfer.getSelectedIndex(),
                                  cmbToCollection.getSelectedIndex() + 17,
                                  bmp2, cmbToTransfer.getSelectedIndex()
                    );
        }
        else if(!chkUseScript.isSelected()){
            model.convertTextures(levelIndex,
                                  cmbFromCollection.getSelectedIndex() + 17,
                                  bmp1,
                                  cmbToCollection.getSelectedIndex() + 17,
                                  bmp2
                    );
        }
        else if (chkUseScript.isSelected()) {
            //model.convertTextures(levelIndex);
            String openFile = "";
            String saveFile = "";
            JFileChooser fc = new JFileChooser();
            fc.setCurrentDirectory(new File(Preferences.SCRIPTS_PATH));
            int retVal = fc.showOpenDialog(ConvertFrame.this);
            if (retVal == JFileChooser.APPROVE_OPTION)
                openFile = fc.getSelectedFile().getPath();
            else if (retVal == JFileChooser.CANCEL_OPTION) return;

            saveFile = openFile + ".undo";

            TextureScriptParser t = new TextureScriptParser(levelIndex, model, openFile, saveFile);
            model.removeChildFrame(this);
            this.dispose();
        }
    } // end OK button

    public void btnCancel_actionPerformed(ActionEvent e) {
        model.removeChildFrame(this);
        this.dispose();
    }

    public void cmbFromCollection_actionPerformed(ActionEvent e) {
        if(cmbFromCollection.getSelectedItem().equals("Jjaro")) {
            for(int i = 0; i < 35; i++){
            }
        } else {
            for(int i = 0; i < 30; i++){
            }
        }
    }

    public void cmbToCollection_actionPerformed(ActionEvent e) {
        if(cmbToCollection.getSelectedItem().equals("Jjaro")) {
            for(int i = 0; i < 35; i++){
            }
        } else {
            for(int i = 0; i < 30; i++){
            }
        }
    }

    public void chkUseScript_actionPerformed(ActionEvent e) {
        if(chkUseScript.isSelected()){
            cmbFromCollection.setEnabled(false);
            cmbToCollection.setEnabled(false);
            txtFromBitmap.setEnabled(false);
            txtToBitmap.setEnabled(false);
            chkTransferMode.setEnabled(false);
            cmbFromTransfer.setEnabled(false);
            cmbToTransfer.setEnabled(false);
        }
        else if(!chkUseScript.isSelected() && chkTransferMode.isSelected()){
            cmbFromCollection.setEnabled(true);
            cmbToCollection.setEnabled(true);
            txtFromBitmap.setEnabled(true);
            txtToBitmap.setEnabled(true);
            chkTransferMode.setEnabled(true);
            cmbFromTransfer.setEnabled(true);
            cmbToTransfer.setEnabled(true);
        }
        else if(!chkUseScript.isSelected() && !chkTransferMode.isSelected()){
            cmbFromCollection.setEnabled(true);
            cmbToCollection.setEnabled(true);
            txtFromBitmap.setEnabled(true);
            txtToBitmap.setEnabled(true);
            chkTransferMode.setEnabled(true);
        }
    }

    public void chkTransferMode_actionPerformed(ActionEvent e) {
        if(chkTransferMode.isSelected()){
            cmbFromTransfer.setEnabled(true);
            cmbToTransfer.setEnabled(true);
        }
        else{
            cmbFromTransfer.setEnabled(false);
            cmbToTransfer.setEnabled(false);
        }
    }
}


class ConvertFrame_chkTransferMode_actionAdapter implements ActionListener {
    private ConvertFrame adaptee;
    ConvertFrame_chkTransferMode_actionAdapter(ConvertFrame adaptee) {
        this.adaptee = adaptee;
    }

    public void actionPerformed(ActionEvent e) {
        adaptee.chkTransferMode_actionPerformed(e);
    }
}


class ConvertFrame_chkUseScript_actionAdapter implements ActionListener {
    private ConvertFrame adaptee;
    ConvertFrame_chkUseScript_actionAdapter(ConvertFrame adaptee) {
        this.adaptee = adaptee;
    }

    public void actionPerformed(ActionEvent e) {
        adaptee.chkUseScript_actionPerformed(e);
    }
}


class ConvertFrame_cmbToCollection_actionAdapter implements ActionListener {
    private ConvertFrame adaptee;
    ConvertFrame_cmbToCollection_actionAdapter(ConvertFrame adaptee) {
        this.adaptee = adaptee;
    }

    public void actionPerformed(ActionEvent e) {
        adaptee.cmbToCollection_actionPerformed(e);
    }
}


class ConvertFrame_cmbFromCollection_actionAdapter implements ActionListener {
    private ConvertFrame adaptee;
    ConvertFrame_cmbFromCollection_actionAdapter(ConvertFrame adaptee) {
        this.adaptee = adaptee;
    }

    public void actionPerformed(ActionEvent e) {
        adaptee.cmbFromCollection_actionPerformed(e);
    }
}


class ConvertFrame_btnCancel_actionAdapter implements ActionListener {
    private ConvertFrame adaptee;
    ConvertFrame_btnCancel_actionAdapter(ConvertFrame adaptee) {
        this.adaptee = adaptee;
    }

    public void actionPerformed(ActionEvent e) {
        adaptee.btnCancel_actionPerformed(e);
    }
}


class ConvertFrame_btnOK_actionAdapter implements ActionListener {
    private ConvertFrame adaptee;
    ConvertFrame_btnOK_actionAdapter(ConvertFrame adaptee) {
        this.adaptee = adaptee;
    }

    public void actionPerformed(ActionEvent e) {
        adaptee.btnOK_actionPerformed(e);
    }
}
