/*
  Copyright 2007 Darren Watts

  This file is part of JUICE.

  JUICE is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  JUICE is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with JUICE; if not, write to the Free Software
  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

package src.gui;

import java.awt.*;
import javax.swing.*;
import src.backend.StaticFunctions;
import javax.swing.table.TableColumn;
import javax.swing.event.TableModelListener;
import javax.swing.event.TableModelEvent;
import javax.swing.table.TableCellRenderer;
import java.awt.event.MouseListener;
import java.awt.event.MouseEvent;
import java.awt.event.WindowEvent;
import java.awt.event.WindowListener;

public class LinsFrame extends JFrame {
    private String[] columnNames = {
                                   "Index", "Solid", "Transparent", "Landscaped",
                                   "Length", "High Floor", "Low Ceiling",
                                   "CW poly owner", "CCW poly owner"
    };

    private Model model;
    private int levelIndex;

    private JTable table;
    private JScrollPane scroller;
    private LinsTableModel tableModel;

    private int clickCount;

    public LinsFrame(Model model, int levelIndex) {
        try {
            this.model = model;
            this.levelIndex = levelIndex;
            jbInit();
        } catch (Exception exception) {
            exception.printStackTrace();
        }
    }

    private void jbInit() throws Exception {
        getContentPane().setLayout(new BorderLayout());
        tableModel = new LinsTableModel(columnNames, model, levelIndex);
        tableModel.addTableModelListener(new LinsFrame.LinsTableModelListener());
        table = new JTable(tableModel);
        table.setSurrendersFocusOnKeystroke(true);
        table.getTableHeader().setReorderingAllowed(false);

        TableColumn solidCol = table.getColumnModel().getColumn(1);
        TableColumn transparentCol = table.getColumnModel().getColumn(2);
        TableColumn landscapedCol = table.getColumnModel().getColumn(3);
        TableColumn cpowner = table.getColumnModel().getColumn(7);
        TableColumn ccpowner = table.getColumnModel().getColumn(8);
        JCheckBox chkSolid = new JCheckBox();
        JCheckBox chkTransparent = new JCheckBox();
        JCheckBox chkLandscape = new JCheckBox();

        solidCol.setCellEditor(new DefaultCellEditor(chkSolid));
        transparentCol.setCellEditor(new DefaultCellEditor(chkTransparent));
        landscapedCol.setCellEditor(new DefaultCellEditor(chkLandscape));

        solidCol.setCellRenderer(new TableCellRenderer() {
            public Component getTableCellRendererComponent(JTable table,
                    Object value,
                    boolean isSelected,
                    boolean hasFocus, int row,
                    int column) {

                JCheckBox box = new JCheckBox();
                box.setBackground(Color.white);
                try {
                    if (((Boolean)value).booleanValue()) {
                        box.setSelected(true);
                        return box;
                    } else {
                        box.setSelected(false);
                        return box;
                    }
                } catch (Exception e) {}
                return new JCheckBox();
            }
        });

        transparentCol.setCellRenderer(new TableCellRenderer() {
            public Component getTableCellRendererComponent(JTable table,
                    Object value,
                    boolean isSelected,
                    boolean hasFocus, int row,
                    int column) {

                JCheckBox box = new JCheckBox();
                box.setBackground(Color.white);
                try {
                    if (((Boolean)value).booleanValue()) {
                        box.setSelected(true);
                        return box;
                    } else {
                        box.setSelected(false);
                        return box;
                    }
                } catch (Exception e) {}
                return new JCheckBox();
            }
        });

        landscapedCol.setCellRenderer(new TableCellRenderer() {
            public Component getTableCellRendererComponent(JTable table,
                    Object value,
                    boolean isSelected,
                    boolean hasFocus, int row,
                    int column) {

                JCheckBox box = new JCheckBox();
                box.setBackground(Color.white);
                try {
                    if (((Boolean)value).booleanValue()) {
                        box.setSelected(true);
                        return box;
                    } else {
                        box.setSelected(false);
                        return box;
                    }
                } catch (Exception e) {}
                return new JCheckBox();
            }
        });

        cpowner.setCellRenderer(new TableCellRenderer() {
            public Component getTableCellRendererComponent(JTable table,
                    Object value,
                    boolean isSelected,
                    boolean hasFocus, int row,
                    int column) {

                JTextArea text = new JTextArea();
                text.setBackground(Color.LIGHT_GRAY);

                try {
                    text.setText(Integer.toString(((Integer)value).intValue()));
                } catch (Exception e) {}
                return text;
            }
        });

        ccpowner.setCellRenderer(new TableCellRenderer() {
            public Component getTableCellRendererComponent(JTable table,
                    Object value,
                    boolean isSelected,
                    boolean hasFocus, int row,
                    int column) {

                JTextArea text = new JTextArea();
                text.setBackground(Color.LIGHT_GRAY);

                try {
                    if(((Integer)value).intValue() == -1) text.setText("None");
                    else text.setText(Integer.toString(((Integer)value).intValue()));
                } catch (Exception e) {}
                return text;
            }
        });

        scroller = new JScrollPane(table);

        table.getTableHeader().addMouseListener(new MouseListener() {
            public void mouseExited(MouseEvent m) {}
            public void mouseEntered(MouseEvent m) {}
            public void mouseReleased(MouseEvent m) {}
            public void mousePressed(MouseEvent m) {}
            public void mouseClicked(MouseEvent m) {
                Point mousePoint = new Point(m.getX(), m.getY());
                int header = table.getTableHeader().columnAtPoint(mousePoint);

                clickCount++;
                boolean reverseOrder = false;
                if ((clickCount % 2) == 0) reverseOrder = true;
                tableModel.sort(header, reverseOrder);
            }
        });

        this.getContentPane().add(scroller);
        this.addWindowListener(new WindowListener() {
            public void windowDeactivated(WindowEvent e) {}
            public void windowActivated(WindowEvent e) {}
            public void windowDeiconified(WindowEvent e) {}
            public void windowIconified(WindowEvent e) {}
            public void windowClosed(WindowEvent e) {}
            public void windowClosing(WindowEvent e) {
                closeTable();
            }

            public void windowOpened(WindowEvent e) {}
        });

        clickCount = 0;

        setSize(600, 500);
        setTitle("Lines - " +
                   model.getLevelSelectName());
        StaticFunctions.centerFrame(this);
        setVisible(true);
    }

    private void closeTable() {
        model.removeChildFrame(this);
    }

    class LinsTableModelListener implements TableModelListener {
        public void tableChanged(TableModelEvent evt) {
            if (evt.getType() == TableModelEvent.UPDATE) {
                int column = evt.getColumn();
                int row = evt.getFirstRow();
                try {
                    table.setRowSelectionInterval(row, row);
                } catch (Exception e) {}
            }
        }
    }

}
