/*
  Copyright 2007 Darren Watts

  This file is part of JUICE.

  JUICE is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  JUICE is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with JUICE; if not, write to the Free Software
  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */

package src.gui;

import java.awt.*;
import javax.swing.*;
import src.backend.ObjsEntry;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyEvent;
import src.backend.StaticFunctions;

public class ObjsFlagsFrame extends JFrame {
    private final int MONSTER = 0;
    private final int SCENERY = 1;
    private final int ITEM = 2;
    private final int PLAYER = 3;
    private final int GOAL = 4;
    private final int SOUND = 5;

    private final int teleportIn = 0x0001;
    private final int platformSound = 0x0001;
    private final int fromCeiling = 0x0002;
    private final int isBlind = 0x0004;
    private final int isDeaf = 0x0008;
    private final int floats = 0x0010;
    private final int networkOnly = 0x0020;

    private final int not_teleportIn = 0xFFFE;
    private final int not_platformSound = 0xFFFE;
    private final int not_fromCeiling = 0xFFFD;
    private final int not_isBlind = 0xFFFB;
    private final int not_isDeaf = 0xFFF7;
    private final int not_floats = 0xFFEF;
    private final int not_networkOnly = 0xFFDF;

    private Model model;
    private ObjsEntry entry;
    JCheckBox chkFromCeiling = new JCheckBox();
    JCheckBox chkFloats = new JCheckBox();
    JCheckBox chkTeleportIn = new JCheckBox();
    JCheckBox chkDeaf = new JCheckBox();
    JCheckBox chkPlatformSound = new JCheckBox();
    JCheckBox chkBlind = new JCheckBox();
    JCheckBox chkNetworkOnly = new JCheckBox();
    JButton jButton1 = new JButton();
    public ObjsFlagsFrame(Model model, int levelIndex, ObjsEntry entry) {
        this.model = model;
        this.entry = entry;
        try {
            jbInit();
        } catch (Exception exception) {
            exception.printStackTrace();
        }
    }

    private void jbInit() throws Exception {
        getContentPane().setLayout(null);
        chkFromCeiling.setText("From Ceiling");
        chkFromCeiling.setBounds(new Rectangle(8, 47, 222, 23));
        chkFromCeiling.addActionListener(new
                                         ObjsFlagsFrame_chkFromCeiling_actionAdapter(this));
        chkNetworkOnly.setText("Network Only");
        chkNetworkOnly.setBounds(new Rectangle(8, 123, 139, 23));
        chkNetworkOnly.addActionListener(new
                                         ObjsFlagsFrame_chkNetworkOnly_actionAdapter(this));
        chkBlind.setText("Is blind");
        chkBlind.setBounds(new Rectangle(8, 66, 179, 23));
        chkBlind.addActionListener(new ObjsFlagsFrame_chkBlind_actionAdapter(this));
        chkPlatformSound.setText("Is platform sound");
        chkPlatformSound.setBounds(new Rectangle(8, 28, 167, 23));
        chkPlatformSound.addActionListener(new
                                           ObjsFlagsFrame_chkPlatformSound_actionAdapter(this));
        chkDeaf.setText("Is deaf");
        chkDeaf.setBounds(new Rectangle(8, 85, 184, 23));
        chkDeaf.addActionListener(new ObjsFlagsFrame_chkDeaf_actionAdapter(this));
        chkTeleportIn.setText("Teleports in");
        chkTeleportIn.setBounds(new Rectangle(8, 9, 154, 23));
        chkTeleportIn.addActionListener(new
                                        ObjsFlagsFrame_chkTeleportIn_actionAdapter(this));
        jButton1.setBounds(new Rectangle(52, 153, 78, 26));
        jButton1.setSelected(true);
        jButton1.setText("OK");
        jButton1.addActionListener(new ObjsFlagsFrame_jButton1_actionAdapter(this));
        chkFloats.addActionListener(new ObjsFlagsFrame_chkFloats_actionAdapter(this));
        this.getContentPane().add(chkTeleportIn);
        this.getContentPane().add(chkFromCeiling);
        this.getContentPane().add(chkFloats);
        this.getContentPane().add(chkDeaf);
        this.getContentPane().add(chkPlatformSound);
        this.getContentPane().add(chkNetworkOnly);
        this.getContentPane().add(chkBlind);
        this.getContentPane().add(jButton1);
        chkFloats.setEnabled(true);
        chkFloats.setText("Floats");
        chkFloats.setBounds(new Rectangle(8, 104, 165, 23));

        showRelevantBoxes();
        fillBoxes();
        this.addKeyListener(new java.awt.event.KeyAdapter() {
            public void keyPressed(KeyEvent e) {
                if(e.getKeyCode() == KeyEvent.VK_ENTER){
                    closeWindow();
                }
            }
        });
        jButton1.addKeyListener(new java.awt.event.KeyAdapter() {
            public void keyPressed(KeyEvent e) {
                if(e.getKeyCode() == KeyEvent.VK_ENTER){
                    closeWindow();
                }
            }
        });
        chkTeleportIn.addKeyListener(new java.awt.event.KeyAdapter() {
            public void keyPressed(KeyEvent e) {
                if(e.getKeyCode() == KeyEvent.VK_ENTER){
                    closeWindow();
                }
            }
        });
        chkFromCeiling.addKeyListener(new java.awt.event.KeyAdapter() {
            public void keyPressed(KeyEvent e) {
                if(e.getKeyCode() == KeyEvent.VK_ENTER){
                    closeWindow();
                }
            }
        });
        chkPlatformSound.addKeyListener(new java.awt.event.KeyAdapter() {
            public void keyPressed(KeyEvent e) {
                if(e.getKeyCode() == KeyEvent.VK_ENTER){
                    closeWindow();
                }
            }
        });
        chkBlind.addKeyListener(new java.awt.event.KeyAdapter() {
            public void keyPressed(KeyEvent e) {
                if(e.getKeyCode() == KeyEvent.VK_ENTER){
                    closeWindow();
                }
            }
        });
        chkDeaf.addKeyListener(new java.awt.event.KeyAdapter() {
            public void keyPressed(KeyEvent e) {
                if(e.getKeyCode() == KeyEvent.VK_ENTER){
                    closeWindow();
                }
            }
        });
        chkFloats.addKeyListener(new java.awt.event.KeyAdapter() {
            public void keyPressed(KeyEvent e) {
                if(e.getKeyCode() == KeyEvent.VK_ENTER){
                    closeWindow();
                }
            }
        });
        chkNetworkOnly.addKeyListener(new java.awt.event.KeyAdapter() {
            public void keyPressed(KeyEvent e) {
                if(e.getKeyCode() == KeyEvent.VK_ENTER){
                    closeWindow();
                }
            }
        });

        setSize(200, 210);
        setResizable(false);
        setTitle("Object flags");
        StaticFunctions.centerFrame(this);
        setVisible(true);

    }

    public void closeWindow(){
        model.removeChildFrame(this);
        this.dispose();
    }

    private void showRelevantBoxes(){
        try{
            if (entry.getType() == MONSTER) {
                chkPlatformSound.setVisible(false);
                chkNetworkOnly.setVisible(false);

                chkFloats.setText("Teleports out");
            } else if (entry.getType() == SCENERY) {
                chkTeleportIn.setVisible(false);
                chkPlatformSound.setVisible(false);
                chkBlind.setVisible(false);
                chkDeaf.setVisible(false);
                chkFloats.setVisible(false);
                chkNetworkOnly.setVisible(false);
            } else if (entry.getType() == ITEM) {
                chkPlatformSound.setVisible(false);
                chkBlind.setVisible(false);
                chkDeaf.setVisible(false);
                chkFloats.setVisible(false);
            } else if (entry.getType() == PLAYER) {
                chkTeleportIn.setVisible(false);
                chkPlatformSound.setVisible(false);
                chkBlind.setVisible(false);
                chkDeaf.setVisible(false);
                chkFloats.setVisible(false);
                chkNetworkOnly.setVisible(false);
            } else if (entry.getType() == GOAL) {
                chkTeleportIn.setVisible(false);
                chkPlatformSound.setVisible(false);
                chkFromCeiling.setVisible(false);
                chkBlind.setVisible(false);
                chkDeaf.setVisible(false);
                chkFloats.setVisible(false);
                chkNetworkOnly.setVisible(false);
            } else if (entry.getType() == SOUND) {
                chkTeleportIn.setVisible(false);
                chkBlind.setVisible(false);
                chkDeaf.setVisible(false);
                chkNetworkOnly.setVisible(false);

                chkPlatformSound.setText("Is on platform");
            }
        } catch(Exception e){
            return;
        }
    }

    private void fillBoxes(){
        try{
            int flags = entry.getFlags();

            if (entry.getType() == MONSTER) {
                // Teleports in, teleports out, from ceiling, is blind, is deaf
                chkTeleportIn.setSelected((flags & teleportIn) != 0);
                chkFloats.setSelected((flags & floats) != 0);
                chkFromCeiling.setSelected((flags & fromCeiling) != 0);
                chkBlind.setSelected((flags & isBlind) != 0);
                chkDeaf.setSelected((flags & isDeaf) != 0);
            } else if (entry.getType() == SCENERY) {
                // From ceiling
                chkFromCeiling.setSelected((flags & fromCeiling) != 0);
            } else if (entry.getType() == ITEM) {
                // Telepots in, from ceiling, network only
                chkTeleportIn.setSelected((flags & teleportIn) != 0);
                chkFromCeiling.setSelected((flags & fromCeiling) != 0);
                chkNetworkOnly.setSelected((flags * networkOnly) != 0);
            } else if (entry.getType() == PLAYER) {
                // From ceiling
                chkFromCeiling.setSelected((flags & fromCeiling) != 0);
            } else if (entry.getType() == GOAL) {
                // Nothing
            } else if (entry.getType() == SOUND) {
                // Is on platform, from ceiling, floats
                chkPlatformSound.setSelected((flags & platformSound) != 0);
                chkFromCeiling.setSelected((flags & fromCeiling) != 0);
                chkFloats.setSelected((flags & floats) != 0);
            }
        } catch(Exception e){
            return;
        }
    }

    public void jButton1_actionPerformed(ActionEvent e) {
        closeWindow();
    }

    public void chkTeleportIn_actionPerformed(ActionEvent e) {
        try{
            int flags = entry.getFlags();
            if (chkTeleportIn.isSelected())
                flags = flags | teleportIn;
            else flags = flags & not_teleportIn;
            entry.setFlags((short) flags);
        } catch(Exception ex){ return; }
    }

    public void chkPlatformSound_actionPerformed(ActionEvent e) {
        try{
            int flags = entry.getFlags();
            if (chkPlatformSound.isSelected())
                flags = flags | platformSound;
            else flags = flags & not_platformSound;
            entry.setFlags((short) flags);
        } catch(Exception ex){ return; }
    }

    public void chkFromCeiling_actionPerformed(ActionEvent e) {
        try{
            int flags = entry.getFlags();
            if (chkFromCeiling.isSelected())
                flags = flags | fromCeiling;
            else flags = flags & not_fromCeiling;
            entry.setFlags((short) flags);
        } catch(Exception ex){ return; }
    }

    public void chkBlind_actionPerformed(ActionEvent e) {
        try{
            int flags = entry.getFlags();
            if (chkBlind.isSelected())
                flags = flags | isBlind;
            else flags = flags & not_isBlind;
            entry.setFlags((short) flags);
        } catch(Exception ex){ return; }
    }

    public void chkDeaf_actionPerformed(ActionEvent e) {
        try{
            int flags = entry.getFlags();
            if (chkDeaf.isSelected())
                flags = flags | isDeaf;
            else flags = flags & not_isDeaf;
            entry.setFlags((short) flags);
        } catch(Exception ex){ return; }
    }

    public void chkFloats_actionPerformed(ActionEvent e) {
        try{
            int flags = entry.getFlags();
            if (chkFloats.isSelected())
                flags = flags | floats;
            else flags = flags & not_floats;
            entry.setFlags((short) flags);
        } catch(Exception ex){ return; }
    }

    public void chkNetworkOnly_actionPerformed(ActionEvent e) {
        try{
            int flags = entry.getFlags();
            if (chkNetworkOnly.isSelected())
                flags = flags | networkOnly;
            else flags = flags & not_networkOnly;
            entry.setFlags((short) flags);
        } catch(Exception ex){ return; }
    }
}


class ObjsFlagsFrame_chkNetworkOnly_actionAdapter implements ActionListener {
    private ObjsFlagsFrame adaptee;
    ObjsFlagsFrame_chkNetworkOnly_actionAdapter(ObjsFlagsFrame adaptee) {
        this.adaptee = adaptee;
    }

    public void actionPerformed(ActionEvent e) {
        adaptee.chkNetworkOnly_actionPerformed(e);
    }
}


class ObjsFlagsFrame_chkFloats_actionAdapter implements ActionListener {
    private ObjsFlagsFrame adaptee;
    ObjsFlagsFrame_chkFloats_actionAdapter(ObjsFlagsFrame adaptee) {
        this.adaptee = adaptee;
    }

    public void actionPerformed(ActionEvent e) {
        adaptee.chkFloats_actionPerformed(e);
    }
}


class ObjsFlagsFrame_chkDeaf_actionAdapter implements ActionListener {
    private ObjsFlagsFrame adaptee;
    ObjsFlagsFrame_chkDeaf_actionAdapter(ObjsFlagsFrame adaptee) {
        this.adaptee = adaptee;
    }

    public void actionPerformed(ActionEvent e) {
        adaptee.chkDeaf_actionPerformed(e);
    }
}


class ObjsFlagsFrame_chkBlind_actionAdapter implements ActionListener {
    private ObjsFlagsFrame adaptee;
    ObjsFlagsFrame_chkBlind_actionAdapter(ObjsFlagsFrame adaptee) {
        this.adaptee = adaptee;
    }

    public void actionPerformed(ActionEvent e) {
        adaptee.chkBlind_actionPerformed(e);
    }
}


class ObjsFlagsFrame_chkFromCeiling_actionAdapter implements ActionListener {
    private ObjsFlagsFrame adaptee;
    ObjsFlagsFrame_chkFromCeiling_actionAdapter(ObjsFlagsFrame adaptee) {
        this.adaptee = adaptee;
    }

    public void actionPerformed(ActionEvent e) {
        adaptee.chkFromCeiling_actionPerformed(e);
    }
}


class ObjsFlagsFrame_chkPlatformSound_actionAdapter implements ActionListener {
    private ObjsFlagsFrame adaptee;
    ObjsFlagsFrame_chkPlatformSound_actionAdapter(ObjsFlagsFrame adaptee) {
        this.adaptee = adaptee;
    }

    public void actionPerformed(ActionEvent e) {
        adaptee.chkPlatformSound_actionPerformed(e);
    }
}


class ObjsFlagsFrame_chkTeleportIn_actionAdapter implements ActionListener {
    private ObjsFlagsFrame adaptee;
    ObjsFlagsFrame_chkTeleportIn_actionAdapter(ObjsFlagsFrame adaptee) {
        this.adaptee = adaptee;
    }

    public void actionPerformed(ActionEvent e) {
        adaptee.chkTeleportIn_actionPerformed(e);
    }
}


class ObjsFlagsFrame_jButton1_actionAdapter implements ActionListener {
    private ObjsFlagsFrame adaptee;
    ObjsFlagsFrame_jButton1_actionAdapter(ObjsFlagsFrame adaptee) {
        this.adaptee = adaptee;
    }

    public void actionPerformed(ActionEvent e) {
        adaptee.jButton1_actionPerformed(e);
    }
}
