/*
  Copyright 2007 Darren Watts

  This file is part of JUICE.

  JUICE is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  JUICE is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with JUICE; if not, write to the Free Software
  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */

package src.gui;

import java.awt.*;
import javax.swing.*;
import javax.swing.table.TableColumn;
import javax.swing.event.TableModelListener;
import javax.swing.event.TableModelEvent;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import src.backend.*;
import javax.swing.table.TableCellEditor;
import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;
import java.awt.event.KeyEvent;
import java.util.ArrayList;
import java.awt.event.WindowEvent;
import java.awt.event.WindowListener;

public class ObjsFrame extends JFrame {
    private String[] columnNames = {
                                   "Type", "Subtype", "Facing/volume",
                                   "Polygon", "x", "y", "z"
    };

    private Model model;
    private int levelIndex;

    private JTable table;
    private JScrollPane scroller;
    private ObjsTableModel tableModel;
    private ObjsEntry bufferedEntry;
    private ObjsEntry duplicatedEntry;
    private boolean hasBuffered;

    private int clickCount;

    JMenuBar mnuBar = new JMenuBar();
    JMenu mnuCommands = new JMenu();
    JMenuItem mnuCommandsCopy = new JMenuItem();
    JMenuItem mnuCommandsPaste = new JMenuItem();
    JMenuItem mnuCommandsDuplicate = new JMenuItem();
    JMenuItem mnuCommandsEditFlags = new JMenuItem();
    JMenuItem mnuCommandsCenterObject = new JMenuItem();
    JMenuItem mnuCommandsAdd = new JMenuItem();
    JMenuItem mnuCommandsRemove = new JMenuItem();
    JMenuItem mnuCommandsZeroZs = new JMenuItem();
    JMenuItem mnuCommandsZByType = new JMenuItem();
    JMenuItem mnuCommandsZBySubtype = new JMenuItem();
    JMenuItem mnuCommandsChangeAll = new JMenuItem();

    public ObjsFrame(Model model, int levelIndex) {
        this.model = model;
        this.levelIndex = levelIndex;
        try {
            jbInit();
        } catch (Exception exception) {
            exception.printStackTrace();
        }
    }

    private void jbInit() throws Exception {
        getContentPane().setLayout(new BorderLayout());
        tableModel = new ObjsTableModel(columnNames, model, levelIndex);
        tableModel.addTableModelListener(new ObjsFrame.ObjsTableModelListener());
        table = new JTable(tableModel);
        table.setSurrendersFocusOnKeystroke(true);
        table.getTableHeader().setReorderingAllowed(false);

        updateComboBoxes();

        scroller = new JScrollPane(table);

        table.getTableHeader().addMouseListener(new MouseListener() {
            public void mouseExited(MouseEvent m) {}

            public void mouseEntered(MouseEvent m) {}

            public void mouseReleased(MouseEvent m) {}

            public void mousePressed(MouseEvent m) {}

            public void mouseClicked(MouseEvent m) {
                Point mousePoint = new Point(m.getX(), m.getY());
                int header = table.getTableHeader().columnAtPoint(mousePoint);

                clickCount++;
                boolean reverseOrder = false;
                if ((clickCount % 2) == 0) reverseOrder = true;
                tableModel.sort(header, reverseOrder);
            }
        });

        this.getContentPane().add(scroller);

        clickCount = 0;

        this.setJMenuBar(mnuBar);
        mnuCommands.setText("Commands");
        mnuCommandsCopy.setText("Copy (ctrl+c)");
        mnuCommandsCopy.addActionListener(new ObjsFrame_mnuCommandsCopy_actionAdapter(this));
        mnuCommandsPaste.setText("Paste (ctrl+v)");
        mnuCommandsPaste.addActionListener(new ObjsFrame_mnuCommandsPaste_actionAdapter(this));
        mnuCommandsDuplicate.setText("Duplicate (ctrl+d)");
        mnuCommandsDuplicate.addActionListener(new ObjsFrame_mnuCommandsDuplicate_actionAdapter(this));
        mnuCommandsEditFlags.setText("Edit Flags");
        mnuCommandsEditFlags.addActionListener(new ObjsFrame_mnuCommandsEditFlags_actionAdapter(this));
        mnuCommandsCenterObject.setText("Center in polygon (alt+c)");
        mnuCommandsCenterObject.addActionListener(new ObjsFrame_mnuCommandsCenterObject_actionAdapter(this));
        mnuCommandsAdd.setText("Add object (ctrl+n)");
        mnuCommandsAdd.addActionListener(new ObjsFrame_mnuCommandsAdd_actionAdapter(this));
        mnuCommandsRemove.setText("Remove object (delete)");
        mnuCommandsRemove.addActionListener(new ObjsFrame_mnuCommandsRemove_actionAdapter(this));
        mnuCommandsZeroZs.setText("Set all Zs to 0 (alt+z)");
        mnuCommandsZeroZs.addActionListener(new ObjsFrame_mnuCommandsZeroZs_actionAdapter(this));
        mnuCommandsZByType.setText("Set all Z by type");
        mnuCommandsZByType.addActionListener(new ObjsFrame_mnuCommandsZByType_actionAdapter(this));
        mnuCommandsZBySubtype.setText("Set all Z by subtype");
        mnuCommandsZBySubtype.addActionListener(new ObjsFrame_mnuCommandsZBySubtype_actionAdapter(this));
        mnuCommandsChangeAll.setText("Change all ...");
        mnuCommandsChangeAll.addActionListener(new ObjsFrame_mnuCommandsChangeAll_actionAdapter(this));
        mnuBar.add(mnuCommands);
        mnuCommands.add(mnuCommandsCopy);
        mnuCommands.add(mnuCommandsPaste);
        mnuCommands.add(mnuCommandsDuplicate);
        mnuCommands.add(mnuCommandsCenterObject);
        mnuCommands.add(mnuCommandsAdd);
        mnuCommands.add(mnuCommandsRemove);
        mnuCommands.add(mnuCommandsZeroZs);
        mnuCommands.add(mnuCommandsZByType);
        mnuCommands.add(mnuCommandsZBySubtype);
        mnuCommands.add(mnuCommandsEditFlags);
        mnuCommands.add(mnuCommandsChangeAll);

        bufferedEntry = new ObjsEntry(model.getObjsChunk(levelIndex));
        duplicatedEntry = new ObjsEntry(model.getObjsChunk(levelIndex));
        hasBuffered = false;

        table.addKeyListener(new java.awt.event.KeyAdapter() { // Center object
            public void keyPressed(KeyEvent e) {
                if(e.getKeyCode() == KeyEvent.VK_C && e.isAltDown()){
                    centerObject();
                }
            }
        });

        table.addKeyListener(new java.awt.event.KeyAdapter() { // Create new object
            public void keyPressed(KeyEvent e) {
                if(e.getKeyCode() == KeyEvent.VK_N && e.isControlDown()){
                    addObject();
                }
            }
        });

        table.addKeyListener(new java.awt.event.KeyAdapter() { // Remove object
            public void keyPressed(KeyEvent e) {
                if(e.getKeyCode() == KeyEvent.VK_DELETE){
                    removeObject();
                }
            }
        });

        table.addKeyListener(new java.awt.event.KeyAdapter() { // Copy object
            public void keyPressed(KeyEvent e) {
                if(e.getKeyCode() == KeyEvent.VK_C && e.isControlDown()){
                    copyObject();
                }
            }
        });

        table.addKeyListener(new java.awt.event.KeyAdapter() { // Paste object
            public void keyPressed(KeyEvent e) {
                if(e.getKeyCode() == KeyEvent.VK_V && e.isControlDown()){
                    pasteObject();
                }
            }
        });

        table.addKeyListener(new java.awt.event.KeyAdapter() { // Duplicate object
            public void keyPressed(KeyEvent e) {
                if(e.getKeyCode() == KeyEvent.VK_D && e.isControlDown()){
                    duplicateObject();
                }
            }
        });

        table.addKeyListener(new java.awt.event.KeyAdapter() { // Zs to 0
            public void keyPressed(KeyEvent e) {
                if(e.getKeyCode() == KeyEvent.VK_Z && e.isAltDown()){
                    zeroZs();
                }
            }
        });

        this.addWindowListener(new WindowListener(){
            public void windowDeactivated(WindowEvent e){}
            public void windowActivated(WindowEvent e) {}
            public void windowDeiconified(WindowEvent e) {}
            public void windowIconified(WindowEvent e) {}
            public void windowClosed(WindowEvent e) {}
            public void windowClosing(WindowEvent e) {
                closeTable();
            }
            public void windowOpened(WindowEvent e) {}
        });

        setSize(525, 440);
        setTitle("Objects - " +
                      model.getLevelSelectName());
        StaticFunctions.centerFrame(this);
        setVisible(true);

    }

    // Called when new item is created, make sure it uses a combo box for editing.
    public void updateComboBoxes(){
        TableColumn typeCol = table.getColumnModel().getColumn(0);
        TableColumn subTypeCol = table.getColumnModel().getColumn(1);
        JComboBox typeBox = new JComboBox();

        for (int i = 0; i < StaticFunctions.OBJS_TYPES.length-1; i++) {
            typeBox.addItem(StaticFunctions.OBJS_TYPES[i]);
        }

        typeCol.setCellEditor(new DefaultCellEditor(typeBox));
        subTypeCol.setCellEditor(new DynamicCellEditor());
    }

    private void closeTable(){
        model.removeChildFrame(this);
    }

    private void copyObject(){
        try{
            hasBuffered = true;
            ObjsEntry from = ((ObjsEntry)tableModel.getData().get(table.getSelectedRow()));
            bufferedEntry.setType(from.getType());
            bufferedEntry.setSubtype(from.getSubtype());
            bufferedEntry.setFacing(from.getFacing());
        } catch(Exception e){
            return;
        }
    }

    private void pasteObject(){
        try{
            if (hasBuffered) {
                int row = table.getSelectedRow();
                if (bufferedEntry.getType() == 5) // sound
                    return;
                ((ObjsEntry)tableModel.getData().get(row)).setType(bufferedEntry.getType());
                ((ObjsEntry)tableModel.getData().get(row)).setSubtype(bufferedEntry.
                        getSubtype());
                ((ObjsEntry)tableModel.getData().get(row)).setFacing(bufferedEntry.getFacing());
                tableModel.fireTableRowsUpdated(row, row);
            } else return;
        } catch(Exception e){
            return;
        }
    }

    private void duplicateObject(){
        try{
            duplicatedEntry = new ObjsEntry(model.getObjsChunk(levelIndex));
            int row = table.getSelectedRow();
            if (((ObjsEntry)tableModel.getData().get(row)).getType() == 5) // sound
                return;
            duplicatedEntry.setType(((ObjsEntry)tableModel.getData().get(row)).getType());
            duplicatedEntry.setSubtype(((ObjsEntry)tableModel.getData().get(row)).getSubtype());
            duplicatedEntry.setFacing(((ObjsEntry)tableModel.getData().get(row)).getFacing());
            duplicatedEntry.setPolygon(((ObjsEntry)tableModel.getData().get(row)).getPolygon());
            duplicatedEntry.setX(((ObjsEntry)tableModel.getData().get(row)).getX());
            duplicatedEntry.setY(((ObjsEntry)tableModel.getData().get(row)).getY());
            duplicatedEntry.setZ(((ObjsEntry)tableModel.getData().get(row)).getZ());
            model.addObject(levelIndex, duplicatedEntry);
            tableModel.loadTable();
        } catch(Exception e){
            return;
        }
    }

    private void zeroZs(){
        try{
            for(int i = 0; i < tableModel.getData().size(); i++){
                ((ObjsEntry)tableModel.getData().get(i)).setZ(0);
                tableModel.fireTableCellUpdated(i, 6);
            }
        } catch(Exception e){
            return;
        }
    }

    private void ZByType(){
        try{
            int type = ((ObjsEntry)tableModel.getData().get(table.getSelectedRow())).getType();
            double z = ((ObjsEntry)tableModel.getData().get(table.getSelectedRow())).getZ();
            for(int i = 0; i < tableModel.getData().size(); i++){
                if( ((ObjsEntry)tableModel.getData().get(i)).getType() == type){
                    ((ObjsEntry)tableModel.getData().get(i)).setZ(z);
                }
            }
            tableModel.fireTableDataChanged();
        }catch(Exception e){
            return;
        }
    }

    private void ZBySubtype(){
        try{
            int subtype = ((ObjsEntry)tableModel.getData().get(table.getSelectedRow())).getSubtype();
            int type = ((ObjsEntry)tableModel.getData().get(table.getSelectedRow())).getType();
            double z = ((ObjsEntry)tableModel.getData().get(table.getSelectedRow())).getZ();
            for(int i = 0; i < tableModel.getData().size(); i++){
                if( ((ObjsEntry)tableModel.getData().get(i)).getSubtype() == subtype &&
                ((ObjsEntry)tableModel.getData().get(i)).getType() == type){
                    ((ObjsEntry)tableModel.getData().get(i)).setZ(z);
                }
            }
            tableModel.fireTableDataChanged();
        }catch(Exception e){
            return;
        }
    }

    private void removeObject(){
        try{
            tableModel.removeEntry(table.getSelectedRow());
            tableModel.fireTableRowsDeleted(table.getSelectedRow(),
                                            table.getSelectedRow());
        } catch(Exception e){
            return;
        }
    }

    private void addObject(){
        try{
            ObjsEntry newEntry = model.addObject(levelIndex);
            tableModel.addEntry(newEntry);
            updateComboBoxes();
            tableModel.fireTableRowsInserted(tableModel.getData().size(),
                                             tableModel.getData().size());
        } catch(Exception e){
            return;
        }
    }

    private void showFlagsFrame(){
        try{
            ArrayList data = tableModel.getData();
            ObjsFlagsFrame objFF = new ObjsFlagsFrame(model, levelIndex,
                    (ObjsEntry)data.get(table.getSelectedRow()));
            model.addChildFrame(objFF);
        } catch(Exception e) {}
    }

    private void centerObject(){
        try{
            ArrayList theData = tableModel.getData();
            int row = table.getSelectedRow();
            double dblX = model.getPolygonCenterX(levelIndex,
                                                  ((ObjsEntry)theData.get(row)).getPolygon());
            double dblY = model.getPolygonCenterY(levelIndex,
                                                  ((ObjsEntry)theData.get(row)).getPolygon());
            String strX = String.valueOf(dblX);
            String strY = String.valueOf(dblY);
            tableModel.setValueAt(strX, row, 4);
            tableModel.setValueAt(strY, row, 5);
            tableModel.fireTableCellUpdated(row, 4);
            tableModel.fireTableCellUpdated(row, 5);
        } catch(Exception e) {}
    }

    public class DynamicCellEditor extends AbstractCellEditor implements TableCellEditor {
        JComboBox dynamicBox = new JComboBox();

        public Component getTableCellEditorComponent(JTable table, Object value,
                                                     boolean isSelected, int row,
                                                     int column) {
            String type = "";
            try {
                type = (String) table.getValueAt(row, column-1);
            } catch (Exception e) {System.out.println(e);}

            if(type.equals("Monster")){
                dynamicBox = new JComboBox();
                for (int i = 0; i < StaticFunctions.OBJS_SUBTYPES[0].length; i++) {
                    dynamicBox.addItem(StaticFunctions.OBJS_SUBTYPES[0][i]);
                }
            }
            else if (type.equals("Scenery")) {
                dynamicBox = new JComboBox();
                for(int i = 0; i < StaticFunctions.scenery.length; i++){
                    dynamicBox.addItem(StaticFunctions.scenery[i]);
                }
            }
            else if (type.equals("Item")) {
                dynamicBox = new JComboBox();
                for (int i = 0; i < StaticFunctions.OBJS_SUBTYPES[2].length; i++) {
                    dynamicBox.addItem(StaticFunctions.OBJS_SUBTYPES[2][i]);
                }
            }
            else if (type.equals("Player")) {
                dynamicBox = new JComboBox();
                for (int i = 0; i < StaticFunctions.OBJS_SUBTYPES[3].length; i++) {
                    dynamicBox.addItem(StaticFunctions.OBJS_SUBTYPES[3][i]);
                }
            }
            else if (type.equals("Goal")) {
                dynamicBox = new JComboBox();
                for (int i = 0; i < StaticFunctions.OBJS_SUBTYPES[4].length; i++) {
                    dynamicBox.addItem(StaticFunctions.OBJS_SUBTYPES[4][i]);
                }
            }
            else if (type.equals("Sound")) {
                dynamicBox = new JComboBox();
                for (int i = 0; i < StaticFunctions.OBJS_SUBTYPES[5].length; i++) {
                    dynamicBox.addItem(StaticFunctions.OBJS_SUBTYPES[5][i]);
                }
            }

            return dynamicBox;
        }

        public Object getCellEditorValue() {
            return (String)dynamicBox.getSelectedItem();
        }
    }

    public void mnuCommandsEditFlags_actionPerformed(ActionEvent e) { // Edit flags menu
        showFlagsFrame();
    }

    public void mnuCommandsCenterObject_actionPerformed(ActionEvent e) { // Center object menu
        centerObject();
    }

    public void mnuCommandsAdd_actionPerformed(ActionEvent e) { // Add menu
        addObject();
    }

    public void mnuCommandsRemove_actionPerformed(ActionEvent e) { // Remove menu
        removeObject();
    }

    public void mnuCommandsCopy_actionPerformed(ActionEvent e){ // Copy menu
        copyObject();
    }

    public void mnuCommandsPaste_actionPerformed(ActionEvent e){ // Paste menu
        pasteObject();
    }

    public void mnuCommandsDuplicate_actionPerformed(ActionEvent e){ // Duplicate menu
        duplicateObject();
    }

    public void mnuCommandsZeroZs_actionPerformed(ActionEvent e){ // Zero Zs menu
        zeroZs();
    }

    public void mnuCommandsZByType_actionPerformed(ActionEvent e){ // Z by type menu
        ZByType();
    }

    public void mnuCommandsZBySubtype_actionPerformed(ActionEvent e){ // Z by subtype menu
        ZBySubtype();
    }

    public void mnuCommandsChangeAll_actionPerformed(ActionEvent e){ // Change all menu
        ObjsChangeFrame ch = new ObjsChangeFrame(model, table, tableModel, levelIndex);
        model.addChildFrame(ch);
    }

    class ObjsFrame_mnuCommandsCopy_actionAdapter implements ActionListener {
        private ObjsFrame adaptee;
        ObjsFrame_mnuCommandsCopy_actionAdapter(ObjsFrame adaptee) {
            this.adaptee = adaptee;
        }

        public void actionPerformed(ActionEvent e) {
            adaptee.mnuCommandsCopy_actionPerformed(e);
        }
    }

    class ObjsFrame_mnuCommandsPaste_actionAdapter implements ActionListener {
        private ObjsFrame adaptee;
        ObjsFrame_mnuCommandsPaste_actionAdapter(ObjsFrame adaptee) {
            this.adaptee = adaptee;
        }

        public void actionPerformed(ActionEvent e) {
            adaptee.mnuCommandsPaste_actionPerformed(e);
        }
    }

    class ObjsFrame_mnuCommandsDuplicate_actionAdapter implements ActionListener {
        private ObjsFrame adaptee;
        ObjsFrame_mnuCommandsDuplicate_actionAdapter(ObjsFrame adaptee) {
            this.adaptee = adaptee;
        }

        public void actionPerformed(ActionEvent e) {
            adaptee.mnuCommandsDuplicate_actionPerformed(e);
        }
    }

    class ObjsFrame_mnuCommandsEditFlags_actionAdapter implements ActionListener {
        private ObjsFrame adaptee;
        ObjsFrame_mnuCommandsEditFlags_actionAdapter(ObjsFrame adaptee) {
            this.adaptee = adaptee;
        }

        public void actionPerformed(ActionEvent e) {
            adaptee.mnuCommandsEditFlags_actionPerformed(e);
        }
    }

    class ObjsFrame_mnuCommandsCenterObject_actionAdapter implements ActionListener {
        private ObjsFrame adaptee;
        ObjsFrame_mnuCommandsCenterObject_actionAdapter(ObjsFrame adaptee) {
            this.adaptee = adaptee;
        }

        public void actionPerformed(ActionEvent e) {
            adaptee.mnuCommandsCenterObject_actionPerformed(e);
        }
    }

    class ObjsFrame_mnuCommandsAdd_actionAdapter implements ActionListener {
        private ObjsFrame adaptee;
        ObjsFrame_mnuCommandsAdd_actionAdapter(ObjsFrame adaptee) {
            this.adaptee = adaptee;
        }

        public void actionPerformed(ActionEvent e) {
            adaptee.mnuCommandsAdd_actionPerformed(e);
        }
    }


    class ObjsFrame_mnuCommandsRemove_actionAdapter implements ActionListener {
        private ObjsFrame adaptee;
        ObjsFrame_mnuCommandsRemove_actionAdapter(ObjsFrame adaptee) {
            this.adaptee = adaptee;
        }

        public void actionPerformed(ActionEvent e) {
            adaptee.mnuCommandsRemove_actionPerformed(e);
        }
    }

    class ObjsFrame_mnuCommandsZeroZs_actionAdapter implements ActionListener {
        private ObjsFrame adaptee;
        ObjsFrame_mnuCommandsZeroZs_actionAdapter(ObjsFrame adaptee) {
            this.adaptee = adaptee;
        }

        public void actionPerformed(ActionEvent e) {
            adaptee.mnuCommandsZeroZs_actionPerformed(e);
        }
    }

    class ObjsFrame_mnuCommandsZByType_actionAdapter implements ActionListener {
        private ObjsFrame adaptee;
        ObjsFrame_mnuCommandsZByType_actionAdapter(ObjsFrame adaptee) {
            this.adaptee = adaptee;
        }

        public void actionPerformed(ActionEvent e) {
            adaptee.mnuCommandsZByType_actionPerformed(e);
        }
    }

    class ObjsFrame_mnuCommandsZBySubtype_actionAdapter implements ActionListener {
        private ObjsFrame adaptee;
        ObjsFrame_mnuCommandsZBySubtype_actionAdapter(ObjsFrame adaptee) {
            this.adaptee = adaptee;
        }

        public void actionPerformed(ActionEvent e) {
            adaptee.mnuCommandsZBySubtype_actionPerformed(e);
        }
    }

    class ObjsFrame_mnuCommandsChangeAll_actionAdapter implements ActionListener {
        private ObjsFrame adaptee;
        ObjsFrame_mnuCommandsChangeAll_actionAdapter(ObjsFrame adaptee) {
            this.adaptee = adaptee;
        }

        public void actionPerformed(ActionEvent e) {
            adaptee.mnuCommandsChangeAll_actionPerformed(e);
        }
    }

    class ObjsTableModelListener implements TableModelListener {
        public void tableChanged(TableModelEvent evt) {
            if (evt.getType() == TableModelEvent.UPDATE) {
                int row = evt.getFirstRow();
                try{
                    table.setRowSelectionInterval(row, row);
                } catch(Exception e){}
            }
        }
    }
}
