/*
  Copyright 2007 Darren Watts

  This file is part of JUICE.

  JUICE is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  JUICE is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with JUICE; if not, write to the Free Software
  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

package src.gui;

import javax.swing.table.AbstractTableModel;
import src.backend.StaticFunctions;
import src.backend.*;
import java.util.ArrayList;

public class ObjsTableModel extends AbstractTableModel{
    public static final int OBJS_TYPE = 0;
    public static final int OBJS_SUBTYPE = 1;
    public static final int OBJS_FACING = 2;
    public static final int OBJS_POLYGON = 3;
    public static final int OBJS_X = 4;
    public static final int OBJS_Y = 5;
    public static final int OBJS_Z = 6;

    public static final int INF_AMOUNT = 65535;
    public static final double MAX_COORD_NUM = 31.999;
    public static final double MIN_COORD_NUM = -32;

    private String[] colNames;
    private ArrayList data;
    private Model model;
    private int levelIndex;
    private boolean lastReverse;
    private int lastSortColumn;

    public ObjsTableModel(String[] colNames, Model model, int levelIndex) {
        this.colNames = colNames;
        this.model = model;
        this.levelIndex = levelIndex;
        lastReverse = false;
        lastSortColumn = 0;
        //data = new ObjsEntry[model.getNumObjs(levelIndex)];
        loadTable();
    }

    public void loadTable(){
        data = new ArrayList();
        for (int i = 0; i < model.getNumObjs(levelIndex); i++) {
            data.add(i, this.model.getObjsItem(levelIndex, i));
        }
        sort(lastSortColumn, lastReverse);
    }

    public String getColumnName(int col){
        return colNames[col];
    }

    public boolean isCellEditable(int row, int column) {
        try{
            if (column == OBJS_TYPE &&
                ((String) getValueAt(row, column)).equals("Sound"))
                return false;
            return true;
        } catch(Exception e){
            return false;
        }
    }

    public Object getValueAt(int row, int column) {
        switch (column) {
        case OBJS_TYPE:
            try{
                return StaticFunctions.OBJS_TYPES[((ObjsEntry)data.get(row)).getType()];
            }catch(Exception e) { return "0"; }
        case OBJS_SUBTYPE:
            try{
                return StaticFunctions.OBJS_SUBTYPES[((ObjsEntry)data.get(row)).getType()][((ObjsEntry)data.get(row)).getSubtype()];
            }catch(Exception e) { return "0"; }
        case OBJS_FACING:
            try{
                return new Integer(((ObjsEntry)data.get(row)).getFacing());
            }catch(Exception e) { return "0"; }
        case OBJS_POLYGON:
            try{
                return new Integer(((ObjsEntry)data.get(row)).getPolygon());
            }catch(Exception e) { return "0"; }
        case OBJS_X:
            try{
                return new Double(((ObjsEntry)data.get(row)).getX());
            }catch(Exception e) { return "0"; }
        case OBJS_Y:
            try{
                return new Double(((ObjsEntry)data.get(row)).getY());
            }catch(Exception e) { return "0"; }
        case OBJS_Z:
            try{
                return new Double(((ObjsEntry)data.get(row)).getZ());
            }catch(Exception e) { return "0"; }
        default:
            return new Object();
        }
   }

   private int getTypeNum(String str){
       for(int i = 0; i < StaticFunctions.OBJS_TYPES.length; i++){
           if(str.equals(StaticFunctions.OBJS_TYPES[i]))
               return i;
       }
       return -1;
   }

   private int getSubTypeNum(int row, String str){
       for(int i = 0; i < StaticFunctions.OBJS_SUBTYPES[row].length; i++){
           if(str.equals(StaticFunctions.OBJS_SUBTYPES[row][i]))
               return i;
       }
       return -1;
   }

   public void setValueAt(Object value, int row, int column) {
       int intValue = 0;
       double dblVal = 0;
       try{
           intValue = Integer.parseInt((String) value);
       } catch(Exception e) {}
       try {
       } catch (Exception ex) {}
       try{
           switch (column) {
           case OBJS_TYPE:
               intValue = getTypeNum((String)value);
               ((ObjsEntry)data.get(row)).setType((short)intValue);
               break;
           case OBJS_SUBTYPE:
               intValue = getSubTypeNum(((ObjsEntry)data.get(row)).getType(), (String)value);
               ((ObjsEntry)data.get(row)).setSubtype((short)intValue);
               break;
           case OBJS_FACING:
               int highNum = 0;
               int lowNum = 0;
               String type = (String)getValueAt(row, OBJS_TYPE);
               if(type.equals("Sound")){
                   highNum = 180;
                   lowNum = -1 * (model.getPolyNumber(levelIndex));
               }
               else{
                   highNum = 360;
                   lowNum = 0;
               }
               if (intValue > highNum) ((ObjsEntry)data.get(row)).setFacing((short) highNum);
               else if (intValue < lowNum) ((ObjsEntry)data.get(row)).setFacing((short) lowNum);
               else ((ObjsEntry)data.get(row)).setFacing((short) intValue);
               break;
           case OBJS_POLYGON:
               if (intValue > model.getPolyNumber(levelIndex) - 1) ((ObjsEntry)data.get(row)).setPolygon((short)(model.getPolyNumber(levelIndex) - 1));
               else if (intValue < 0) ((ObjsEntry)data.get(row)).setPolygon((short) 0);
               else ((ObjsEntry)data.get(row)).setPolygon((short) intValue);
               break;
           case OBJS_X:
               dblVal = Double.parseDouble((String)value);
               if (dblVal > MAX_COORD_NUM) ((ObjsEntry)data.get(row)).setX(MAX_COORD_NUM);
               else if (dblVal < MIN_COORD_NUM) ((ObjsEntry)data.get(row)).setX(MIN_COORD_NUM);
               else ((ObjsEntry)data.get(row)).setX(dblVal);
               break;
           case OBJS_Y:
               dblVal = Double.parseDouble((String)value);
               if (dblVal > MAX_COORD_NUM) ((ObjsEntry)data.get(row)).setY(MAX_COORD_NUM);
               else if (dblVal < MIN_COORD_NUM) ((ObjsEntry)data.get(row)).setY(MIN_COORD_NUM);
               else ((ObjsEntry)data.get(row)).setY(dblVal);
               break;
           case OBJS_Z:
               dblVal = Double.parseDouble((String)value);
               if (dblVal > MAX_COORD_NUM) ((ObjsEntry)data.get(row)).setZ(MAX_COORD_NUM);
               else if (dblVal < MIN_COORD_NUM) ((ObjsEntry)data.get(row)).setZ(MIN_COORD_NUM);
               else ((ObjsEntry)data.get(row)).setZ(dblVal);
               break;
           default:
               break;
           }
       }catch(Exception e) {System.out.println(e);}
       fireTableCellUpdated(row, column);
   }

   public int getRowCount() {
       return data.size();
   }

   public int getColumnCount(){
       return colNames.length;
   }

   public void sort(int header, boolean reverse){
       lastSortColumn = header;
       lastReverse = reverse;
       switch (header) {
       case OBJS_TYPE:
           sortBy(reverse, "type");
           break;
       case OBJS_SUBTYPE:
           sortBy(reverse, "subtype");
           break;
       case OBJS_FACING:
           sortBy(reverse, "facing");
           break;
       case OBJS_POLYGON:
           sortBy(reverse, "polygon");
           break;
       case OBJS_X:
           sortBy(reverse, "x");
           break;
       case OBJS_Y:
           sortBy(reverse, "y");
           break;
       case OBJS_Z:
           sortBy(reverse, "z");
           break;
       }
       this.fireTableDataChanged();
   }

   private void sortBy(boolean reverse, String type){
       mergeSort(0, data.size(), reverse, type);
   }

   private void mergeSort(int first, int n, boolean reverse, String type){
       int n1; // Size of the first half of the array
       int n2; // Size of the second half of the array

       if (n > 1) {
           // Compute sizes of the two halves
           n1 = n / 2;
           n2 = n - n1;

           mergeSort(first, n1, reverse, type); // Sort data[first] through data[first+n1-1]
           mergeSort(first + n1, n2, reverse, type); // Sort data[first+n1] to the end

           // Merge the two sorted halves.
           merge(first, n1, n2, reverse, type);
       }
   }

   private void merge(int first, int n1, int n2, boolean reverse, String type) {
       ObjsEntry[] tempEntry = new ObjsEntry[n1 + n2];
       int copied = 0; // Number of elements copied from data to temp
       int copied1 = 0; // Number copied from the first half of data
       int copied2 = 0; // Number copied from the second half of data
       int i; // Array index to copy from temp back into data

       // Merge elements, copying from two halves of data to the temporary array.
       while ((copied1 < n1) &&
              (copied2 < n2)) {
           if (compare(reverse, type, first, copied1, copied2, n1)) {
               tempEntry[copied++] = (ObjsEntry)data.get(first + copied1++);
           } else {
               tempEntry[copied++] = (ObjsEntry)data.get(first + n1 + copied2++);
           }
       }

       // Copy any remaining entries in the left and right subarrays.
       while (copied1 < n1) {
           tempEntry[copied++] = (ObjsEntry)data.get(first + copied1++);
       } while (copied2 < n2) {
           tempEntry[copied++] = (ObjsEntry)data.get(first + n1 + copied2++);
       }
       // Copy from temp back to the data array.
       for (i = 0; i < n1 + n2; i++) {
           data.set(first + i, tempEntry[i]);
       }
   }

   private boolean compare(boolean reverse, String type, int first,
                           int copied1, int copied2, int n1) {
       try{
           if (!reverse && type.equals("type")) {
               return ((ObjsEntry)data.get(first + copied1)).getType() <
                       ((ObjsEntry)data.get(first + n1 + copied2)).getType();
           } else if (reverse && type.equals("type")) {
               return ((ObjsEntry)data.get(first + copied1)).getType() >
                       ((ObjsEntry)data.get(first + n1 + copied2)).getType();
           } else if (!reverse && type.equals("subtype")) {
               return ((ObjsEntry)data.get(first + copied1)).getSubtype() <
                       ((ObjsEntry)data.get(first + n1 + copied2)).getSubtype();
           } else if (reverse && type.equals("subtype")) {
               return ((ObjsEntry)data.get(first + copied1)).getSubtype() >
                       ((ObjsEntry)data.get(first + n1 + copied2)).getSubtype();
           } else if (!reverse && type.equals("facing")) {
               return ((ObjsEntry)data.get(first + copied1)).getFacing() <
                       ((ObjsEntry)data.get(first + n1 + copied2)).getFacing();
           } else if (reverse && type.equals("facing")) {
               return ((ObjsEntry)data.get(first + copied1)).getFacing() >
                       ((ObjsEntry)data.get(first + n1 + copied2)).getFacing();
           } else if (!reverse && type.equals("polygon")) {
               return ((ObjsEntry)data.get(first + copied1)).getPolygon() <
                       ((ObjsEntry)data.get(first + n1 + copied2)).getPolygon();
           } else if (reverse && type.equals("polygon")) {
               return ((ObjsEntry)data.get(first + copied1)).getPolygon() >
                       ((ObjsEntry)data.get(first + n1 + copied2)).getPolygon();
           } else if (!reverse && type.equals("x")) {
               return ((ObjsEntry)data.get(first + copied1)).getX() <
                       ((ObjsEntry)data.get(first + n1 + copied2)).getX();
           } else if (reverse && type.equals("x")) {
               return ((ObjsEntry)data.get(first + copied1)).getX() >
                       ((ObjsEntry)data.get(first + n1 + copied2)).getX();
           } else if (!reverse && type.equals("y")) {
               return ((ObjsEntry)data.get(first + copied1)).getY() <
                       ((ObjsEntry)data.get(first + n1 + copied2)).getY();
           } else if (reverse && type.equals("y")) {
               return ((ObjsEntry)data.get(first + copied1)).getY() >
                       ((ObjsEntry)data.get(first + n1 + copied2)).getY();
           } else if (!reverse && type.equals("z")) {
               return ((ObjsEntry)data.get(first + copied1)).getZ() <
                       ((ObjsEntry)data.get(first + n1 + copied2)).getZ();
           } else if (reverse && type.equals("z")) {
               return ((ObjsEntry)data.get(first + copied1)).getZ() >
                       ((ObjsEntry)data.get(first + n1 + copied2)).getZ();
           } else return false;
       } catch(Exception e){ return false; }
   }

   public ArrayList getData(){
       return data;
   }

   public void addEntry(ObjsEntry newEntry){
       try{
           data.add(newEntry);
       } catch(Exception e) {}
   }

   public void removeEntry(int row){
       try{
           ObjsEntry gone = (ObjsEntry)data.get(row);
           model.removeEntry(levelIndex, gone);
           loadTable();
       } catch(Exception e) {}
   }

}
