/*
  Copyright 2007 Darren Watts

  This file is part of JUICE.

  JUICE is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  JUICE is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with JUICE; if not, write to the Free Software
  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */

package src.gui;

import java.awt.*;
import javax.swing.*;
import javax.swing.table.TableColumn;
import javax.swing.event.TableModelListener;
import javax.swing.event.TableModelEvent;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import javax.swing.table.TableCellRenderer;
import java.awt.event.WindowListener;
import java.awt.event.WindowEvent;
import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;
import src.backend.*;

public class PlacFrame extends JFrame {
    private String[] columnNames = {
                                   "Item", "Initial", "Minimum",
                                   "Maximum", "Random Location",
                                   "Random Number Available",
                                   "Infinite Available",
                                   "Random Appearance Percent"
    };

    public static final int ITEMS = 0;
    public static final int MONSTERS = 1;

    private Model model;
    private int levelIndex;
    private int type;

    private JTable table;
    private JScrollPane scroller;
    private PlacTableModel tableModel;
    private PlacEntry[] redoTable;

    private int clickCount;

    JMenuBar mnuBar = new JMenuBar();
    JMenu mnuCommands = new JMenu();
    JMenuItem mnuCommandsUndo = new JMenuItem();
    JMenuItem mnuCommandsClearKeepInitial = new JMenuItem();
    JMenuItem mnuCommandsClearAll = new JMenuItem();
    JMenuItem mnuCommandsAutoFill = new JMenuItem();
    JMenuItem mnuCommandsZeroRow = new JMenuItem();

    public PlacFrame(Model model, int levelIndex, int type) {
        this.model = model;
        this.levelIndex = levelIndex;
        this.type = type;
        if (type == ITEMS){
            columnNames[0] = "Items";
            setTitle("Item Parameters - " +
                     model.getLevelSelectName());
        }
        else {
            columnNames[0] = "Monsters";
            setTitle("Monster Parameters - " +
                   model.getLevelSelectName());
        }


        try {
            jbInit();
        } catch (Exception exception) {
            exception.printStackTrace();
        }
    }

    private void jbInit() throws Exception {
        getContentPane().setLayout(new BorderLayout());
        if (type == ITEMS)
            tableModel = new PlacTableModel(columnNames, model, levelIndex, 36,
                                            0, 36);
        else tableModel = new PlacTableModel(columnNames, model, levelIndex, 47,
                                             36, 83);
        tableModel.addTableModelListener(new PlacFrame.
                                         PlacTableModelListener());
        table = new JTable(tableModel);
        table.setSurrendersFocusOnKeystroke(true);
        table.getTableHeader().setReorderingAllowed(false);

        TableColumn randomCol = table.getColumnModel().getColumn(4);
        TableColumn infAvailCol = table.getColumnModel().getColumn(6);
        JCheckBox chkRandom = new JCheckBox();
        JCheckBox chkInfAvail = new JCheckBox();

        randomCol.setCellEditor(new DefaultCellEditor(chkRandom));
        infAvailCol.setCellEditor(new DefaultCellEditor(chkInfAvail));
        randomCol.setCellRenderer(new TableCellRenderer() {
            public Component getTableCellRendererComponent(JTable table,
                    Object value,
                    boolean isSelected,
                    boolean hasFocus, int row,
                    int column) {

                JCheckBox box = new JCheckBox();
                box.setBackground(Color.white);
                try {
                    if (((Boolean)value).booleanValue()) {
                        box.setSelected(true);
                        return box;
                    } else {
                        box.setSelected(false);
                        return box;
                    }
                } catch (Exception e) {}
                return new JCheckBox();
            }
        });

        infAvailCol.setCellRenderer(new TableCellRenderer() {
            public Component getTableCellRendererComponent(JTable table,
                    Object value,
                    boolean isSelected,
                    boolean hasFocus, int row,
                    int column) {
                JCheckBox box = new JCheckBox();
                box.setBackground(Color.white);
                try {
                    if (((Boolean)value).booleanValue()) {
                        box.setSelected(true);
                        return box;
                    } else {
                        box.setSelected(false);
                        return box;
                    }
                } catch (Exception e) {}
                return new JCheckBox();
            }
        });
        scroller = new JScrollPane(table);
        //scroller.setBounds(new Rectangle(9, 14, 499, 344));

        table.getTableHeader().addMouseListener(new MouseListener() {
            public void mouseExited(MouseEvent m) {}
            public void mouseEntered(MouseEvent m) {}
            public void mouseReleased(MouseEvent m) {}
            public void mousePressed(MouseEvent m) {}
            public void mouseClicked(MouseEvent m) {
                Point mousePoint = new Point(m.getX(), m.getY());
                int header = table.getTableHeader().columnAtPoint(mousePoint);

                clickCount++;
                boolean reverseOrder = false;
                if ((clickCount % 2) == 0) reverseOrder = true;
                tableModel.sort(header, reverseOrder);
            }
        });

        this.getContentPane().add(scroller);
        this.addWindowListener(new WindowListener() {
            public void windowDeactivated(WindowEvent e) {}
            public void windowActivated(WindowEvent e) {}
            public void windowDeiconified(WindowEvent e) {}
            public void windowIconified(WindowEvent e) {}
            public void windowClosed(WindowEvent e) {}
            public void windowClosing(WindowEvent e) {
                closeTable();
            }

            public void windowOpened(WindowEvent e) {}
        });

        clickCount = 0;

        this.setJMenuBar(mnuBar);
        mnuCommands.setText("Commands");
        mnuCommandsUndo.setText("Undo since last command");
        mnuCommandsUndo.addActionListener(new
                PlacFrame_mnuCommandsUndo_actionAdapter(this));
        mnuCommandsClearKeepInitial.setText("Clear (keep initial)");
        mnuCommandsClearKeepInitial.addActionListener(new
                PlacFrame_mnuCommandsClearKeepInitial_actionAdapter(this));
        mnuCommandsClearAll.setText("Clear all");
        mnuCommandsClearAll.addActionListener(new
                PlacFrame_mnuCommandsClearAll_actionAdapter(this));
        mnuCommandsAutoFill.setText("Auto fill");
        mnuCommandsAutoFill.addActionListener(new
                PlacFrame_mnuCommandsAutoFill_actionAdapter(this));
        mnuCommandsZeroRow.setText("Zero Row");
        mnuCommandsZeroRow.addActionListener(new
                PlacFrame_mnuCommandsZeroRow_actionAdapter(this));
        mnuBar.add(mnuCommands);
        mnuCommands.add(mnuCommandsUndo);
        mnuCommands.add(mnuCommandsClearKeepInitial);
        mnuCommands.add(mnuCommandsClearAll);
        mnuCommands.add(mnuCommandsZeroRow);
        mnuCommands.add(mnuCommandsAutoFill);

        redoTable = new PlacEntry[tableModel.getData().length];
        for (int i = 0; i < redoTable.length; i++) {
            redoTable[i] = new PlacEntry(model.getPlacChunk(levelIndex));
        }
        fillRedoTable();

        setSize(600, 500);

        StaticFunctions.centerFrame(this);
        setVisible(true);

    }

    private void closeTable() {
        model.removeChildFrame(this);
    }

    private void fillRedoTable() {
        try {
            PlacEntry[] data = tableModel.getData();
            for (int i = 0; i < data.length; i++) {
                redoTable[i].setFlags(data[i].getFlags());
                redoTable[i].setInitial(data[i].getInitial());
                redoTable[i].setMaximum(data[i].getMaximum());
                redoTable[i].setMinimum(data[i].getMinimum());
                redoTable[i].setRandomChance(data[i].getRandomChance());
                redoTable[i].setRandomChancePercent(data[i].
                        getRandomChancePercent());
                redoTable[i].setRandomCount(data[i].getRandomCount());
            }
        } catch (Exception e) {
            return;
        }
    }

    private void mnuCommandsUndo_actionPerformed(ActionEvent e) { // Undo menu
        try {
            PlacEntry[] data = tableModel.getData();
            for (int i = 0; i < data.length; i++) {
                data[i].setFlags(redoTable[i].getFlags());
                data[i].setInitial(redoTable[i].getInitial());
                data[i].setMaximum(redoTable[i].getMaximum());
                data[i].setMinimum(redoTable[i].getMinimum());
                data[i].setRandomChance(redoTable[i].getRandomChance());
                data[i].setRandomChancePercent(redoTable[i].
                                               getRandomChancePercent());
                data[i].setRandomCount(redoTable[i].getRandomCount());
            }
            tableModel.fireTableDataChanged();
        } catch (Exception ex) {
            return;
        }
    }

    public void mnuCommandsClearKeepInitial_actionPerformed(ActionEvent e) { // Clear keep initial menu
        try {
            fillRedoTable();
            PlacEntry[] data = tableModel.getData();
            for (int i = 0; i < data.length; i++) {
                data[i].setMaximum((short) 0);
                data[i].setMinimum((short) 0);
                data[i].setFlags((short) 0);
                data[i].setRandomChancePercent(0);
                data[i].setRandomCount(0);
            }
            tableModel.fireTableDataChanged();
        } catch (Exception ex) {
            return;
        }
    }

    public void mnuCommandsClearAll_actionPerformed(ActionEvent e) { // Clear all menu
        try {
            fillRedoTable();
            PlacEntry[] data = tableModel.getData();
            for (int i = 0; i < data.length; i++) {
                data[i].setInitial((short) 0);
                data[i].setMaximum((short) 0);
                data[i].setMinimum((short) 0);
                data[i].setFlags((short) 0);
                data[i].setRandomChancePercent(0);
                data[i].setRandomCount(0);
            }
            tableModel.fireTableDataChanged();
        } catch (Exception ex) {
            return;
        }
    }

    public void mnuCommandsAutoFill_actionPerformed(ActionEvent e) { // Auto fill menu
        try {
            fillRedoTable();
            PlacEntry[] data = tableModel.getData();
            int initVal = 0, appear = 0;
            double max = 0, min = 0;
            for (int i = 0; i < data.length; i++) {
                initVal = data[i].getInitial();
                max = (initVal * Preferences.MAX_MULT);
                min = (initVal * Preferences.MIN_MULT);
                appear = Preferences.APPEAR_PERCENT;
                if (max > 32767) max = 32767;
                if (max < 0) max = 0;
                if (min > 32767) min = 32767;
                if (min < 0) min = 0;
                if (appear > 100) appear = 100;
                if (appear < 0) appear = 0;
                data[i].setMaximum((short) max);
                data[i].setMinimum((short) min);
                if (data[i].getInitial() > 0)
                    data[i].setRandomChancePercent(appear);
            }
            tableModel.fireTableDataChanged();
        } catch (Exception ex) {
            return;
        }
    }

    public void mnuCommandsZeroRow_actionPerformed(ActionEvent e) { // Zero row menu
        try {
            fillRedoTable();
            int row = table.getSelectedRow();
            tableModel.getData()[row].setFlags((short) 0);
            tableModel.getData()[row].setInitial((short) 0);
            tableModel.getData()[row].setMinimum((short) 0);
            tableModel.getData()[row].setMaximum((short) 0);
            tableModel.getData()[row].setRandomChancePercent((short) 0);
            tableModel.getData()[row].setRandomCount((short) 0);
            tableModel.fireTableRowsUpdated(row, row);
        } catch (Exception ex) {
            return;
        }
    }

    class PlacFrame_mnuCommandsUndo_actionAdapter implements ActionListener {
        private PlacFrame adaptee;
        PlacFrame_mnuCommandsUndo_actionAdapter(PlacFrame adaptee) {
            this.adaptee = adaptee;
        }

        public void actionPerformed(ActionEvent e) {
            adaptee.mnuCommandsUndo_actionPerformed(e);
        }
    }


    class PlacFrame_mnuCommandsClearKeepInitial_actionAdapter implements
            ActionListener {
        private PlacFrame adaptee;
        PlacFrame_mnuCommandsClearKeepInitial_actionAdapter(PlacFrame adaptee) {
            this.adaptee = adaptee;
        }

        public void actionPerformed(ActionEvent e) {
            adaptee.mnuCommandsClearKeepInitial_actionPerformed(e);
        }
    }


    class PlacFrame_mnuCommandsClearAll_actionAdapter implements ActionListener {
        private PlacFrame adaptee;
        PlacFrame_mnuCommandsClearAll_actionAdapter(PlacFrame adaptee) {
            this.adaptee = adaptee;
        }

        public void actionPerformed(ActionEvent e) {
            adaptee.mnuCommandsClearAll_actionPerformed(e);
        }
    }


    class PlacFrame_mnuCommandsAutoFill_actionAdapter implements ActionListener {
        private PlacFrame adaptee;
        PlacFrame_mnuCommandsAutoFill_actionAdapter(PlacFrame adaptee) {
            this.adaptee = adaptee;
        }

        public void actionPerformed(ActionEvent e) {
            adaptee.mnuCommandsAutoFill_actionPerformed(e);
        }
    }


    class PlacFrame_mnuCommandsZeroRow_actionAdapter implements ActionListener {
        private PlacFrame adaptee;
        PlacFrame_mnuCommandsZeroRow_actionAdapter(PlacFrame adaptee) {
            this.adaptee = adaptee;
        }

        public void actionPerformed(ActionEvent e) {
            adaptee.mnuCommandsZeroRow_actionPerformed(e);
        }
    }


    class PlacTableModelListener implements TableModelListener {
        public void tableChanged(TableModelEvent evt) {
            if (evt.getType() == TableModelEvent.UPDATE) {
                int column = evt.getColumn();
                int row = evt.getFirstRow();
                //table.setColumnSelectionInterval(column + 1, column + 1);
                table.setRowSelectionInterval(row, row);
            }
        }
    }
}
