/*
  Copyright 2007 Darren Watts

  This file is part of JUICE.

  JUICE is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  JUICE is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with JUICE; if not, write to the Free Software
  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

package src.gui;

import javax.swing.table.AbstractTableModel;
import src.backend.StaticFunctions;
import src.backend.*;

public class PlacTableModel extends AbstractTableModel{
    public static final int PLAC_ITEM = 0;
    public static final int PLAC_INITIAL = 1;
    public static final int PLAC_MIN = 2;
    public static final int PLAC_MAX = 3;
    public static final int PLAC_RANDOM = 4;
    public static final int PLAC_RANDOM_NUM = 5;
    public static final int PLAC_INF_AVAIL = 6;
    public static final int PLAC_RAND_APPEAR = 7;

    public static final int INF_AMOUNT = 65535;

    private String[] colNames;
    private PlacEntry[] data;
    private String[] itemName;
    private Model model;

    public PlacTableModel(String[] colNames, Model model, int levelIndex,
                          int numItems, int startIndex, int endIndex) {
        this.colNames = colNames;
        this.model = model;
        data = new PlacEntry[numItems];
        itemName = new String[numItems];
        int position = 0;
        for (int i = startIndex; i < endIndex; i++) {
            itemName[position] = StaticFunctions.getPlacName(i);
            data[position++] = this.model.getPlacItem(levelIndex, i);
        }
    }

    public String getColumnName(int col){
        return colNames[col];
    }

    public boolean isCellEditable(int row, int column) {
        if (column == PLAC_ITEM) return false;
        return true;
    }

    public Object getValueAt(int row, int column) {
        switch (column) {
        case PLAC_ITEM:
            try{
                return itemName[row];
            }catch (Exception e) { return ""; }
        case PLAC_INITIAL:
            try{
                return new Integer(data[row].getInitial());
            }catch(Exception e) { return "0"; }
        case PLAC_MIN:
            try{
                return new Integer(data[row].getMinimum());
            }catch(Exception e) { return "0"; }
        case PLAC_MAX:
            try{
                return new Integer(data[row].getMaximum());
            }catch(Exception e) { return "0"; }
        case PLAC_RANDOM:
            try{
                if (data[row].getFlags() == 1)return new Boolean(true);
                else return new Boolean(false);
            }catch(Exception e) { return new Boolean(false); }
        case PLAC_RANDOM_NUM:
            try{
                if(data[row].getRandomCount() == INF_AMOUNT)
                    return "Infinite";
                return new Integer(data[row].getRandomCount());
            }catch(Exception e) { return "0"; }
        case PLAC_INF_AVAIL:
            try{
                if (data[row].getRandomCount() == INF_AMOUNT)return new Boolean(true);
                else return new Boolean(false);
            }catch(Exception e) { return new Boolean(false); }
        case PLAC_RAND_APPEAR:
            try{
                return new Integer(data[row].getRandomChancePercent());
            }catch(Exception e) { return "0"; }
        default:
            return new Object();
        }
   }

   public void setValueAt(Object value, int row, int column) {
       int intValue = 0;
       boolean boolValue = false;
       try{
           intValue = Integer.parseInt((String) value);
       } catch(Exception e) {}
       try {
           boolValue = ((Boolean) value).booleanValue();
       } catch (Exception ex) {}
       try{
           switch (column) {
           case PLAC_INITIAL:
               if (intValue > 256) data[row].setInitial((short) 256);
               else if (intValue < 0) data[row].setInitial((short) 0);
               else data[row].setInitial((short) intValue);
               break;
           case PLAC_MIN:
               if (intValue > 256) data[row].setMinimum((short) 256);
               else if (intValue < 0) data[row].setMinimum((short) 0);
               else data[row].setMinimum((short) intValue);
               break;
           case PLAC_MAX:
               if (intValue > 256) data[row].setMaximum((short) 256);
               else if (intValue < 0) data[row].setMaximum((short) 0);
               else data[row].setMaximum((short) intValue);
               break;
           case PLAC_RANDOM:
               if (boolValue) data[row].setFlags((short) 1);
               else data[row].setFlags((short) 0);
               break;
           case PLAC_RANDOM_NUM:
               if (intValue >= INF_AMOUNT) {
                   setValueAt(new Boolean(true), row, column + 1);
                   data[row].setRandomCount(INF_AMOUNT);
               } else if (intValue < 0) {
                   setValueAt(new Boolean(false), row, column + 1);
                   data[row].setRandomCount(0);
               } else {
                   setValueAt(new Boolean(false), row, column + 1);
                   data[row].setRandomCount(intValue);
               }
               fireTableCellUpdated(row, column + 1);
               break;
           case PLAC_INF_AVAIL:
               if (!boolValue)
                   data[row].setRandomCount(0);
               else data[row].setRandomCount(INF_AMOUNT);
               fireTableCellUpdated(row, column - 1);
               break;
           case PLAC_RAND_APPEAR:
               if (intValue > 100) data[row].setRandomChancePercent((short) 100);
               else if (intValue < 0) data[row].setRandomChancePercent((short)
                       0);
               else data[row].setRandomChancePercent((short) intValue);
               break;
           default:
               break;
           }
       }catch(Exception e) {}
       fireTableCellUpdated(row, column);
   }

   public int getRowCount() {
       return data.length;
   }

   public int getColumnCount(){
       return colNames.length;
   }

   public PlacEntry[] getData(){
       return data;
   }

   public void setData(PlacEntry[] data){
       try{
           this.data = data;
       } catch(Exception e) {}
   }

   public void sort(int header, boolean reverse){
       try{
           switch (header) {
           case PLAC_ITEM:
               sortBy(reverse, "item");
               break;
           case PLAC_INITIAL:
               sortBy(reverse, "initial");
               break;
           case PLAC_MIN:
               sortBy(reverse, "min");
               break;
           case PLAC_MAX:
               sortBy(reverse, "max");
               break;
           case PLAC_RANDOM_NUM:
               sortBy(reverse, "random num");
               break;
           case PLAC_RAND_APPEAR:
               sortBy(reverse, "random appear");
               break;
           }
           this.fireTableDataChanged();
       } catch(Exception e){}
   }

   private void sortBy(boolean reverse, String type){
       mergeSort(0, itemName.length, reverse, type);
   }

   private void mergeSort(int first, int n, boolean reverse, String type){
       int n1; // Size of the first half of the array
       int n2; // Size of the second half of the array

       if (n > 1) {
           // Compute sizes of the two halves
           n1 = n / 2;
           n2 = n - n1;

           mergeSort(first, n1, reverse, type); // Sort data[first] through data[first+n1-1]
           mergeSort(first + n1, n2, reverse, type); // Sort data[first+n1] to the end

           // Merge the two sorted halves.
           merge(first, n1, n2, reverse, type);
       }
   }

   private void merge(int first, int n1, int n2, boolean reverse, String type) {
       //int[] temp = new int[n1 + n2]; // Allocate the temporary array
       PlacEntry[] tempEntry = new PlacEntry[n1 + n2];
       String[] tempStr = new String[n1 + n2];
       int copied = 0; // Number of elements copied from data to temp
       int copied1 = 0; // Number copied from the first half of data
       int copied2 = 0; // Number copied from the second half of data
       int i; // Array index to copy from temp back into data

       // Merge elements, copying from two halves of data to the temporary array.
       while ((copied1 < n1) &&
              (copied2 < n2)) {
           if (compare(reverse, type, first, copied1, copied2, n1)) {
               tempStr[copied++] = itemName[first + (copied1++)];
               tempEntry[copied - 1] = data[first + copied1 - 1];
           } else {
               tempStr[copied++] = itemName[first + n1 + (copied2++)];
               tempEntry[copied - 1] = data[first + n1 + copied2 - 1];
           }
       }

       // Copy any remaining entries in the left and right subarrays.
       while (copied1 < n1) {
           tempStr[copied++] = itemName[first + (copied1++)];
           tempEntry[copied - 1] = data[first + copied1 - 1];
       } while (copied2 < n2) {
           tempStr[copied++] = itemName[first + n1 + (copied2++)];
           tempEntry[copied - 1] = data[first + n1 + copied2 - 1];
       }
       // Copy from temp back to the data array.
       for (i = 0; i < n1 + n2; i++) {
           itemName[first + i] = tempStr[i];
           data[first + i] = tempEntry[i];
       }
   }

   private boolean compare(boolean reverse, String type, int first,
                           int copied1, int copied2, int n1) {
       if (!reverse && type.equals("item")) {
           return (itemName[first + copied1].compareTo(itemName[first + n1 +
                   copied2]) < 0);
       }
       else if (reverse && type.equals("item")) {
           return (itemName[first + copied1].compareTo(itemName[first + n1 +
                   copied2]) > 0);
       }
       else if (!reverse && type.equals("initial")) {
           return data[first + copied1].getInitial() <
                   data[first + n1 + copied2].getInitial();
       }
       else if (reverse && type.equals("initial")) {
           return data[first + copied1].getInitial() >
                   data[first + n1 + copied2].getInitial();
       }
       else if (!reverse && type.equals("min")) {
           return data[first + copied1].getMinimum() <
                   data[first + n1 + copied2].getMinimum();
       }
       else if (reverse && type.equals("min")) {
           return data[first + copied1].getMinimum() >
                   data[first + n1 + copied2].getMinimum();
       }
       else if (!reverse && type.equals("max")) {
           return data[first + copied1].getMaximum() <
                   data[first + n1 + copied2].getMaximum();
       }
       else if (reverse && type.equals("max")) {
           return data[first + copied1].getMaximum() >
                   data[first + n1 + copied2].getMaximum();
       }
       else if (!reverse && type.equals("random num")) {
           return data[first + copied1].getRandomCount() <
                   data[first + n1 + copied2].getRandomCount();
       }
       else if (reverse && type.equals("random num")) {
           return data[first + copied1].getRandomCount() >
                   data[first + n1 + copied2].getRandomCount();
       }
       else if (!reverse && type.equals("random appear")) {
           return data[first + copied1].getRandomChancePercent() <
                   data[first + n1 + copied2].getRandomChancePercent();
       }
       else if (reverse && type.equals("random appear")) {
           return data[first + copied1].getRandomChancePercent() >
                   data[first + n1 + copied2].getRandomChancePercent();
       }
       else return false;
   }

}
