/*
  Copyright 2007 Darren Watts

  This file is part of JUICE.

  JUICE is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  JUICE is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with JUICE; if not, write to the Free Software
  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */

package src.gui;

import java.awt.*;
import javax.swing.*;
import javax.swing.table.TableColumn;
import javax.swing.event.TableModelListener;
import javax.swing.event.TableModelEvent;
import src.backend.*;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.awt.event.WindowEvent;
import java.awt.event.WindowListener;

public class PolyFrame extends JFrame {
    private String[] columnNames = {
                                   "Index", "Type", "Permutation"
    };
    /*private String[] columnNames = {
                                   "Index", "Type", "Permutation",
                                   "Floor Height",
                                   "Ceiling Height", "Media"
         };*/
    private Model model;
    private int levelIndex;

    private JTable table;
    private JScrollPane scroller;
    private PolyTableModel tableModel;

    private int clickCount;


    public PolyFrame(Model model, int levelIndex) {
        this.model = model;
        this.levelIndex = levelIndex;
        try {
            jbInit();
        } catch (Exception exception) {
            exception.printStackTrace();
        }
    }

    private void jbInit() throws Exception {
        getContentPane().setLayout(new BorderLayout());
        tableModel = new PolyTableModel(columnNames, model, levelIndex);
        tableModel.addTableModelListener(new PolyFrame.PolyTableModelListener());
        table = new JTable(tableModel);
        table.setSurrendersFocusOnKeystroke(true);
        table.getTableHeader().setReorderingAllowed(false);

        TableColumn sportColumn = table.getColumnModel().getColumn(1);
        JComboBox comboBox = new JComboBox();
        for (int i = 0; i < StaticFunctions.polygonTypeNames.length; i++) {
            comboBox.addItem(StaticFunctions.polygonTypeNames[i]);
        }
        sportColumn.setCellEditor(new DefaultCellEditor(comboBox));

        scroller = new JScrollPane(table);

        table.getTableHeader().addMouseListener(new MouseListener() {
            public void mouseExited(MouseEvent m) {}

            public void mouseEntered(MouseEvent m) {}

            public void mouseReleased(MouseEvent m) {}

            public void mousePressed(MouseEvent m) {}

            public void mouseClicked(MouseEvent m) {
                Point mousePoint = new Point(m.getX(), m.getY());
                int header = table.getTableHeader().columnAtPoint(mousePoint);

                clickCount++;
                boolean reverseOrder = false;
                if ((clickCount % 2) == 0) reverseOrder = true;
                tableModel.sort(header, reverseOrder);
            }
        });

        this.getContentPane().add(scroller);
        this.addWindowListener(new WindowListener() {
            public void windowDeactivated(WindowEvent e) {}

            public void windowActivated(WindowEvent e) {}

            public void windowDeiconified(WindowEvent e) {}

            public void windowIconified(WindowEvent e) {}

            public void windowClosed(WindowEvent e) {}

            public void windowClosing(WindowEvent e) {
                closeTable();
            }

            public void windowOpened(WindowEvent e) {}
        });

        clickCount = 0;

        setSize(525, 440);
        setTitle("Polygons - " +
                 model.getLevelSelectName());
        StaticFunctions.centerFrame(this);
        //setResizable(false);
        setVisible(true);

    }

    private void closeTable() {
        model.removeChildFrame(this);
    }

    class PolyTableModelListener implements TableModelListener {
        public void tableChanged(TableModelEvent evt) {
            if (evt.getType() == TableModelEvent.UPDATE) {
                int column = evt.getColumn();
                int row = evt.getFirstRow();
                try {
                    table.setRowSelectionInterval(row, row);
                } catch (Exception e) {}
            }
        }
    }
}
