/*
  Copyright 2007 Darren Watts

  This file is part of JUICE.

  JUICE is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  JUICE is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with JUICE; if not, write to the Free Software
  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

package src.gui;

import javax.swing.table.AbstractTableModel;
import src.backend.PolyEntry;
import src.backend.StaticFunctions;

public class PolyTableModel extends AbstractTableModel{
    public static final int POLY_INDEX = 0;
    public static final int POLY_TYPE = 1;
    public static final int POLY_PERM = 2;
    public static final int POLY_FH = 3;
    public static final int POLY_CH = 4;
    public static final int POLY_MEDIA = 5;

    private String[] colNames;
    private PolyEntry[] data;
    private int[] indexNums;
    private Model model;

    public PolyTableModel(String[] colNames, Model model, int levelIndex) {
        this.colNames = colNames;
        this.model = model;
        data = new PolyEntry[this.model.getPolyNumber(levelIndex)];
        indexNums = new int[this.model.getPolyNumber(levelIndex)];
        for(int i = 0; i < this.model.getPolyNumber(levelIndex); i++){
            indexNums[i] = i;
            data[i] = this.model.getPoly(levelIndex, i);
        }
    }

    public String getColumnName(int col){
        return colNames[col];
    }

    public boolean isCellEditable(int row, int column) {
        if (column == POLY_INDEX) return false;
        /*if(column == POLY_PERM){
            if (getValueAt(row, column - 1) == "Platform")
                return false;
        }*/
        return true;
    }

    public Object getValueAt(int row, int column) {
        //PolyEntry polygon = data[row];
        switch (column) {
        case POLY_INDEX:
            return new Integer(indexNums[row]);
        case POLY_TYPE:
            return getTypeString(data[row].getType());
        case POLY_PERM:
            return new Integer(data[row].getPermutation());
        /*case POLY_FH:
            return polygon.getFloorHeight();
        case POLY_CH:
            return polygon.getCeilingHeight();
        case POLY_MEDIA:
            return polygon.getMedia();*/
        default:
            return new Object();
        }
   }

   private String getTypeString(int num){
       return StaticFunctions.polygonTypeNames[num];
   }

   private Integer getTypeNum(String str){
       for(int i = 0; i < StaticFunctions.polygonTypeNames.length; i++){
           if(str.equals(StaticFunctions.polygonTypeNames[i]))
               return new Integer(i);
       }
       return new Integer(-1);
   }

   public void setValueAt(Object value, int row, int column) {
       //PolyEntry polygon = data[row];
       Integer tempValue;
       switch (column) {
       case POLY_TYPE:
           tempValue = getTypeNum((String)value);
           data[row].setType(tempValue.shortValue());
           break;
       case POLY_PERM:
           tempValue = Integer.valueOf(((String)value));
           data[row].setPermutation(tempValue.shortValue());
           break;
       /*case POLY_FH:
           tempValue = Integer.valueOf(((String)value));
           polygon.setFloorHeight(tempValue.shortValue());
           break;
       case POLY_CH:
           tempValue = Integer.valueOf(((String)value));
           polygon.setCeilingHeight(tempValue.shortValue());
           break;
       case POLY_MEDIA:
           tempValue = Integer.valueOf(((String)value));
           polygon.setMedia(tempValue.shortValue());
           break;*/
       default:
           System.out.println("Invalid PolyTable index");
       }
       fireTableCellUpdated(row, column);
   }

   public int getRowCount() {
       return data.length;
   }

   public int getColumnCount(){
       return colNames.length;
   }

   public void sort(int header, boolean reverse){
       switch (header) {
       case POLY_INDEX:
           sortBy(reverse, "index");
           break;
       case POLY_TYPE:
           sortBy(reverse, "type");
           break;
       case POLY_PERM:
           sortBy(reverse, "perm");
           break;
       }
       this.fireTableDataChanged();
   }

   private void sortBy(boolean reverse, String type){
       mergeSort(0, indexNums.length, reverse, type);
   }

   private void mergeSort(int first, int n, boolean reverse, String type){
       int n1; // Size of the first half of the array
       int n2; // Size of the second half of the array

       if (n > 1) {
           // Compute sizes of the two halves
           n1 = n / 2;
           n2 = n - n1;

           mergeSort(first, n1, reverse, type); // Sort data[first] through data[first+n1-1]
           mergeSort(first + n1, n2, reverse, type); // Sort data[first+n1] to the end

           // Merge the two sorted halves.
           merge(first, n1, n2, reverse, type);
       }
   }

   private void merge(int first, int n1, int n2, boolean reverse, String type) {
       int[] temp = new int[n1 + n2]; // Allocate the temporary array
       PolyEntry[] tempEntry = new PolyEntry[n1 + n2];
       int copied = 0; // Number of elements copied from data to temp
       int copied1 = 0; // Number copied from the first half of data
       int copied2 = 0; // Number copied from the second half of data
       int i; // Array index to copy from temp back into data

       // Merge elements, copying from two halves of data to the temporary array.
       while ((copied1 < n1) &&
              (copied2 < n2)) {
           if (compare(reverse, type, first, copied1, copied2, n1)) {
               temp[copied++] = indexNums[first + (copied1++)];
               tempEntry[copied - 1] = data[first + copied1 - 1];
           } else {
               temp[copied++] = indexNums[first + n1 + (copied2++)];
               tempEntry[copied - 1] = data[first + n1 + copied2 - 1];
           }
       }

       // Copy any remaining entries in the left and right subarrays.
       while (copied1 < n1) {
           temp[copied++] = indexNums[first + (copied1++)];
           tempEntry[copied - 1] = data[first + copied1 - 1];
       } while (copied2 < n2) {
           temp[copied++] = indexNums[first + n1 + (copied2++)];
           tempEntry[copied - 1] = data[first + n1 + copied2 - 1];
       }
       // Copy from temp back to the data array.
       for (i = 0; i < n1 + n2; i++) {
           indexNums[first + i] = temp[i];
           data[first + i] = tempEntry[i];
       }
   }

   private boolean compare(boolean reverse, String type, int first,
                           int copied1, int copied2, int n1) {
       if (!reverse && type.equals("index")) {
           return (indexNums[first + copied1] < indexNums[first + n1 + copied2]);
       }
       else if (reverse && type.equals("index")) {
           return (indexNums[first + copied1] > indexNums[first + n1 + copied2]);
       }
       else if (!reverse && type.equals("type")) {
           return data[first + copied1].getType() <
                   data[first + n1 + copied2].getType();
       }
       else if (reverse && type.equals("type")){
           return data[first + copied1].getType() >
                   data[first + n1 + copied2].getType();
       }
       else if (!reverse && type.equals("perm")) {
           return data[first + copied1].getPermutation() <
                   data[first + n1 + copied2].getPermutation();
       }
       else if (reverse && type.equals("perm")){
           return data[first + copied1].getPermutation() >
                   data[first + n1 + copied2].getPermutation();
       }
       else return false;
   }

}
