/*
  Copyright 2007 Darren Watts

  This file is part of JUICE.

  JUICE is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  JUICE is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with JUICE; if not, write to the Free Software
  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

package src.gui;

import java.awt.*;
import javax.swing.*;
import src.backend.SidsEntry;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import src.backend.*;

public class SidsFlagsFrame extends JFrame {
    private final int CP_STATUS = 0x0001;
    private final int IS_CP = 0x0002;
    private final int REPAIR = 0x0004;
    private final int DESTRUCTIVE = 0x0008;
    private final int LIGHT = 0x0010;
    private final int CAN_BE_DESTROYED = 0x0020;
    private final int ONLY_PROJECTILES = 0x0040;

    private Model model;
    private int levelIndex;
    private SidsEntry entry;
    JCheckBox chkIsCP = new JCheckBox();
    JCheckBox chkIsRepair = new JCheckBox();
    JCheckBox chkIsDestructive = new JCheckBox();
    JCheckBox chkIsLightSwitch = new JCheckBox();
    JCheckBox chkCanBeDestroyed = new JCheckBox();
    JCheckBox chkOnlyProjectiles = new JCheckBox();
    JCheckBox chkCPStatus = new JCheckBox();
    JButton btnOK = new JButton();
    JButton btnCancel = new JButton();

    public SidsFlagsFrame(Model model, int levelIndex, SidsEntry entry) {
        try {
            this.model = model;
            this.levelIndex = levelIndex;
            this.entry = entry;
            jbInit();
        } catch (Exception exception) {
            exception.printStackTrace();
        }
    }

    private void jbInit() throws Exception {
        getContentPane().setLayout(null);
        chkIsCP.setText("Is Control Panel");
        chkIsCP.setBounds(new Rectangle(9, 28, 164, 19));
        chkIsDestructive.setText("Is Destructive Switch");
        chkIsDestructive.setBounds(new Rectangle(9, 66, 212, 19));
        chkIsLightSwitch.setText("Is Light Switch");
        chkIsLightSwitch.setBounds(new Rectangle(9, 85, 176, 19));
        chkCanBeDestroyed.setText("Can be destroyed");
        chkCanBeDestroyed.setBounds(new Rectangle(9, 104, 187, 19));
        chkOnlyProjectiles.setText("Toggled by Projectile");
        chkOnlyProjectiles.setBounds(new Rectangle(9, 123, 219, 19));
        chkCPStatus.setText("Control Panel Status");
        chkCPStatus.setBounds(new Rectangle(9, 9, 158, 19));
        btnOK.setBounds(new Rectangle(110, 151, 78, 31));
        btnOK.setText("OK");
        btnOK.addActionListener(new SidsFlagsFrame_btnOK_actionAdapter(this));
        btnCancel.setBounds(new Rectangle(26, 151, 78, 31));
        btnCancel.setText("Cancel");
        btnCancel.addActionListener(new SidsFlagsFrame_btnCancel_actionAdapter(this));
        this.getContentPane().add(chkCPStatus);
        this.getContentPane().add(chkIsCP);
        this.getContentPane().add(chkIsRepair);
        this.getContentPane().add(chkIsDestructive);
        this.getContentPane().add(chkIsLightSwitch);
        this.getContentPane().add(chkCanBeDestroyed);
        this.getContentPane().add(chkOnlyProjectiles);
        this.getContentPane().add(btnCancel);
        this.getContentPane().add(btnOK);
        chkIsRepair.setText("Is Repair Switch");
        chkIsRepair.setBounds(new Rectangle(9, 47, 207, 19));

        loadInfo();

        setSize(230, 220);
        setTitle("Side Flags");
        StaticFunctions.centerFrame(this);
        setResizable(false);
        setVisible(true);
    }

    private void loadInfo(){
        int flags = entry.getFlags();
        if((flags & CP_STATUS) > 0) chkCPStatus.setSelected(true);
        if((flags & IS_CP) > 0) chkIsCP.setSelected(true);
        if((flags & REPAIR) > 0) chkIsRepair.setSelected(true);
        if((flags & DESTRUCTIVE) > 0) chkIsDestructive.setSelected(true);
        if((flags & LIGHT) > 0) chkIsLightSwitch.setSelected(true);
        if((flags & CAN_BE_DESTROYED) > 0) chkCanBeDestroyed.setSelected(true);
        if((flags & ONLY_PROJECTILES) > 0) chkOnlyProjectiles.setSelected(true);
    }

    public void btnCancel_actionPerformed(ActionEvent e) {
        model.removeChildFrame(this);
        dispose();
    }

    public void btnOK_actionPerformed(ActionEvent e) {
        short flags = 0;
        if(chkCPStatus.isSelected()) flags += CP_STATUS;
        if(chkIsCP.isSelected()) flags += IS_CP;
        if(chkIsRepair.isSelected()) flags += REPAIR;
        if(chkIsDestructive.isSelected()) flags += DESTRUCTIVE;
        if(chkIsLightSwitch.isSelected()) flags += LIGHT;
        if(chkCanBeDestroyed.isSelected()) flags += CAN_BE_DESTROYED;
        if(chkOnlyProjectiles.isSelected()) flags += ONLY_PROJECTILES;
        entry.setFlags(flags);

        model.removeChildFrame(this);
        dispose();
    }
}


class SidsFlagsFrame_btnOK_actionAdapter implements ActionListener {
    private SidsFlagsFrame adaptee;
    SidsFlagsFrame_btnOK_actionAdapter(SidsFlagsFrame adaptee) {
        this.adaptee = adaptee;
    }

    public void actionPerformed(ActionEvent e) {
        adaptee.btnOK_actionPerformed(e);
    }
}


class SidsFlagsFrame_btnCancel_actionAdapter implements ActionListener {
    private SidsFlagsFrame adaptee;
    SidsFlagsFrame_btnCancel_actionAdapter(SidsFlagsFrame adaptee) {
        this.adaptee = adaptee;
    }

    public void actionPerformed(ActionEvent e) {
        adaptee.btnCancel_actionPerformed(e);
    }
}
