/*
  Copyright 2007 Darren Watts

  This file is part of JUICE.

  JUICE is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  JUICE is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with JUICE; if not, write to the Free Software
  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

package src.gui;

import java.awt.*;
import javax.swing.*;
import javax.swing.table.TableColumn;
import javax.swing.table.TableCellRenderer;
import src.backend.StaticFunctions;
import java.awt.event.MouseListener;
import java.awt.event.WindowEvent;
import java.awt.event.MouseEvent;
import java.awt.event.WindowListener;
import javax.swing.event.TableModelListener;
import javax.swing.event.TableModelEvent;
import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;
import java.awt.event.KeyEvent;

public class SidsFrame extends JFrame {
    private String[] columnNames = {
                                   "Index", "Type", "Line Index"};

    private Model model;
    private int levelIndex;

    private JTable table;
    private JScrollPane scroller;
    private SidsTableModel tableModel;

    private int clickCount;
    JMenuBar jMenuBar1 = new JMenuBar();
    JMenu jMenu1 = new JMenu();
    JMenuItem jMenuItem1 = new JMenuItem();
    JMenuItem jMenuItem2 = new JMenuItem();

    public SidsFrame(Model model, int levelIndex) {
        try {
            this.model = model;
            this.levelIndex = levelIndex;
            jbInit();
        } catch (Exception exception) {
            exception.printStackTrace();
        }
    }

    private void jbInit() throws Exception {
        getContentPane().setLayout(new BorderLayout());
        tableModel = new SidsTableModel(columnNames, model, levelIndex);
        tableModel.addTableModelListener(new SidsFrame.SidsTableModelListener());
        table = new JTable(tableModel);
        table.setSurrendersFocusOnKeystroke(true);
        table.getTableHeader().setReorderingAllowed(false);

        TableColumn typeCol = table.getColumnModel().getColumn(1);
        JComboBox cmbType = new JComboBox();
        cmbType.addItem("Full");
        cmbType.addItem("High");
        cmbType.addItem("Low");
        cmbType.addItem("Composite");
        cmbType.addItem("Split");
        typeCol.setCellEditor(new DefaultCellEditor(cmbType));

        scroller = new JScrollPane(table);

        TableColumn lineIndexCol = table.getColumnModel().getColumn(2);

        table.getTableHeader().addMouseListener(new MouseListener() {
            public void mouseExited(MouseEvent m) {}
            public void mouseEntered(MouseEvent m) {}
            public void mouseReleased(MouseEvent m) {}
            public void mousePressed(MouseEvent m) {}
            public void mouseClicked(MouseEvent m) {
                Point mousePoint = new Point(m.getX(), m.getY());
                int header = table.getTableHeader().columnAtPoint(mousePoint);

                clickCount++;
                boolean reverseOrder = false;
                if ((clickCount % 2) == 0) reverseOrder = true;
                tableModel.sort(header, reverseOrder);
            }
        });

        this.getContentPane().add(scroller);
        this.addWindowListener(new WindowListener() {
            public void windowDeactivated(WindowEvent e) {}
            public void windowActivated(WindowEvent e) {}
            public void windowDeiconified(WindowEvent e) {}
            public void windowIconified(WindowEvent e) {}
            public void windowClosed(WindowEvent e) {}
            public void windowClosing(WindowEvent e) {
                closeTable();
            }

            public void windowOpened(WindowEvent e) {}
        });

        lineIndexCol.setCellRenderer(new TableCellRenderer() {
            public Component getTableCellRendererComponent(JTable table,
                    Object value,
                    boolean isSelected,
                    boolean hasFocus, int row,
                    int column) {

                JTextArea text = new JTextArea();
                text.setBackground(Color.LIGHT_GRAY);

                try {
                    text.setText(Integer.toString(((Integer)value).intValue()));
                } catch (Exception e) {}
                return text;
            }
        });

        clickCount = 0;

        jMenu1.setText("Commands");
        jMenuItem1.setText("Properties (ctrl-p)");
        jMenuItem1.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                jMenuItem1_actionPerformed(e);
            }
        });
        jMenuItem2.setText("Flags (ctrl-f)");
        jMenuItem2.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                jMenuItem2_actionPerformed(e);
            }
        });
        jMenuBar1.add(jMenu1);
        jMenu1.add(jMenuItem1);
        jMenu1.add(jMenuItem2);
        this.setJMenuBar(jMenuBar1);

        table.addKeyListener(new java.awt.event.KeyAdapter() { // SidsPropertiesFrame
            public void keyPressed(KeyEvent e) {
                if(e.getKeyCode() == KeyEvent.VK_P && e.isControlDown()){
                    try{
                        SidsPropertiesFrame s = new SidsPropertiesFrame(model,
                                levelIndex,
                                tableModel.getData()[table.getSelectedRow()]);
                        model.addChildFrame(s);
                    } catch(Exception ex){}
                }
            }
        });

        table.addKeyListener(new java.awt.event.KeyAdapter() { // SidsFlagsFrame
            public void keyPressed(KeyEvent e) {
                if(e.getKeyCode() == KeyEvent.VK_F && e.isControlDown()){
                    try{
                        SidsFlagsFrame s = new SidsFlagsFrame(model, levelIndex,
                                tableModel.getData()[table.getSelectedRow()]);
                        model.addChildFrame(s);
                    } catch(Exception ex){}
                }
            }
        });

        setSize(600, 500);
        setTitle("Sides - " +
                   model.getLevelSelectName());
        StaticFunctions.centerFrame(this);
        setVisible(true);
    }

    private void closeTable() {
        model.removeChildFrame(this);
    }

    public void jMenuItem1_actionPerformed(ActionEvent e) { // Properties menu
        try{
            SidsPropertiesFrame s = new SidsPropertiesFrame(model, levelIndex,
                    tableModel.getData()[table.getSelectedRow()]);
            model.addChildFrame(s);
        } catch(Exception ex){
            return;
        }
    }

    public void jMenuItem2_actionPerformed(ActionEvent e) { // Flags menu
        try{
            SidsFlagsFrame s = new SidsFlagsFrame(model, levelIndex,
                                                  tableModel.getData()[table.
                                                  getSelectedRow()]);
            model.addChildFrame(s);
        } catch(Exception ex){
            return;
        }
    }

    class SidsTableModelListener implements TableModelListener {
        public void tableChanged(TableModelEvent evt) {
            if (evt.getType() == TableModelEvent.UPDATE) {
                int column = evt.getColumn();
                int row = evt.getFirstRow();
                try {
                    table.setRowSelectionInterval(row, row);
                } catch (Exception e) {}
            }
        }
    }
}
