/*
  Copyright 2007 Darren Watts

  This file is part of JUICE.

  JUICE is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  JUICE is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with JUICE; if not, write to the Free Software
  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

package src.gui;
import src.backend.*;
import javax.swing.table.AbstractTableModel;

public class SidsTableModel extends AbstractTableModel{
    public static final int SIDS_INDEX = 0;
    public static final int SIDS_TYPE = 1;
    public static final int SIDS_LINEINDEX = 2;

    private String[] colNames;
    private SidsEntry[] data;
    private int[] indexNums;
    private Model model;

    public SidsTableModel(String[] colNames, Model model, int levelIndex) {
        this.colNames = colNames;
        this.model = model;
        data = new SidsEntry[this.model.getSidsNumber(levelIndex)];
        indexNums = new int[this.model.getSidsNumber(levelIndex)];

        for(int i = 0; i < this.model.getSidsNumber(levelIndex); i++){
            indexNums[i] = i;
            data[i] = this.model.getSide(levelIndex, i);
        }
    }

    public String getColumnName(int col){
        return colNames[col];
    }

    public boolean isCellEditable(int row, int column) {
        if(column == SIDS_TYPE) return true;
        else return false;
    }

    private String getType(int num){
        if(num == 0) return "Full";
        else if(num == 1) return "High";
        else if(num == 2) return "Low";
        else if(num == 3) return "Composite";
        else return "Split";
    }

    private short getTypeNum(String str){
        if(str.equals("Full")) return 0;
        else if(str.equals("High")) return 1;
        else if(str.equals("Low")) return 2;
        else if(str.equals("Composite")) return 3;
        else return 4;
    }

    public Object getValueAt(int row, int column) {
        Integer num;
        switch (column) {
        case SIDS_INDEX:
            return new Integer(indexNums[row]);
        case SIDS_TYPE:
            return getType(data[row].getType());
        case SIDS_LINEINDEX:
            return new Integer(data[row].getLineIndex());
        default:
            return new Object();
        }
   }

   public void setValueAt(Object value, int row, int column) {
       short tempValue;

       switch (column) {
       case SIDS_TYPE:
           data[row].setType(getTypeNum((String)value));
           break;
       default:
           System.out.println("Invalid LinsTable index");
       }
       fireTableCellUpdated(row, column);
   }

   public int getRowCount() {
       return data.length;
   }

   public int getColumnCount(){
       return colNames.length;
   }

   public void sort(int header, boolean reverse){
       try{
           switch (header) {
           case SIDS_INDEX:
               sortBy(reverse, "index");
               break;
           case SIDS_TYPE:
               sortBy(reverse, "type");
               break;
           case SIDS_LINEINDEX:
               sortBy(reverse, "lineindex");
               break;
           }
           this.fireTableDataChanged();
       } catch(Exception e){}
   }

   private void sortBy(boolean reverse, String type){
       mergeSort(0, data.length, reverse, type);
   }

   private void mergeSort(int first, int n, boolean reverse, String type){
       int n1; // Size of the first half of the array
       int n2; // Size of the second half of the array

       if (n > 1) {
           // Compute sizes of the two halves
           n1 = n / 2;
           n2 = n - n1;

           mergeSort(first, n1, reverse, type); // Sort data[first] through data[first+n1-1]
           mergeSort(first + n1, n2, reverse, type); // Sort data[first+n1] to the end

           // Merge the two sorted halves.
           merge(first, n1, n2, reverse, type);
       }
   }

   private void merge(int first, int n1, int n2, boolean reverse, String type) {
       int[] temp = new int[n1 + n2]; // Allocate the temporary array
       SidsEntry[] tempEntry = new SidsEntry[n1 + n2];
       int copied = 0; // Number of elements copied from data to temp
       int copied1 = 0; // Number copied from the first half of data
       int copied2 = 0; // Number copied from the second half of data
       int i; // Array index to copy from temp back into data

       // Merge elements, copying from two halves of data to the temporary array.
       while ((copied1 < n1) &&
              (copied2 < n2)) {
           if (compare(reverse, type, first, copied1, copied2, n1)) {
               temp[copied++] = indexNums[first + (copied1++)];
               tempEntry[copied - 1] = data[first + copied1 - 1];
           } else {
               temp[copied++] = indexNums[first + n1 + (copied2++)];
               tempEntry[copied - 1] = data[first + n1 + copied2 - 1];
           }
       }

       // Copy any remaining entries in the left and right subarrays.
       while (copied1 < n1) {
           temp[copied++] = indexNums[first + (copied1++)];
           tempEntry[copied - 1] = data[first + copied1 - 1];
       } while (copied2 < n2) {
           temp[copied++] = indexNums[first + n1 + (copied2++)];
           tempEntry[copied - 1] = data[first + n1 + copied2 - 1];
       }
       // Copy from temp back to the data array.
       for (i = 0; i < n1 + n2; i++) {
           indexNums[first + i] = temp[i];
           data[first + i] = tempEntry[i];
       }
   }

   private boolean compare(boolean reverse, String type, int first,
                           int copied1, int copied2, int n1) {
       if (!reverse && type.equals("index")) {
           return (indexNums[first + copied1] < indexNums[first + n1 + copied2]);
       }
       else if (reverse && type.equals("index")) {
           return (indexNums[first + copied1] > indexNums[first + n1 + copied2]);
       }
       else if (!reverse && type.equals("type")) {
           return (data[first + copied1].getType() < data[first + n1 + copied2].getType());
       }
       else if (reverse && type.equals("type")){
           return (data[first + copied1].getType() > data[first + n1 + copied2].getType());
       }
       else if (!reverse && type.equals("lineindex")) {
           return (data[first + copied1].getLineIndex() < data[first + n1 + copied2].getLineIndex());
       }
       else if (reverse && type.equals("lineindex")){
           return (data[first + copied1].getLineIndex() > data[first + n1 + copied2].getLineIndex());
       }
       else return false;
   }

   public SidsEntry[] getData(){
       return data;
   }
}
